require 'active_support/concern'

module Metasploit::Pro
  module Engine
    module Arguments
      extend ActiveSupport::Concern
  
      module ClassMethods
        def parse_arguments(arguments=ARGV)
          parser_arguments.parse(arguments) do |option, index, value|
            case option
              when '-a'
                configuration.flash_policy_server.host = value
                configuration.server.host = value
              when '-c'
                configuration.console_mode = true
              when '-S'
                configuration.ssl = false
              when '-p'
                configuration.server.port = value.to_i
              when '-T'
                configuration.token = value
              when '-E'
                configuration.environment = value
              when '-u'
                configuration.uri = value
              when '-l'
                configuration.log_file = value
              when '-q'
                configuration.quiet = true
              when '-x'
                configuration.flash_policy_server.port = value.to_i
              when '-X'
                configuration.flash_policy_server.run = false
              when '-D'
                configuration.database_debug = true
              when '-h'
                print("\nUsage: prosvc <options>\n" +	parser_arguments.usage)
                exit
            end
          end
        end
  
        def parser_arguments
          @parser_arguments ||= Rex::Parser::Arguments.new(
              '-a' => [
                  true,
                  'Bind to this IP address'
              ],
              '-p' => [
                  true,
                  'Bind to this port instead of 50505'
              ],
              '-T' => [
                  true,
                  'Specify the token key to access msfrpcd'
              ],
              '-E' => [
                  true,
                  'Specify the database environment'
              ],
              '-t' => [
                  true,
                  'Server type, [Basic|Web]'
              ],
              '-u' => [
                  true,
                  'URI for Web server'
              ],
              '-S' => [
                  false,
                  'Disable SSL on the XMLRPC socket'
              ],
              '-x' => [
                  false,
                  'Flash policy server port (default 843)'
              ],
              '-X' => [
                  false,
                  'Disable the Flash policy server.'
              ],
              '-q' => [
                  false,
                  'Run in quiet mode (no XMLRPC logging)'
              ],
              '-l' => [
                  true,
                  'File name for the XMLRPC log'
              ],
              '-c' => [
                  false,
                  'Spawn an interactive console'
              ],
              '-D' => [
                  false,
                  'Enable debug mode logging of DB operations'
              ],
              '-h' => [
                  false,
                  'Help banner'
              ]
          )
        end
      end
    end
  end
end