#
# Gems
#
require 'active_support/ordered_options'
require 'active_support/string_inquirer'

#
# Project
#
require 'metasploit/pro'

module Metasploit::Pro::Engine
  class Configuration < Metasploit::Configuration
    attr_accessor :console_mode

    attr_accessor :database_debug

    def database_debug?
      database_debug == true
    end

    attr_reader :environment

    def environment=(environment)
      @environment = ActiveSupport::StringInquirer.new(environment)
    end

    attr_reader :flash_policy_server

    def initialize
      @console_mode = false
      @database_debug = false
      @dependencies = [
          Metasploit::Framework,
          Metasploit::Pro::UI
      ]
      @environment = ActiveSupport::StringInquirer.new(Metasploit::Pro::Engine::DEFAULT_ENVIRONMENT)

      @flash_policy_server = ActiveSupport::OrderedOptions.new
      flash_policy_server.host = '127.0.0.1'
      flash_policy_server.port = 843
      flash_policy_server.run = false

      # needs to be done before eager loading as eager loading if load ActiveRecord::Base models
      initializers.unshift :configure_database_logging
      # ensure  Mdm::Profile responds to load_latest_smtp_configuration by loading Pro UI concerns
      initializers << :initialize_metasploit_data_models

      @log_file = nil
      @quiet = false

      @root = Metasploit::Pro.root.join('engine')

      lib_pathname = root.join('lib')
      lib_path = lib_pathname.to_s
      autoload_paths << lib_path

      @run_in_foreground = false

      @server = ActiveSupport::OrderedOptions.new
      server.host = '127.0.0.1'
      server.port = 50505

      @ssl = false
      @uri = '/api'
    end

    attr_accessor :log_file

    def platform
      unless instance_variable_defined? :@platform
        platform = 'unknown'

        case RUBY_PLATFORM
          when /x86_64-linux/
            platform = 'linux64'
          when /i[3456]86-linux/
            platform = 'linux32'
          when /mingw32/
            platform = 'win32'
        end

        @platform = ActiveSupport::StringInquirer.new(platform)
      end

      @platform
    end

    attr_accessor :quiet

    def quiet?
      quiet == true
    end

    attr_reader :server

    attr_accessor :ssl

    def ssl?
      ssl == true
    end

    attr_accessor :token
    attr_accessor :uri
  end
end