#
# Gems
#
require 'active_support/concern'

#
# Project
#
require 'metasploit/pro/ui'

module Metasploit::Pro::Engine
  module Database
    extend ActiveSupport::Concern

    DEFAULT_DATABASE_WAIT_TIMEOUT = 300
    MAX_DATABASE_POOL = 75

    module ClassMethods
      def connect_to_database

        $stdout.puts '[*] Connecting to the database...'

        yaml_pathname = Metasploit::Pro::UI.root.join('config', 'database.yml')
        dbinfo = YAML.load(yaml_pathname.read)

        db = dbinfo[configuration.environment]
        unless db
          $stderr.puts "[-] No database definition for environment #{configuration.environment}"
          exit(0)
        end

        # Cap the pool size at 75 for ProSvc no matter what
        if db['pool'].to_i > MAX_DATABASE_POOL
          db['pool'] = MAX_DATABASE_POOL
        end

        # Set the wait timeout to 300 if its not set
        db['wait_timeout'] ||= DEFAULT_DATABASE_WAIT_TIMEOUT

        # Add the Pro migrations to the migration path
        migration_pathname = Metasploit::Pro::UI.root.join('db', 'migrate_pro')
        migration_path = migration_pathname.to_s
        framework.db.add_migration_path(migration_path)

        until framework.db.connect(db)
          config = db.select { |k,v| k != 'password' } # don't print password
          $stderr.puts "[-] DB Error: #{framework.db.error} #{config.inspect} #{framework.db.error.backtrace}"

          sleep_time = 15
          $stderr.puts "[-] Sleeping #{sleep_time} seconds and trying to reconnect..."
          sleep(sleep_time)
        end

        # Build the module cache in a background thread
        framework.threads.spawn("ModuleCacheBuilder", true) do
          framework.modules.refresh_cache_from_module_files
        end
      end
    end
  end
end
