$:.push(File.join(File.dirname(__FILE__), '..', '..', '..', 'msf3', 'lib'))

# This *MUST* be before msf/core since otherwise it will load the old RPC first
require 'msf/core/rpc/v10/client'
require 'timeout'


module Pro
class Client

	attr_accessor :rpc
	attr_accessor :last_error

	def initialize(opts={})
		@opts = {
			'SSL'  => false,
			'ServerPort' => 50505,
			'ServerHost' => '127.0.0.1',
			'URI'        => '/api/'
		}

		@opts.merge!(opts)

		if(not @opts['Token'])
			path = ::File.expand_path(::File.join(::File.dirname(__FILE__), "..", "..", "tmp", "servicekey.txt"))
			data = ''
			::File.open(path, "rb") do |fd|
				data = fd.read
			end
			@opts['Token'] = data
		end

		connect
	end

	def connect
		self.rpc = Msf::RPC::Client.new(
			:host => @opts['ServerHost'],
			:port => @opts['ServerPort'],
			:uri  => @opts['URI'],
			:ssl  => @opts['SSL']
		)
		rpc.token = @opts['Token']
	end

	def call(*args)
		begin
			self.rpc.call(*args)
		rescue ::Exception => e
			self.last_error = e
			raise e
		end
	end

	def close
		self.rpc.close
	end

	#
	# Licensing and updates
	#

	def license
		call("pro.license")
	end

	def register(product_key)
		call("pro.register", product_key)
	end

	def revert_license
		call("pro.revert_license")
	end

	def activate(conf)
		call("pro.activate", conf)
	end

	def activate_offline(path)
		call("pro.activate_offline", path)
	end

	def update_available(conf)
		call("pro.update_available", conf)
	end

	def update_install(conf)
		call("pro.update_install", conf)
	end

	def update_install_offline(path)
		call("pro.update_install_offline", path)
	end

	def update_status(conf)
		call("pro.update_status", conf)
	end

	def update_stop(conf)
		call("pro.update_stop", conf)
	end

	def restart_service(conf)
		call("pro.restart_service", conf)
	end

	def wait_for_restart
		self.close
		sleep 2
		begin
			sleep 1
			while true
				self.connect
				break
			end
		rescue ::Exception => e
			retry
		end
	end

	def about
		{
			'core' => call("core.version"),
			'pro'  => call("pro.about"),
			'license' => call("pro.license"),
			'ruby' => { 'version' => RUBY_VERSION, 'platform' => RUBY_PLATFORM }
		}
	end

	#
	# Task management
	#

	def task_stop(task_id)
		call("pro.task_stop", task_id.to_s)
	end

	def task_delete_log(task_id)
		call("pro.task_delete_log", task_id)
	end

	def task_list
		call("pro.task_list")
	end

	def task_wait(task_id)
		t = task_id.to_s
		r = nil
		while(true)
			r = call('pro.task_status', t)
			break if not r[t]
			break if r[t]['status'] != 'running'
			select(nil, nil, nil, 0.5)
		end

		if(r and r[t] and r[t]['status'] != 'invalid')
			call('pro.task_stop', t)
		end
	end

	#
	# Reporting
	#

	def start_export(conf)
		call("pro.start_export", conf)
	end

	def start_import(conf)
		call("pro.start_import", conf)
	end

	def start_report(conf)
		call("pro.start_report", conf)
	end

	def start_import_creds(conf)
		call("pro.start_import_creds", conf)
	end

	def start_report_template(conf)
		call("pro.start_report_template", conf)
	end

	def report_delete_file(report_id)
		call("pro.report_delete_file", report_id)
	end

	def report_template_delete_file(template_id)
		call("pro.report_template_delete_file", template_id)
	end

	def validate_import_file(data)
		call("pro.validate_import_data", data)
	end

	#
	# Scanning and exploitation
	#

	def start_bruteforce(conf)
		call("pro.start_bruteforce", conf)
	end

	def start_campaign(conf)
		call("pro.start_campaign", conf)
	end

  # Start new-style Social Engineering campaign
	def start_se_campaign(conf)
		call("pro.start_se_campaign", conf)
	end

	def start_discover(conf)
		call("pro.start_discover", conf)
	end

	def start_exploit(conf)
		call("pro.start_exploit", conf)
	end

	def start_nexpose(conf)
		call("pro.start_nexpose", conf)
	end

	def start_webscan(conf)
		call("pro.start_webscan", conf)
	end

	def start_webaudit(conf)
		call("pro.start_webaudit", conf)
	end

	def start_websploit(conf)
		call("pro.start_websploit", conf)
	end

	#
	# Post-exploitation
	#

	def start_collect(conf)
		call("pro.start_collect", conf)
	end

	def start_cleanup(conf)
		call("pro.start_cleanup", conf)
	end
	
	def start_upgrade_sessions(conf)
	  call("pro.start_upgrade_sessions", conf)
  end

	def start_download(conf)
		call("pro.start_download", conf)
	end

	def session_list
		call("session.list")
	end

	def session_stop(id)
		call("session.stop", id)
	end

	def start_tunnel(conf)
		call("pro.start_tunnel", conf)
	end

	def start_upload(conf)
		call("pro.start_upload", conf)
	end

	def start_listener(conf)
		call("pro.start_listener", conf)
	end

	def start_replay(conf)
		call("pro.start_replay", conf)
	end

	#
	# Misc
	#

	def generate_exe(conf)
		call("pro.generate_exe", conf)
	end

	def portable_file_generate(conf)
		call("pro.portable_file_generate", conf)
	end

	def loot_delete_file(loot_id)
		call("pro.loot_delete_file", loot_id)
	end

	def loot_create(opts={})
		call("pro.loot_create", opts)
	end


	def module_compatible_sessions(name)
		call("module.compatible_sessions", name)
	end

	def session_compatible_modules(sid)
		call("session.compatible_modules", sid)
	end

	def module_details(type)
		call("pro.modules", type)
	end

	def module_execute(type, name, options)
		call("module.execute", type, name, options)
	end

	def module_options(type, name)
		call("module.options", type, name)
	end

	def module_validate(name, options)
		call("pro.module_validate", name, options)
	end

	def start_single(conf)
		call("pro.start_single", conf)
	end

	def start_nexpose_asset_group_push(conf)
		call("pro.start_nexpose_asset_group_push", conf)
	end

	def start_nexpose_exception_push(conf)
		call("pro.start_nexpose_exception_push", conf)
	end

	def lport_available?(lport,ipv6=false)
		call("pro.lport_available?", lport, ipv6)
	end

end
end

