module Msf
module Pro
module Credentials

class FtpCreds < Builder
	SERVICE_LIMIT = 1000
	DEEP_FACTOR = 1

	def should_downcase_user?
		false
	end

end

class FtpKnownUserpassCreds < FtpCreds
	include KnownUtils
end

class FtpQuickCreds < FtpCreds
	include QuickUtils
end

class FtpCommonCreds < FtpCreds
	include CommonUtils

	ONE_OFF_CREDENTIALS = [
		"anonymous mozilla@example.com",
		# According to Phenoelit...
		# (Dupes removed)
		"anonymous",
		"login access",
		"xbox xbox",
		"rapport r@p8p0r+",
		"anonymous Exabyte",
		"ftp_inst pbxk1064",
		"ftp_admi kilo1987",
		"ftp_oper help1954",
		"ftp_nmc tuxalize",
		# IPC@CHIP (xforce:6597)
		"anonymous ftp",
		# Default FreeBSD (xforce:7537)
		"FTP"
	]

	def seed_with_these_creds?
		true
	end

	def create_common_users
		%w{ftp root user ftpuser admin}
	end

	def create_common_passwords
		%w{pass password pass pass1 1234}
	end

end

class FtpImportedCreds < FtpCreds
	include ImportedUtils
end

class FtpRecombinedCreds < FtpCreds
	include RecombinedUtils
end

class FtpGuessedPasswordCreds < FtpCreds
	include UnixWordlistUtils
	include GuessedPasswordUtils
end

class FtpGuessedUserpassCreds < FtpCreds
	include UnixWordlistUtils
	include GuessedUserpassUtils
end

class FtpMutantCreds < FtpCreds
	include MutantPasswordUtils
end

end
end
end

