module Msf
module Pro
module Credentials

class HttpCreds < Builder
	SERVICE_LIMIT = 6000
	DEEP_FACTOR = 3

	def blank_usernames_allowed?
		false
	end

end

class HttpKnownUserpassCreds < HttpCreds
	include KnownUtils
end

class HttpQuickCreds < HttpCreds
	include QuickUtils
end

class HttpCommonCreds < HttpCreds

	include CommonUtils

	ONE_OFF_CREDENTIALS = [
		"Administrator 100" # Seimen's default.
	]

	def downcase_usernames?
		!(self.flavor == :tomcat)
	end

	# On the fence about this one
	def seed_with_these_creds?
		true
	end

	def create_common_creds
		users = create_common_users
		passes = create_common_passwords
		create_creds_from_users_and_passes(users,passes)
		create_username_based_credentials()
		create_additional_creds
		if self.flavor == :tomcat
			wordlist_proto = "tomcat_mgr"
		else
			wordlist_proto = this_service
		end
		create_creds_from_protocol_wordlists(wordlist_proto)
	end

	def create_common_users
		if self.flavor == :tomcat
			%w{admin manager role1 root tomcat both}
		else
			%w{admin webadmin}
		end
	end

	def create_common_passwords
		common_pws = []
		case self.flavor 
		when :tomcat
			common_pws = %w{root tomcat}
		when :vmware
			common_pws = %w{root vmware esx}
		end
		common_pws + 	%w{password pass pass1 password1 admin}
	end

end

class HttpImportedCreds < HttpCreds
	include ImportedUtils
end

class HttpRecombinedCreds < HttpCreds
	include RecombinedUtils
end

class HttpGuessedPasswordCreds < HttpCreds
	include UnixWordlistUtils
	include GuessedPasswordUtils
end

class HttpGuessedUserpassCreds < HttpCreds
	include UnixWordlistUtils
	include GuessedUserpassUtils
end

class HttpMutantCreds < HttpCreds
	include MutantPasswordUtils
end

end
end
end

