module Msf
module Pro
module Credentials

class SmbCreds < Builder
	SERVICE_LIMIT = 4000
	DEEP_FACTOR = 3

	def blank_usernames_allowed?
		false
	end

	def downcase_usernames?
		true
	end
	
end

class SmbKnownUserpassCreds < SmbCreds
	include KnownUtils
end

class SmbQuickCreds < SmbCreds
	include QuickUtils
end

class SmbCommonCreds < SmbCreds
	include CommonUtils
	include CommonShellUtils

	# Overrides CommonUtils since we want to add other things
	# based on flavor
	def create_common_creds
		users = create_common_users
		passes = create_common_passwords
		create_creds_from_users_and_passes(users,passes)
		create_username_based_credentials()
		case self.flavor
		when :samba
			common_shell_creds
		when :microsoft, :microsoft_nt
			add_microsoft_creds()
		end
		wordlist_proto = [this_service,self.flavor].join("_")
		create_creds_from_protocol_wordlists(wordlist_proto)
	end

	def create_common_users
		%w{administrator admin backup}
	end

	def create_common_passwords
		common_passwords + %w{admin backup}
	end

	def add_microsoft_creds
		add_cred "guest"
		add_cred "Administrator hpinvent"
		add_cred "Administrator admin"
		add_cred "Administrator backup"
	end

end

class SmbImportedCreds < SmbCreds
	include ImportedUtils
end

class SmbRecombinedCreds < SmbCreds
	include RecombinedUtils
end

class SmbGuessedPasswordCreds < SmbCreds
	include UnixWordlistUtils
	include GuessedPasswordUtils
end

class SmbGuessedUserpassCreds < SmbCreds
	include UnixWordlistUtils
	include GuessedUserpassUtils
end

class SmbMutantCreds < SmbCreds
	include MutantPasswordUtils
end

end
end
end

