module Msf
module Pro
module Credentials

# TelnetCreds is the basic cred builder for shell-like sessions.
class TelnetCreds < Builder
	SERVICE_LIMIT = 1000
	DEEP_FACTOR = 1
end

class TelnetKnownUserpassCreds < TelnetCreds
	include KnownUtils
end

class TelnetQuickCreds < TelnetCreds
	include QuickUtils
end

class TelnetCommonCreds < TelnetCreds
	include CommonUtils
	include CommonShellUtils

	ONE_OFF_CREDENTIALS = [
		"Administrator 100" # Seimen's default.
	]

	# Overrides CommonUtils since we want shell creds, too.
	# Also want to add in some special cisco creds.
	def create_common_creds
		users = create_common_users
		passes = create_common_passwords
		create_creds_from_users_and_passes(users,passes)
		create_username_based_credentials()
		common_shell_creds()
		wordlist_proto = [this_service,self.flavor].join("_")
		create_creds_from_protocol_wordlists(wordlist_proto)
		create_telnet_cisco_creds if self.flavor == :cisco
		create_additional_creds
	end

	def create_telnet_cisco_creds
		pairs = ["cisco sanfran pix cisco", "pix cisco"]
		pairs.each do |pair|
			parse_userpass_pair(pair).each {|c| add_cred c}
		end
	end

	def create_common_users
		%w{guest root user toor lp admin}
	end

	def create_common_passwords
		common_passwords + %w{password pass pass1 password1 admin}
	end
end

class TelnetImportedCreds < TelnetCreds
	include ImportedUtils
end

class TelnetRecombinedCreds < TelnetCreds
	include RecombinedUtils
end

class TelnetGuessedPasswordCreds < TelnetCreds
	include UnixWordlistUtils
	include GuessedPasswordUtils
end

class TelnetGuessedUserpassCreds < TelnetCreds
	include UnixWordlistUtils
	include GuessedUserpassUtils
end

class TelnetMutantCreds < TelnetCreds
	include MutantPasswordUtils
end

end
end
end

