module Pro
module Hooks
class CampaignHttp

  require 'msf/core/exploit/http/server'

  attr_accessor :framework

  def initialize(framework)
    self.framework   = framework
    Msf::Exploit::Remote::HttpServer.module_eval do
      include Pro::Hooks::CampaignHttpMixin
    end
    Msf::Exploit::Remote::HttpServer::HTML.module_eval do
      include Pro::Hooks::CampaignHttpMixin
    end
    Msf::Exploit::Remote::HttpServer::PHPInclude.module_eval do
      include Pro::Hooks::CampaignHttpMixin
    end
  end
end


#
# This mixin is injected into all HttpServer modules
#
module CampaignHttpMixin

  # Fires on every request when a web module is running
  def filter_request_uri(cli, req)
    return unless datastore['CAMPAIGN_ID']
    print_status("Hook: #{self.refname} - #{cli.peerhost}:#{cli.peerport}")

    case req.method
      when "POST"
        pro_hook_process_post_request(req, cli)
      when "GET"
        pro_hook_process_get_request(req, cli)
    end

    # Always return false to allow the request to hit the
    # actual module (non-false prevents it)
    return false
  end

  def pro_hook_process_post_request(req,cli)
    print_status("POST request with data from #{cli.peerhost}")
  end

  # Takes a Rex::Proto::Http::Request and a client and makes a VisitRequest.
  # Then it does a redirect based on VisitRequest#type
  def pro_hook_process_get_request(req, cli)
    SocialEngineering::VisitRequest.framework ||= framework
    visit_request = SocialEngineering::VisitRequest.new(req, cli.peerhost)

    case visit_request.type
    when :tracked_user
      print_status("This human target is already tracked")
    when :track_request
      pro_hook_set_cookie_and_redirect(cli, visit_request)
    when :email_open_track
      pro_hook_track_email_and_respond(cli, visit_request)
    when :unknown_user
      print_status("Unknown user")
      pro_hook_track_new_user(cli,req)
    when :bogus_request
      print_error("It looks like someone is trying to mess with us from #{cli.peerhost}:#{cli.peerport}")
    when :favicon
      print_status("Favicon request -- just ignore")
    end
  end

  def pro_hook_track_new_user(cli,req)
    print_status("Not one of our original targets?")
  end

  def pro_hook_set_cookie_and_redirect(cli, visit_request)
    print_good("Request is of type :tracking_request -- set cookie and redirect")
    visit_request.create_visit!
    send_redirect(cli, visit_request.uri_parts['Resource'], '', headers = {'Set-Cookie' => "#{visit_request.encoded_tracking_cookie_string}"})
  end

  def pro_hook_track_email_and_respond(cli, visit_request)
    print_good("Tracking email open")
    visit_request.create_email_opening!
    send_response(cli, "")
  end

end
end
end

Pro::Hooks::Loader.add_hook( Pro::Hooks::CampaignHttp )

