require 'msf/core'
require 'rex/text'

# TODO: This should be removed before release. 
# If there are any dependencies on this mixin, it's a bug.

module Msf
module Pro
module Credentials
end
end
end

module Msf::Pro::Credentials

	# http://www.pcmag.com/article2/0,2817,2113976,00.asp, additions from gawker dump of 12/13/2010
	COMMON_PASSWORDS = %w{!! 123456 12345678 111111 12345 qwerty abc123 letmein monkey link182 changeme test1234}
	BRUTE_SERVICES = %w{ssh mysql mssql db2 postgres http tomcat oracle telnet smb ftp snmp}
	NO_BLANK_USERNAMES = %w{smb mysql mssql db2 postgres oracle login shell exec}

	def gen_found_user_list
		users = []

		myworkspace.notes.find_all_by_ntype('smb.domain.enumusers').each do |u|
			if u.data and u.data[:users].respond_to? :each_key
				u.data[:users].each_key do |uid|
					next if u.data[:users][uid][0,1] == "_" # Skip Mac OS X dummy accounts
					next if uid == 501 # Skip Guest accounts.
					next if u.data[:users][uid] =~ /^SUPPORT_[0-9a-f]{8}$/i # Skip built-in support accounts
					users << u.data[:users][uid].to_s.downcase
				end
			end
		end
		myworkspace.notes.find_all_by_ntype('smb.domain.lookupsid').each do |u|
			if u.data and u.data[:users].respond_to? :each_key
				u.data[:users].each_key do |uid|
					next if u.data[:users][uid][0,1] == "_" # Skip Mac OS X dummy accounts
					next if uid == 501 # Skip Guest accounts.
					next if u.data[:users][uid] =~ /^SUPPORT_[0-9a-f]{8}$/i # Skip built-in support accounts
					users << u.data[:users][uid].to_s.downcase
				end
			end
		end

		# Finger users
		myworkspace.notes.find_all_by_ntype('finger.users').each do |u|
			if u.data and u.data[:users].respond_to? :each
				u.data[:users].each do |username|
					users << username
				end
			end
		end

		users.uniq
	end

	def gen_common_ftp_creds
		creds = []
		# Pretend to be Firefox
		creds << "anonymous mozilla@example.com"

		# According to Phenoelit...
		creds << "anonymous "
		creds << "login access"
		creds << "admin password"
		creds << "xbox xbox"
		creds << "user pass"
		creds << "rapport r@p8p0r+"
		creds << "root 1234"
		creds << "anonymous Exabyte"
		creds << "ftp_inst pbxk1064"
		creds << "ftp_admi kilo1987"
		creds << "ftp_oper help1954"
		creds << "ftp_nmc tuxalize"
		creds << "ftpuser 1234"

		# Hey this is CVE-2008-1264
		creds << "admin admin"

		# IPC@CHIP (xforce:6597)
		creds << "anonymous ftp"

		# Default Linux
		creds << "ftp "

		# Default FreeBSD (xforce:7537)
		creds << "FTP "

		# Serv-U, local-only, don't bother, xforce:16925
		# creds << "LocalAdministrator #l$ak#.lk;0P"

		# ZyXel router
		creds << "admin 1234"
		creds << "admin admin"

		creds.uniq
	end

	# Note that this does not ensure uniqueness of credentials --
	# that's up to the caller.
	# TODO: Merge in: http://www.vulnerabilityassessment.co.uk/fgdump_passwords.txt
	def gen_common_shell_creds
		creds = []

		# Blank username, admin password. Linksys default, see #3022
		creds << " admin"

		# HPUX as of 2010-06-07 apparently
		creds << "catadm catadm"
		creds << "security transfer"

		creds << "pix cisco"      # Cisco PIX
		creds << "root alpine"    # iPhone
		creds << "root dottie"    # iPhone
		creds << "hscroot abc123" # IBM HSC
		creds << "admin admin123" # NAI Intrushield IPS

		# DRAC5
		creds << "root calvin"

		# Various (BIG-IP, Meridian)
		creds << "service smile"
		creds << "service sm1l3"

		# Various DVR gear
		creds << "root admin"

		# Tons of equipment, including Snap servers, Viking VPNs, etc
		creds << "admin admin"
		creds << "guest guest"
		creds << "root default"

		# SimpleNET
		creds << "sshadmin simplenet"

		# BlueCoat
		creds << "admin articon"

		# Radware
		creds << "radware radware"
		creds << "lp lp"

		# AX-100P TrixBox / AtCom IP-PBX
		creds << "root 12xerXes06"

		# MN4L
		creds << "root 0000"

		# IP04
		creds << "root uClinux"

		# InReach MRV
		creds << "InReach access"
		creds << "Superuser system"

		# AdTran System Controller Unit
		creds << "ADMIN PASSWORD"

		# Baytech DS
		creds << "root baytech"

		# APC
		creds << "apc apc"
		creds << "device apc"

		# Avaya
		creds << "avniuadm r8nd0m"

		# WANScaler: http://support.citrix.com/article/CTX112404 (ssh is the same as web)
		creds << "admin orbital"
		creds << "admin wanscaler"

		# VirtualIQ
		creds << "IQMANAGER IQ"

		# Cisco ANA
		creds << "sheer sheer"

		# ISPAdmin
		creds << "root ispadmin"

		# WD ShareSpace
		creds << "downloader downloader"

		# Plesk
		creds << "admin setup"

		# Sentry
		creds << "admn admn"

		# SonicWall: https://www.fuzeqna.com/sonicwallkb/consumer/kbdetail.asp?kbid=3641
		creds << "snwlcli password"
		creds << "root password"
		creds << "admin password"

		# PASTA Microsensor
		creds << "root rootme"

		# Raritan
		creds << "admin raritan"
		creds << "dominion "

		# Various
		creds << "default "
		creds << "nobody "

		# Trend Micro - Threat Discovery Appliance
		creds << "tda "
		creds << "tda admin"

		# AutumnTECH Spam Fighting Appliance
		creds << "root password"
		creds << "spam password"

		# Cymphonix
		creds << "menu cymphonix"

		# Vonage
		creds << "user user"

		# VicidialNOW (mysql as well)
		creds << "root vicidialnow"

		# LANTRONIX SCS100/200/400
		creds << "login access"
		creds << "root system"

		# Porttracker
		creds << "admin appliance"

		# Wireless LAN Device Series ZWA-G120 User's Manual
		creds << "root qwert"

		# Allied Telesis
		creds << "manager friend"

		# OpenSLUG
		creds << "root opeNSLUg"

		# TANDBERG
		creds << "admin TANDBERG"

		# Various
		creds << "root cisco123"

		# Digi Passport
		creds << "root dbps"

		# Various
		creds << "admin guest"
		creds << "admin default"

		# Cyclades ACS
		creds << "root tslinux"

		# Yoggie
		creds << "root yoggie"

		# NetOptics
		creds << "customer netoptics"

		# JetNEXUS
		creds << "admin jetNEXUS"

		# Vyatta
		creds << "root vyatta"

		# Various dev kits
		creds << "root pass"

		# SPA500 Firewall
		creds << "admin pfsense"

		# SilverStorm 9024
		creds << "admin adminpass"

		# BrightMail Appliance
		creds << "admin symantec"

		# FreeNAS
		creds << "admin freenas"

		# FreeWRT
		creds << "admin FreeWRT"

		# NBox
		creds << "root nbox"

		# Wyse
		creds << "root wyse"

		# Vertex
		creds << "root vertex25"

		# AlienTechnology ALR-9900 default root password and backdoor
		# http://seclists.org/fulldisclosure/2010/May/63
		creds << "root alien"

		### Telnet Credentials ###

		# http://www.phenoelit-us.org/dpl/dpl.html
		# Looks like a fine list; lots of root passwords though which will slow
		# things down for Ubuntu machines quite a bit. Need to fix up some kind
		# of parallel model in the telnet module proper to do this right, and
		# probably need to wise up after the fifth root timeout that hey, root
		# is never going to work here.
		creds << 'admin admin'
		creds << 'admin default'
		creds << 'admin hello'
		creds << 'Administrator admin'
		creds << 'Administrator ggdaseuaimhrke'
		creds << 'Administrator '
		creds << 'admin '
		creds << 'admin P@55w0rd!'
		creds << 'admin password'
		creds << 'admin switch'
		creds << 'ADMN admn'
		creds << 'adm '
		creds << 'Alphanetworks wrgg15_di524'
		creds << 'bbsd-client changeme2'
		creds << 'bbsd-client NULL'
		creds << 'cablecom router'
		creds << 'cisco '
		creds << 'corecess corecess'
		creds << 'debug synnet'
		creds << 'd.e.b.u.g User'
		creds << 'diag switch'
		creds << 'D-Link D-Link'
		creds << 'draytek 1234'
		creds << 'echo User'
		creds << 'GEN1 gen1'
		creds << 'GEN2 gen2'
		creds << 'guest User'
		creds << 'iclock timely'
		creds << 'installer installer'
		creds << 'login admin'
		creds << 'login password'
		creds << 'manage !manage'
		creds << 'Manager Admin'
		creds << 'Manager Manager'
		creds << 'Manager '
		creds << 'MD110 help'
		creds << 'netopia netopia'
		creds << 'operator '
		creds << 'recovery recovery'
		creds << 'root 1234'
		creds << 'root admin'
		creds << 'root admin_1'
		creds << 'root default'
		creds << 'root test'
		creds << 'root ggdaseuaimhrke'
		creds << 'root iDirect'
		creds << '!root '
		creds << 'root '
		creds << 'root pass'
		creds << 'root root'
		creds << 'root tini'
		creds << 'security security'
		creds << 'service smile'
		creds << 'super super'
		creds << 'super surt'
		creds << 'superuser 123456'
		creds << 'superuser admin'
		creds << 'superuser '
		creds << 'support support'
		creds << 'sysadm Admin'
		creds << 'sysadm anicust'
		creds << 'sysadm sysadm'
		creds << 'target password'
		creds << 'tech '
		creds << 'tech tech'
		creds << 'User '
		creds << 'write private'

		return creds.uniq

	end

	def gen_username_based_passwords(users)
		creds = []
		users.each { |u|
			creds << "#{u}"
			creds << "#{u} #{u}"
			creds << "#{u} #{u * 2}"
			creds << "#{u} #{u}123"
			creds << "#{u} #{u.reverse}"
			creds << "#{u} #{u}!"
		}
		return creds.uniq
	end

	# Converts a creds array into a hash to count unique lines, and
	# moves the most common combinations to the top. This implies that dupes
	# should be preserved up until this point to get an accurate idea of
	# "most common." Important: Finalize_credentials, despite its name, should
	# only be used during per-section generation, and not on a final list of
	# credentials; otherwise, all the carefully laid out sections will be
	# muddled together.
	def finalize_credentials(creds)
		return [] unless creds.kind_of? Array
		unique_credentials = {}
		creds.flatten.each do |pair|
			unique_credentials[pair] ||= 0
			unique_credentials[pair] += 1
		end
		return unique_credentials.keys.sort {|a,b| unique_credentials[b] <=> unique_credentials[a] }
	end

	# Adjusts the case of usernames and passwords. Note that we shouldn't
	# be creating duplicates with this process, but we do want to preserve
	# dupes that existed previously. TODO: fix to ensure that new dupes
	# aren't created.
	def fix_credentials_case(creds=[],ucase=false,pcase=false)
		new_creds = []
		creds.each do |pair|
			u,p = pair.split(/[\s]+/,2)
			u = u.to_s.downcase if ucase
			p = p.to_s.downcase if pcase
			new_creds << "#{u} #{p}"
		end
		return new_creds
	end

	def gen_even_more_shell_creds
		creds = []
		root_creds = File.join(Msf::Config.install_root, "data", "wordlists", "root_userpass.txt")
		unix_user_file = File.join(Msf::Config.install_root, "data", "wordlists", "unix_users.txt")
		unix_pass_file = File.join(Msf::Config.install_root, "data", "wordlists", "unix_passwords.txt")

		if File.exist? root_creds
			creds << File.open(root_creds) {|f| f.read(f.stat.size)}.split(/\n/)
		end

		if File.exist?(unix_user_file) and File.exist?(unix_pass_file)
			user = File.open(unix_user_file) {|f| f.read(f.stat.size)}.split(/\n/).map {|x| x.strip}
			pass = File.open(unix_pass_file) {|f| f.read(f.stat.size)}.split(/\n/).map {|x| x.strip}
			user.each do |u|
				pass.each do |p|
					creds << "#{u} #{p}"
				end
			end
		end
		return creds
	end

	# If a wordlist file is found and is readable, canoncialize_wordlist_file
	# rewrites that protocol's USERPASS_FILE datastore to include the full
	# pathname, and returns true. Otherwise, returns false.
	def canonicalize_wordlist_file(proto='')
		proto = proto.to_s.upcase
		brute_file = "#{proto}_USERPASS_FILE.txt"
		# brute_file = datastore["#{proto}_USERPASS_FILE"].to_s.strip
		fullpath_file = File.join(Msf::Config.install_root, "data", "wordlists" , brute_file)
		if File.readable? fullpath_file
			datastore["#{proto}_USERPASS_FILE"] = fullpath_file
		else
			datastore["#{proto}_USERPASS_FILE"] = nil # It's bad, so toss it.
		end
		# Return the state of the userpass file.
		!datastore["#{proto}_USERPASS_FILE"].nil?
	end

	def split_creds(in_array,factor)
		speeds = [1,1,2,4,8,16]
		round_robin_split(in_array,speeds[factor])
	end


	# Array splitter.
	def round_robin_split(in_array,count)
		len = in_array.size
		return in_array.map {|x| [x]} if count >= len
		return [in_array] if count <= 1
		arr = in_array.dup
		split_arr = []
		count.times { split_arr << [] }
		i = 0
		while !arr.empty? do
			split_arr[i % count] << arr.shift
			i += 1
		end
		return split_arr
	end

	# If hex notation is present, turn them into a character.
	def dehex(str)
		hexen = str.scan(/\x5cx[0-9a-fA-F]{2}/)
		hexen.each { |h|
			str.gsub!(h,h[2,2].to_i(16).chr)
		}
		return str
	end

	# Takes the BRUTEFORCE_QUICKMODE_CREDS datastore (populated
	# via the UI as DS_BRUTEFORCE_QUICKMODE_CREDS), validates
	# each user pass combination, and returns the lot as a
	# creds array. Note that blank passwords are allowed, but
	# blank usernames are not. Also, multiple passwords are allowed.
	# Also, pwdump hash lines are allowed, and split accordingly --
	# it's up to the consuming protocol to ignore pwdump hashes.
	def collect_quickmode_creds(host)
		creds = []
		return creds unless datastore['BRUTEFORCE_QUICKMODE_CREDS']
		return creds if datastore['BRUTEFORCE_QUICKMODE_CREDS'].empty?
		datastore['BRUTEFORCE_QUICKMODE_CREDS'].strip.split("\n").each do |pair|
			creds = creds | parse_userpass_pair(pair,host)
		end
		return creds
	end

	# Takes a "user pass" space-seperated pair, splits into user,pass,
	# and fills an array of processed versions of the strings. PWDump-style
	# hashes are split up (and NO PASSWORD PWDump hashes are dealt with),
	# hex characters like \x20 and \xff are de-hexed, and lines of 
	# user pass1 pass2 pass3 are split into "user pass1", "user pass2", etc.
	def parse_userpass_pair(pair,host)
		creds = []
		# Test for a pwdump-style of SMB hashes.
		if pair.strip =~ /^([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/
			if host.service == "smb"
				user,hash = $1,$2
				creds << "#{user} #{hash}"
			end
		# Test for a pwdump-style of SMB hashes when you don't have a password.
		elsif pair.strip =~ /^([^\s:]+):([0-9]+):NO PASSWORD\*+:NO PASSWORD\*+[^\s]*$/
			if host.service == "smb"
				user,sid = $1,$2.to_i
				if sid != 501 # Skip the Guest account (by SID)
					creds << "#{user} "
				end
			end
		else
			u,passes = pair.split("\s",2)
			if(u.kind_of?(String) and !u.empty? and u =~ /^[\x21-\x7f]/)
				if(passes and !passes.empty? and passes =~ /^[\x21-\x7f]/)
					passes.split("\s").each {|p| creds << "#{u} #{dehex(p)}" }
				else
					creds << "#{u}"
				end
			end
		end
		return creds
	end

	# Similar to parse_userpass_pair, parse_just_users takes a string,
	# strips it, dehexes it, and returns it. It also
	# deals with PWDump lines by chopping at the first colon. Unlike
	# parse_userpass_pair, we'll suck the usernames out regardless if
	# the target host is actually smb.
	#
	# Note that if someone accidentally calls a userpass file a user file,
	# we should be able to handle it sensibly (ie, only returning the 
	# userpart)
	def parse_just_users(user)
		this_user = nil
		# Test for a pwdump-style of SMB hashes.
		if user.to_s.strip =~ /^([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/
			this_user = $1
		else
			userpart = user.to_s.split(/\s+/).first
			this_user = dehex(userpart) if userpart
		end
		return this_user
	end

	# Unlike usernames, we can't be sure what the user meant if they mislabel their
	# file. Is it user pass, user pass1 pass2, or pass1 pass2? It's best to just take
	# them at their word, and just take the whole line as a password. This means that
	# passwords can easily have spaces, which is often complained about.
	#
	# That said, we can still tell if it's a PWDump file, and return the hash part
	# if the target can consume that.
	def parse_just_passes(pass,host)
		this_pass = nil
		# Test for a pwdump-style of SMB hashes.
		if this_pass.to_s.strip =~ /^([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/
			if host.service == "smb"
				hash = $2
				this_pass = hash 
			end
		else 
			# Passwords of all whitespace? Nearly always going to be a user error.
			unless pass.to_s.strip.empty?
				this_pass = dehex(pass) if pass
			end
		end
		return this_pass
	end

	# In the event the user didn't provide anything in additional hosts,
	# supply some for quick mode credentials.
	def gen_credentials_quick(host)
		if collect_quickmode_creds(host).empty?
			creds = %W{
				admin\ admin
				admin\ admin1
				admin\ admin!
				test\ test
				test\ test1234
				test123\ test123
				cisco\ cisco
				user\ user
				administrator\ administrator
				root\ root
				root\ toor
				admin
				test
				test123
				cisco
				user
				administrator
				root
			}
			creds << " admin" unless NO_BLANK_USERNAMES.include?(host.service)
		else
			creds = collect_quickmode_creds(host)
		end
		return creds.uniq
	end

	# If a username is in the form "domain/username," extract
	# the username part. Can take a Cred or two strings.
	def short_username(c="",s=nil)
		if c.respond_to? :user
			u = c.user
			s ||= c.service.name # Allow a service override
		else
			u = c
		end
		if u && u[/^[^\x2f]+\x2f./] && s =~ /^(smb|postgres|db2|oracle)/
			u.split("/",2)[1]
		else
			u
		end
	end

	# Returns an array of domain names from a space or
	# comma delimited list. At the least, return an array
	# of ["."]
	def smb_domain_list
		domains = datastore["SMB_DOMAINS"].split(/[,\s]+/)
		domains = ["."] if domains.empty?
		return domains
	end

	# Takes a plain cred array and prepends all the domains to all
	# the users.
	def prepend_smb_domains(creds)
		creds_with_domains = []
		creds.each do |cred|

			user,pass = cred.split(/[\s]+/,2)

			if datastore["PRESERVE_DOMAINS"]
				if user != short_username(user,"smb")
					creds_with_domains << cred
				end
			end

			smb_domain_list().each do |domain|
				du = [domain,short_username(user,"smb")].join("/")
				creds_with_domains << [du,pass].join(" ")
			end
		end
		return creds_with_domains.uniq
	end

	# Generate the data for
	# credentials. Credentials files will take into account:
	#
	#     * The chosen protocol
	#     * The flavor of that protocol (Samba vs Microst, eg)
	#     * Known credentials
	#     * Known machine names.
	#     * Protocol and flavor-specific credentials.
	#     * "Fast" or "Deep" modes (short or long, basically)
	#
	# What this amounts to is that we shouldn't generate credentials
	# lists until we know all these things first, which means it needs
	# to be deferred until we're ready to kick off the login module.
	# This also means that the login modules need to run segregated
	# by protocol flavor. For now this will mean one at a time (though
	# threading will speed things up).
	#
	def gen_credentials(host)

		credentials = {}
		credentials_array = []

		# There doesn't seem to be much point skipping known good
		# usernames and passwords for really any service in this
		# workspace -- why can't an SMB user have the pass credential
		# for SSH?
		#
		# In other words, always start with known, unless the user
		# doesn't want knowns

		credentials[:known] = collect_known_credentials()
		if host.service == "smb" and host.flavor.to_s[/microsoft/]
			credentials[:known] = credentials[:known] | collect_known_hashes
		end
		if NO_BLANK_USERNAMES.include? host.service
			credentials[:known] = credentials[:known].reject {|cred| 
				u = cred.split(/[\s]+/,2)[0]
				u.nil? || u.empty? || u[/\x2f$/]
			}
		end
		if host.service == "ssh"
			credentials[:known] = credentials[:known] | collect_known_keys
		end
		credentials[:known] = finalize_credentials(credentials[:known])
		if datastore["INCLUDE_KNOWN"]
			credentials_array = credentials_array | credentials[:known]
		end

		# Process quick credentials. This will be either the very small set of
		# quick creds, or the set provided in the Additional Credentials box.
		credentials[:quick] = finalize_credentials(gen_credentials_quick(host))

		# If the user selects "known" return known creds + any additional
		# creds the user selected. This can lead to zero creds if the user
		# skips knowns and doesn't populate Additional Credentials.
		if datastore["BRUTEFORCE_SCOPE"] == "known"
			unless collect_quickmode_creds(host).empty?
				credentials_array = credentials_array | credentials[:quick]
			end
			if host.service == "smb" and host.flavor == :microsoft
				credentials_array = downcase_usernames(credentials_array)
			end
			debug_credential_breakdown(credentials,host)
			if host.service == "smb"
				return prepend_smb_domains(credentials_array.uniq)
			elsif host.service == "vnc"
				return just_uniq_passwords(credentials_array)
			elsif host.service == "shell"
				return just_uniq_users(credentials_array)
			else
				return credentials_array.uniq
			end
		end

		# Deal with imported credentials. Populates the following:
		# credentials[:imported] - userpass files
		# credentials[:imported_users] - just usernames
		# credentials[:imported_passes] - just passwords
		# 
		# Note that this method relies on pass-by-reference for credentials.
		collect_imported_credentials(credentials,host)

		# If the user selects "imported", handle the imported files,
		# and just return those plus quickmode. 
		if datastore["BRUTEFORCE_SCOPE"] == "imported"
			unless collect_quickmode_creds(host).empty?
				if credentials[:quick]
					credentials_array = credentials_array | credentials[:quick]
				end
			end
			# Add imported stuff.
			if credentials[:imported]
				credentials_array = credentials_array | credentials[:imported]
			end
			if credentials[:imported_users] && credentials[:imported_passwords]
				credentials[:imported_users].uniq.each do |u|
					credentials[:imported_passwords].uniq.each do |p|
						credentials_array << "#{u} #{p}"
					end
				end
			end

			debug_credential_breakdown(credentials,host)

			# Do service specific transforms:
			if host.service == "smb" 
				if host.flavor == :microsoft
					credentials_array = downcase_usernames(credentials_array)
				end
				return prepend_smb_domains(finalize_credentials(credentials_array))
			elsif host.service == "vnc"
				return just_uniq_passwords(credentials_array)
			elsif host.service == "shell"
				return just_uniq_users(credentials_array)
			else
				return finalize_credentials(credentials_array)
			end
		end

		# If the user selects "quick," return known creds + additional
		# creds OR the small set of quick creds. 
		if datastore["BRUTEFORCE_SCOPE"] == "quick"
			# Unlike known, always include credentials[:quick]
			credentials_array = credentials_array | credentials[:quick]
			debug_credential_breakdown(credentials,host)
			if host.service == "smb" and host.flavor == :microsoft
				credentials_array = downcase_usernames(credentials_array)
			end
			case host.service
			when "smb"
				return prepend_smb_domains(credentials_array.uniq)
			when "vnc", "snmp"
				return just_uniq_passwords(credentials_array)
			when "shell"
				return just_uniq_users(credentials_array)
			else
				return credentials_array.uniq
			end
		end

		credentials[:defaults] = finalize_credentials(gen_credentials_defaults(host))

		if datastore["BRUTEFORCE_SCOPE"] == "defaults"
			# Always include known and default, maybe quick if there's user
			# input there.
			unless collect_quickmode_creds(host).empty?
				credentials_array = credentials_array | credentials[:quick]
			end
			credentials_array = credentials_array | credentials[:defaults]
			if host.service == "smb" and host.flavor == :microsoft
				credentials_array = downcase_usernames(credentials_array)
			end
			debug_credential_breakdown(credentials,host)
			if host.service == "smb"
				return prepend_smb_domains(credentials_array.uniq)
			elsif host.service == "vnc"
				return just_uniq_passwords(credentials_array)
			elsif host.service == "snmp"
				return just_uniq_passwords(credentials_array)				
			elsif host.service == "shell"
				return just_uniq_users(credentials_array)
			else
				return credentials_array.uniq
			end
		end

		credentials[:baseline] = gen_credentials_baseline(
			credentials[:imported_users],
			credentials[:imported_passwords]
		)

		# Credentials based on discovered usernames. Note that "SKIP KNOWN" has
		# no effect on this -- we still at least want to know usernames to try other
		# passwords.
		credentials[:known_users] = gen_credentials_known_users(credentials[:known],host)
		
		# Now we're into new guesses.

		credentials[:baseline] = finalize_credentials(credentials[:baseline])
		case host.service
		when "ssh"
			credentials[:service] = gen_credentials_ssh
		when "telnet"
			credentials[:service] = gen_credentials_telnet
			credentials[:flavor]  = gen_credentials_telnet(host.flavor)
		when "ftp"
			credentials[:service] = gen_credentials_ftp
			credentials[:flavor]  = gen_credentials_ftp(host.flavor)
		when "smb"
			credentials[:service] = gen_credentials_smb
			credentials[:flavor]  = gen_credentials_smb(host.flavor)
		when /^http/
			case host.flavor
			when :tomcat
				credentials[:service] = gen_credentials_tomcat
			else
				credentials[:service] = gen_credentials_http
			end
		when "mysql"
			credentials[:service] = gen_credentials_mysql
		when "mssql"
			credentials[:service] = gen_credentials_mssql
		when "db2"
			credentials[:service] = gen_credentials_db2
		when "postgres"
			credentials[:service] = gen_credentials_postgres
		when "oracle"
			credentials[:service] = gen_credentials_oracle
		when "vnc"
			credentials[:service] = gen_credentials_vnc	
=begin
		# Nothing special here
		when "login", "shell", "exec"
			credentials[:service] = gen_credentials_rservices
=end
		end

		# Finalize each section (Already did known and quick)
		# Note this ensures each section has at least an empty array.
		[:known_users, :baseline, :service, :flavor, :imported].each { |key|
			credentials[key] = finalize_credentials(credentials[key])
		}

		debug_credential_breakdown(credentials,host)
		# Once that's all done, we need to convert it to one big array.
		credentials_array = credentials_array | credentials[:quick]
		credentials_array = credentials_array | credentials[:imported]
		credentials_array = credentials_array | credentials[:defaults]
		credentials_array = credentials_array | credentials[:known_users]
		# Service credentials are specific to that service; they should
		# come next.
		service_credentials = []
		service_credentials = credentials_array | credentials[:flavor]
		service_credentials = credentials_array | credentials[:service]
		credentials_array = credentials_array | service_credentials

		# Baseline credentials are generated guesses, or drawn from imported
		# but unconnected user and pass files. Try these last, time/space
		# permitting.
		other_credentials = credentials[:baseline]
		credentials_array = credentials_array | other_credentials				

		if datastore['MAXGUESSESOVERALL'].to_i.abs != 0
			@running_cred_total ||= 0
			max_guesses = datastore['MAXGUESSESOVERALL'].to_i.abs
			if @running_cred_total >= max_guesses
				credentials_array = []
			else
				credentials_array = adjust_by_service_and_scope(host,credentials_array.flatten.uniq)
				if @running_cred_total + credentials_array.size > max_guesses
					credentials_array = credentials_array[0,max_guesses - @running_cred_total]
				end
			end
			@running_cred_total += credentials_array.size
		else
			credentials_array = adjust_by_service_and_scope(host,credentials_array.flatten.uniq)
		end

		# SMB is special.
		if host.service == "smb"
			domain_adjusted_creds = prepend_smb_domains(credentials_array) # This can be slightly lower than max due to dupe-killing.
		else
			domain_adjusted_creds = credentials_array
		end
		if host.flavor.to_s.empty?
			vprint_status("Truncated base credential list for #{host.service} to #{domain_adjusted_creds.size} due to time limits")
		else
			vprint_status("Truncated base credential list for #{host.service} (#{host.flavor}) to #{domain_adjusted_creds.size} due to time limits")
		end
		return domain_adjusted_creds
	end

	# Currently, we deal with four types of imported credentials:
	# 
	# Userpass files: These are the easiest to deal with. They're straight up
	# userpass files that are normal for metasploit modules.
	#
	# User files: These are lists of usernames to seed with.
	#
	# Pass files: these are lists of passwords to seed with.
	#
	# PWDump files: These are the files generated by PWDump exports, either
	# from a regular pwdump3, or from our PWDump formatted export. If the
	# latter, they can contain userpass combos, smb hashes, and ssh keys.
	#
	# TODO: Import SSH keys. It's annoying we don't do this already.
	def collect_imported_credentials(credentials,host)
		creds = []
		cred_file_ids = datastore["CRED_FILE_IDS"]
		if cred_file_ids && !cred_file_ids.empty?
			files_and_types = {}
			cred_file_ids.split(/\s+/).each do |id|
				fpath = myworkspace.cred_files.find(id).path
				ftype = myworkspace.cred_files.find(id).ftype
				files_and_types[fpath] = ftype
			end
			files_and_types.each do |file,type|
				next unless ::File.readable? file
				case type
				when "userpass"
					credentials[:imported] ||= []
					credentials[:imported] = credentials[:imported] | parse_imported_userpass_file(file,host)
				when "user"
					credentials[:imported_users] ||= []
					credentials[:imported_users] = credentials[:imported_users] | parse_imported_user_file(file,host)
				when "pass"
					credentials[:imported_passwords] ||= []
					credentials[:imported_passwords] = credentials[:imported_passwords] | parse_imported_pass_file(file,host)
				when "pwdump"
					credentials[:imported] ||= []
					credentials[:imported] = credentials[:imported] | parse_imported_pwdump_file(file,host)
				else
					elog "Cannot parse imported credential file #{file.id} with ftype #{type}"
					next
				end
			end
		end
	end

	# Mostly reanimated from the old credential importer. Turns out, never
	# really did much with SSH keys then either. TODO: Make the ssh thing
	# right.
	def parse_imported_pwdump_file(file,host)
		creds = []
		return creds unless ::File.readable? file
		begin
			fh = ::File.open(file, "rb")
			fh.seek 0
			in_ssh = false
			fh.each_line do |line|
				next if line.to_s.strip.empty?
				user = nil; pass = nil; ptype = nil
				case line.to_s.strip
				when /^-----BEGIN [DR]SA PRIVATE KEY-----/
					in_ssh = true
					# Since we have this, this is one of our PWDump files. The last line was almost
					# certainly an ssh key fingerprint, so nuke it if so.
					creds.pop if creds.last =~ /^[^\s]+\s'[0-9a-f:]{47}'/
					next
				when /^-----END [DR]SA PRIVATE KEY-----/
					in_ssh = false
					next
				when /^[\s]*#/ # Comment lines
					next
				when /^[\s]*Warning:/
					next # Discard warning messages.
				when /^[\s]*([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/ # SMB Hash
					user = ([nil, "<BLANK>"].include?($1)) ? "" : $1
					pass = ([nil, "<BLANK>"].include?($2)) ? "" : $2
					ptype = "smb_hash"
				when /^[\s]*([^\s:]+):([0-9]+):NO PASSWORD\*+:NO PASSWORD\*+[^\s]*$/ # SMB Hash
					user = ([nil, "<BLANK>"].include?($1)) ? "" : $1
					pass = ""
					ptype = "smb_hash"
				when /^[\s]*([\x21-\x7f]+)[\s]+([\x21-\x7f]+)?/ # Must be a space-delimited user pass entry
					user = ([nil, "<BLANK>"].include?($1)) ? "" : dehex($1)
					pass = ([nil, "<BLANK>"].include?($2)) ? "" : dehex($2)
					ptype = "userpass"
				else # No spaces, so it's either an SSH key or a password.
					next if in_ssh
					user = ""
					pass = dehex(line.to_s.strip)
					ptype = "password"
				end # case
				if [user,pass,ptype].compact.size == 3
					case ptype
					when "smb_hash"
						if host.service == "smb"
							creds << ("%s %s" % [user,pass])
						end
					when "userpass"
						creds << ("%s %s" % [user,pass])
					when "password"
						# Do something...
					end
				end
			end
		ensure
			fh.close
		end
		return creds
	end

	# TODO: nail down the downcasing situations. Insanely, there are some.
	def parse_imported_pass_file(file,host)
		passes = []
		fdata = ::File.open(file, "rb") {|f| f.read f.stat.size}
		fdata.each_line do |line|
			this_pass = parse_just_passes(line.strip,host)
			passes << this_pass if this_pass
		end
		return passes.uniq
	end

	# TODO: nail down the downcasing situations
	def parse_imported_user_file(file,host)
		users = []
		fdata = ::File.open(file, "rb") {|f| f.read f.stat.size}
		fdata.each_line do |line|
			this_user = parse_just_users(line.strip)
			users << this_user if this_user
		end
		if host.service == "smb" and host.flavor == :microsoft
			return users.map {|u| u.downcase}.uniq
		else
			return users.uniq
		end
	end

	# Returns an array of user pass credentials
	def parse_imported_userpass_file(file,host)
		creds = []
		fdata = ::File.open(file, "rb") {|f| f.read f.stat.size}
		fdata.each_line do |pair|
			creds = creds | parse_userpass_pair(pair,host)
		end
		return creds
	end

	def debug_credential_breakdown(credentials,host)
		# Comment out the return to see exactly where each credential comes into the list.
		return 
		credentials.each { |k,v| print_debug "CRED BREAKDOWN #{host.address}:#{host.service}: #{k.inspect} :: #{v.size} ::: #{v}" }
	end

	def protocol_wordlists(proto, private = false)
		users = []
		passes = []
		creds = []
		
		suffix = private ? "/.." : ""
		
		ufile  = ::File.expand_path(::File.join(Msf::Config.install_root + suffix, "data", "wordlists", "#{proto}_default_users.txt"))
		pfile  = ::File.expand_path(::File.join(Msf::Config.install_root + suffix, "data", "wordlists", "#{proto}_default_pass.txt"))
		upfile = ::File.expand_path(::File.join(Msf::Config.install_root + suffix, "data", "wordlists", "#{proto}_default_userpass.txt"))
			
		if ::File.readable?(ufile)
			users = ::File.open(ufile) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		
		if ::File.readable?(pfile)
			passes = ::File.open(pfile) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		
		if ::File.readable?(upfile)
			creds = ::File.open(upfile) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		
		# Required to allow pass lists to be useful for password-only protocols
		if users.length == 0
			users << "admin"
		end

		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		return creds
	end

	def downcase_usernames(creds)
		new_creds = []
		creds.each do |cred|
			u,p = cred.split(" ", 2)
			new_creds << "#{u.to_s.downcase} #{p}"
		end
		new_creds.uniq
	end

	# This seeks to pad and limit the password list by what is reasonable
	# to try based on speed and desired depth. NOTE: SMB lists are now
	# multiplied by the number of domains you're checking. This is probably
	# the most expected behavior.
	def adjust_by_service_and_scope(host,creds)
		if datastore['BRUTEFORCE_SCOPE'] == "deep"
			# Deep does not affect ssh, telnet, ftp, or login. Takes too long.
			factor = (["ssh","telnet","ftp","login"].include?(host.service)) ? 1 : 3
		else
			factor = 1
		end


		# Since we have the whole host info, we
		# can make this more granular based on
		# OS or :flavor or whatever.
		case host.service
		when "ssh", "telnet", "ftp", "vnc"
			 # This will take a while
			 target = 1000
		when "smb"
			case host.flavor
			when :microsoft_nt # The indirect auth seems to take like 3 seconds per guess.
				target = 1000
			else
				target = 4000
			end
		when "postgres"
			target = 4000
		when "oracle"
			target = 1000
		when "db2"
			target = 3000
		when "mysql", "mssql"
			target = 10_000
		when /^http/
			target = 6000
		when "exec", "shell", "login"
			target = 1000
		when "snmp"
			target = 3000
		else
			print_error("Unknown service type: #{host.service}")
			target = 1000
		end

		case datastore["BRUTEFORCE_SCOPE"]
		when "50k"
			max = 50_000
		else
			max = target * factor
		end

		if datastore["MAXGUESSESOVERALL"].to_i.abs != 0
			if @running_cred_total
				new_max = datastore["MAXGUESSESOVERALL"].to_i.abs - @running_cred_total
				if max > new_max
					vprint_status "Instead of generating #{max} credentials for #{host.service}, only generating #{new_max}."
					max = datastore["MAXGUESSESOVERALL"].to_i.abs - @running_cred_total
				end
			end
		end

		# We can skip generating extra passwords for RSH since
		# it doesn't use them anyway
		if host.service == "shell"
			return just_uniq_users(creds)
		else
			users = creds.map { |pair| pair.split(/[\s]+/, 2).first.to_s }.uniq
			if datastore["BRUTEFORCE_SCOPE"] == "50k"
				vprint_status "Generating 50,000 credentials for hosts running #{host.service}"
			end
			new_passes = gen_other_common_passwords()
			new_passes.each do |p|
				users.each do |u|
					break if creds.size >= max
					creds << "#{u} #{p}" unless creds.include? "#{u} #{p}"
				end
			end

			if creds.size >= max
				if ["vnc", "snmp"].include?(host.service)
					return just_uniq_passwords(creds)
				end
				return creds[0,max]
			end

			more_creds = creds.dup

			creds.each do |pair|
				break if more_creds.size >= max
				user,pass = pair.split(/[\s]+/, 2)
				passes = character_convert(pass.to_s)
				passes.each do |p|
					break if more_creds.size >= max
					more_creds << "#{user} #{p}" unless more_creds.include? "#{user} #{p}"
				end
			end

			# It's pretty unlikely to run out once we've gotten this far, since
			# character_convert is generally going to at least 4x the number of
			# starting credentials.
			if ["vnc", "snmp"].include?(host.service)
				return just_uniq_passwords(more_creds)
			end
			return more_creds
		end

		# ?!
		[]
	end

	# Returns an array of the given string with several
	# characters converted in normal password-altering
	# ways: Numbers subbed for letters, "1" and "!" tacked on
	# the end, and all uppercase. Could include more but
	# then you risk bumping even more candidates off the end.
	# Example:
	# >> character_convert "babygirl"
	# => ["babyg1rl", "babyg!rl", "babygir1", "babygir!", "b4bygirl", "b@bygirl", "b4byg1rl", "b4byg!rl", "b4bygir1", "b4bygir!", "b@byg1rl", "b@byg!rl", "b@bygir1", "b@bygir!", "babygirl!", "babygirl1", "BABYGIRL", "Babygirl"]
	def character_convert(pass)
		passwords = []
		return passwords unless pass.kind_of? String
		passwords << pass.tr("Oo","0") if pass =~ /O/i
		passwords << pass.tr("Ii","1") if pass =~ /I/i
		passwords << pass.tr("Ii","!") if pass =~ /I/i
		passwords << pass.tr("Ll","1") if pass =~ /L/i
		passwords << pass.tr("Ll","!") if pass =~ /L/i
		passwords << pass.tr("Ee","3") if pass =~ /E/i
		passwords << pass.tr("Aa","4") if pass =~ /A/i
		passwords << pass.tr("Aa","@") if pass =~ /A/i
		passwords << pass.tr("Tt","7") if pass =~ /T/i
		passwords << pass.tr("OoIiEeAaTt","0011334477")
		passwords << pass.tr("OoIiEeAaTt","00!!334477")
		passwords << pass.tr("OoLlEeAaTt","0011334477")
		passwords << pass.tr("OoLlEeAaTt","00!!334477")
		passwords << pass.tr("OoIiEeAaTt","001133@@77")
		passwords << pass.tr("OoIiEeAaTt","00!!33@@77")
		passwords << pass.tr("OoLlEeAaTt","001133@@77")
		passwords << pass.tr("OoLlEeAaTt","00!!33@@77")
		passwords << "#{pass}!" unless pass =~ /!$/
		passwords << "#{pass}1" unless pass =~ /1$/
		passwords << pass.upcase unless pass.upcase == pass
		passwords << pass.capitalize unless pass.capitalize == pass
		return passwords.uniq
	end

	def collect_known_credentials()
		known_creds = []
		myworkspace.each_cred do |cred|
			# Deal with SMB hashes, SSH Keys, and everything that's not a plaintext password elsewhere.
			next unless cred.ptype =~ /^password/
			u = (cred.service == "smb" ? cred.user.to_s : short_username(cred))
			p = cred.pass.to_s
			u = u.to_s.downcase if(cred.service.name == "smb" and cred.service.host.os_name =~ /Microsoft/i)
			next if u.to_s.downcase == "anonymous" and cred.service.name == "ftp"
			known_creds << "#{u} #{p}"
		end
		known_creds = finalize_credentials(known_creds)
	end

	def collect_known_hashes()
		known_creds = []
		myworkspace.each_cred do |cred|
			next unless cred.ptype == "smb_hash"
			next if cred.user.to_s.downcase =~ /^support_[0-9a-f]{8}/
			# Strip off the domain name
			if cred.user =~ /^([^\x2f]+)\x2f([^\x2f]+)/
				user = $2
			else
				user = cred.user
			end
			known_creds << "#{user} #{cred.pass}"
		end
		known_creds = finalize_credentials(known_creds)
		return known_creds
	end

	# Really, this just picks up the usernames. The keys themselves
	# are tried seperately.
	def collect_known_keys()
		known_creds = []
		myworkspace.each_cred do |cred|
			next unless cred.ptype == "ssh_key"
			known_creds << "#{cred.user}"
		end
		known_creds = finalize_credentials(known_creds)
		return known_creds
	end

	# FIXME: Refactor this and gen_credentials_* for consistency. There's
	# a lot of slight differences -- gen_credentials_defaults is basically
	# a paired down version of each gen_credentials_*, so really, gen_credentials_*
	# should be calling this. Right now, this just means some wasted cycles --
	# practically, the Array#uniq calls all over the place ensures that there are
	# no dupes.
	def gen_credentials_defaults(host)
		creds = []
		case host.service
		when "ssh"
			creds << gen_common_shell_creds
		when "telnet"
			case host.flavor
			when :cisco
				creds << protocol_wordlists("telnet_cisco", true)			
			end
			creds << gen_common_shell_creds
		when "ftp"
			creds << gen_common_ftp_creds
		when "smb"
			creds << "administrator administrator"
			creds << "administrator admin"
			creds << "administrator test"
			creds << "backup backup"
			creds << "admin backup"
			creds << "admin admin"
			case host.flavor
			when :microsoft
				creds.unshift "administrator hpinvent"
				creds.unshift "administrator admin"
				creds.unshift "administrator backup"
				creds.unshift "guest"
			when :samba
				creds.concat(gen_common_shell_creds)
			end
		when /^http/
			case host.flavor
			when :tomcat
				creds << "admin admin"
				creds << "manager manager"
				creds << "role1 role1"
				creds << "root root"
				creds << "tomcat tomcat"
				creds << "both both"
			else
				# Blank username, admin password. Linksys default, see #3022
				creds << " admin"

				# Find devices which TELL us the default password
				myworkspace.services.find_all_by_name("http").each do |s|
					next if not (s.info and s.info.length > 0)
					if s.info =~ /default is ([a-z0-9A-z]+)/i
						creds << "anyuser #{$1}"
						next
					end
					if s.info =~ /Default password.([a-z0-9A-z]+)/i
						creds << "anyuser #{$1}"
						next
					end
				end
			end
		when "mysql"
			%w{mysql admin root}.each do |u|
				creds << "#{u}"
				creds << "#{u} #{u}"
				creds << "#{u} blank"
			end
		when "mssql"
			creds << "entldbdbo dbopswd"
			creds << "ARAdmin AR#Admin#"
			creds << "entldbreader rdrpswd"
			%w{sa mon_user probe repl_publisher repl_subscriber}.each do |u|
				creds << "#{u}"
				creds << "#{u} #{u}"
				creds << "#{u} blank"
			end
		when "db2"
			%w{db2inst1 dasusr1 db2fenc1 admin}.each do |u|
				creds << "#{u}"
				creds << "#{u} #{u}"
				creds << "#{u} db2pw"
				creds << "#{u} db2password"
				creds << "#{u} db2pass"
				creds << "#{u} blank"
			end
		when "postgres"
			%w{postgres root admin}.each do |u|
				creds << "#{u}"
				creds << "#{u} #{u}"
				creds << "#{u} blank"
			end
		when "oracle"
			creds << "scott tiger"
		end

		# Incorporate wordlists (private and public) if they exist.
		if host.flavor == "tomcat"
			creds << protocol_wordlists("tomcat_mgr", true)
			creds << protocol_wordlists("tomcat_mgr")
		else
			creds << protocol_wordlists(host.service, true)
			creds << protocol_wordlists(host.service)	
		end
		
		return creds
	end

	def gen_credentials_ssh
		known_defaults = gen_common_shell_creds
		users = []
		users.concat(%w{guest root admin user})
		passes = []
		passes.concat(gen_common_passwords)
		passes.concat(%w{password admin pass pass1 password1 toor})
		creds = []
		creds.concat(gen_username_based_passwords(users))
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		creds = finalize_credentials(creds)
		creds = (known_defaults + creds).flatten.uniq
		return creds
	end

	def gen_credentials_mysql
		users = []
		users.concat(%w{mysql admin root})
		passes = []
		passes << "" # Always try blanks
		passes.concat(%w{password pass pass1})
		passes.concat(gen_common_passwords)
		creds = []
		creds.concat(gen_username_based_passwords(users))
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_mssql
		users = []
		users.concat(%w{sa mon_user probe repl_publisher repl_subscriber})
		passes = []
		passes << "" # Always try blanks
		passes.concat(%w{blank pass password pass pass1})
		passes.concat(gen_common_passwords)
		creds = []
		creds.concat(gen_username_based_passwords(users))
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		# Special accounts just for MSSQL
		creds << "entldbdbo dbopswd"
		creds << "ARAdmin AR#Admin#"
		creds << "entldbreader rdrpswd"
		# Siemens default password, used by malware 2010-07-15
		# http://www.google.com/search?q=2WSXcder
		creds << "WinCCConnect 2WSXcder"
		# MSSQL usernames are case insensitive. Sometimes passwords
		# are case insensitive too, but it's hard to tell a priori
		# (case sensitivity can be toggled by the administrator).
		creds = fix_credentials_case(creds,ucase=true,pcase=false)
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_db2
		users = []
		users.concat(%w{db2inst1 dasusr1 db2fenc1 admin})
		passes = []
		passes.concat(%w{db2inst1 dasusr1 db2fenc1 db2pass db2pw db2password admin})
		passes.concat(gen_common_passwords)
		creds = []
		creds.concat(gen_username_based_passwords(users))
		creds << "db2cae2 db2cae" # Old, DB7 account
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_postgres
		users = []
		users.concat(%w{postgres root admin})
		passes = []
		passes << "" # Always try blanks
		passes.concat(%w{blank pass password pass pass1})
		passes.concat(gen_common_passwords)
		creds = []
		creds.concat(gen_username_based_passwords(users))
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_oracle
		creds = []
		creds << "scott tiger"
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_http
		users  = []
		passes = []
		passes.concat(gen_common_passwords)
		creds  = []

		# Find devices which TELL us the default password
		myworkspace.services.find_all_by_name("http").each do |s|
			next if not (s.info and s.info.length > 0)
			if s.info =~ /default is ([a-z0-9A-z]+)/i
				creds << "anyuser #{$1}"
				next
			end
			if s.info =~ /Default password.([a-z0-9A-z]+)/i
				creds << "anyuser #{$1}"
				next
			end
		end

		# Incorporate the userpass file.
		http_wordlist_file = File.join(Msf::Config.install_root, "data", "wordlists", "http_default_userpass.txt")
		if File.exist?(http_wordlist_file)
			creds << File.open(http_wordlist_file) {|f| f.read(f.stat.size)}.split(/\n/)
		end

		# Incorporate the users file.
		http_wordlist_file = File.join(Msf::Config.install_root, "data", "wordlists", "http_default_users.txt")
		if File.exist?(http_wordlist_file)
			users.concat File.open(http_wordlist_file) {|f| f.read(f.stat.size)}.split(/\n/)
		end

		# Incorporate the passes file.
		http_wordlist_file = File.join(Msf::Config.install_root, "data", "wordlists", "http_default_pass.txt")
		if File.exist?(http_wordlist_file)
			passes.concat File.open(http_wordlist_file) {|f| f.read(f.stat.size)}.split(/\n/)
		end

		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}

		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_tomcat
		users = []
		users.concat(%w{admin manager role1 root tomcat both})
		passes = %w{admin root tomcat}
		passes.concat(%w{password pass pass1 password1 admin})
		passes.concat(gen_common_passwords)
		creds = []
		creds.concat(gen_username_based_passwords(users))

		# Incorporate the userpass file.
		tomcat_wordlist_file = File.join(Msf::Config.install_root, "data", "wordlists", "tomcat_mgr_default_userpass.txt")
		if File.exist?(tomcat_wordlist_file)
			creds << File.open(tomcat_wordlist_file) {|f| f.read(f.stat.size)}.split(/\n/)
		end

		# Strangely, Tomcat usernames are always case sensitive.
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_vnc
		creds = []

		# Incorporate the userpass file.
		vnc_wordlist_file = File.join(Msf::Config.install_root, "data", "wordlists", "vnc_passwords.txt")
		if File.exist?(vnc_wordlist_file)
			creds << File.open(vnc_wordlist_file) { |f| f.read(f.stat.size) }.split(/\n/).map { |p| " " + p }
		end

		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_telnet(flavor=nil)
		known_defaults = gen_common_shell_creds
		users = []
		users.concat(%w{guest root user toor lp admin})
		passes = []
		passes.concat(%w{password pass pass1 password1 admin})
		passes.concat(gen_common_passwords)
		creds = []
		creds.concat(gen_username_based_passwords(users))
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}

		if flavor == :cisco
			known_defaults.unshift("cisco sanfran")
			known_defaults.unshift("cisco pix")
			known_defaults.unshift("pix cisco")
			known_defaults.unshift("cisco cisco")
		end

		creds = (known_defaults + creds).flatten.uniq
		return creds
	end

	def gen_credentials_ftp(flavor=nil)
		known_defaults = gen_common_ftp_creds
		users = []
		users.concat(%w{guest root user toor lp admin})
		passes = []
		passes.concat(%w{password pass pass1 password1 admin 1234})
		passes.concat(gen_common_passwords)
		creds = []
		creds.concat(gen_username_based_passwords(users))
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}

		creds = (known_defaults + creds).flatten.uniq
		return creds
	end

	def gen_credentials_smb(flavor=nil)
		users =  []
		passes = []
		creds =  []
		case flavor
		when nil
			users.concat(%w{administrator admin backup})
			passes.concat(%w{admin backup})
			passes.concat(gen_common_passwords)
			creds.concat(gen_username_based_passwords(users))
			users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
			creds = finalize_credentials(creds)
			return creds.flatten.uniq
		when :microsoft
			users << "guest"
			creds << "Administrator hpinvent"
			creds << "Administrator admin"
			creds << "Administrator backup"
		when :apple
			users.delete('nobody')
		when :samba
			creds.concat(gen_common_shell_creds)
		end
		users.each {|u| passes.each {|p| creds << "#{u} #{p}" }}
		creds = fix_credentials_case(creds,ucase=true,pcase=false)
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def gen_credentials_known_users(known_creds,host)
		users = []
		users = known_creds.map{|up| up.split(/[\s]+/, 2).first.to_s} unless known_creds.empty?
		users.concat(gen_found_user_list)
		if NO_BLANK_USERNAMES.include? host.service
			users = users.reject { |u| u.nil? || u.empty? }
		end
		creds = []
		creds = gen_username_based_passwords(users.compact.uniq)
		creds = finalize_credentials(creds)
		return creds.flatten.uniq
	end

	def collect_machine_names()
		machines = []
		myworkspace.hosts.each do |h|
			# next unless host_allowed?(h)
			next unless h.name.kind_of? String
			next if h.name.nil? || h.name.empty?
			hparts = h.name.split(".")
			next if hparts.size < 1
			hostname = hparts.first.to_s.downcase
			machines << hostname
			components = hostname.split(/[\x2d\x5f]/)
			components.each do |c|
				next unless c.kind_of? String
				next if c.size <= 1
					machines << c
			end
		end
		machines = transform_by_suffix_and_case(machines)
		return machines.uniq
	end

	def transform_by_suffix_and_case(wordlist)
		return machines unless wordlist.kind_of? Array
		transformed_wordlist = []
		wordlist.each do |w|
			# Just the machine name
			transformed_wordlist << w
			# Append a number
			transformed_wordlist << "#{w}1" unless w =~ /[0-9]$/
			transformed_wordlist << "#{w}123" unless w =~ /[0-9]$/
			transformed_wordlist << "#{w}1234" unless w =~ /[0-9]$/
			# Append punctuation
			transformed_wordlist << "#{w}!" unless w =~ /!$/
			transformed_wordlist << "#{w}!" unless w =~ /\$$/
			transformed_wordlist << "#{w}!" unless w =~ /\.$/
			# Shift case
			transformed_wordlist << w.to_s.downcase unless w == w.to_s.downcase
			transformed_wordlist << w.upcase unless w == w.upcase
		end
		return transformed_wordlist
	end

	# If we're given a pw_list, use that. Otherwise,
	# generate some common passwords off of our own
	# lists.
	# Note that every new user added is going to have
	# a geometric effect on how many total guesses we're going
	# to make.
	def gen_credentials_baseline(user_list=nil,pass_list=nil)
		users = user_list || %w{admin user root guest finance info sales it}
		if pass_list
			passes = pass_list
		else
			passes = gen_common_passwords
			passes.concat(collect_machine_names())
			passes = passes.uniq
		end
		creds = []
		creds = gen_username_based_passwords(users)
		users.each { |u| passes.each { |p| creds << "#{u} #{p}" } }
		return creds.flatten.uniq
	end

	def gen_common_passwords
		COMMON_PASSWORDS
	end

	def gen_other_common_passwords
		pass_file = File.join(Msf::Config.install_root, "data", "wordlists", "unix_passwords.txt")
		if File.exist?(pass_file)
			passes = File.open(pass_file) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		return passes.uniq
	end

	def just_uniq_passwords(creds)
		creds.map { |e|
			" " + e.split(/[\s]+/, 2)[1].to_s
		}.uniq
	end

	def just_uniq_users(creds)
		creds.map { |e|
			e.split(/[\s]+/, 2).first.to_s + " "
		}.uniq
	end

end
