module Pro
module RPC
module Stubs
module Reports

	def rpc_report_delete_data_file(report_id)
		r = _find_report(report_id)
		::FileUtils.rm_f(r.path)
		{ 'status' => 'success'}
	end

	# delete_file seems a lot easier to remember than delete_data_file.
	alias :rpc_report_delete_file :rpc_report_delete_data_file

	# Note that these status codes aren't currently being
	# checked by anything, since in the end, ReportTemplateTask#rpc_call
	# will always return nil since we don't get a task_id. This complicates
	# automated testing, maybe making it impossible. IOW, XXX: fix this.
	def rpc_start_report_template(conf={})
		::ActiveRecord::Base.connection_pool.with_connection {
			conf ||= {}
			wspace = conf['workspace'] || self.framework.db.workspace.name
			username = conf['username'] || "unknown"
			report_name = conf['report_name'] || "Custom Report Template"
			rt_path = conf['rt_path'] 

			w = _find_workspace(wspace)

			if not rt_path
				error(500, "Invalid Report Template Path")
			end

			template_record = Mdm::ReportTemplate.find_or_create_by_path(rt_path)
			template_record[:workspace_id] = w[:id]
			template_record[:created_by]   = username
				template_record[:name]         = report_name
			template_record[:updated_at] = Time.now.utc
			template_record.save!
			{ 'status' => 'success', 'report_id' => template_record[:id]}
		}
	end

	def rpc_report_template_delete_file(report_id)
		::ActiveRecord::Base.connection_pool.with_connection {
			r = Mdm::ReportTemplate.find(report_id)
			error(500, "Invalid Template ID") if not r
			::FileUtils.rm_f(r.path)
			{ 'status' => 'success'}
		}
	end
	
	def rpc_report_list(wspace)
		w = _find_workspace(wspace)
		r = {}
		
		w.reports.each do |x|
			r[x.id] = _report_to_hash(x)
		end
		
		r
	end
	
	def rpc_report_download(report_id)
		x = _find_report(report_id)
		r = _report_to_hash(x)
		r[:data] = ''

		if x.path and ::File.exist?(x.path)
			::File.open(x.path, "rb") do |fd|
				r[:data] << fd.read(fd.stat.size)
			end
		end
		
		r	
	end

	def rpc_report_download_by_task(task_id)
		::ActiveRecord::Base.connection_pool.with_connection {
			x = ::Mdm::Report.find_by_task_id(task_id.to_i)
		
			error(500, "No Report for Task ID") if not x
			r = _report_to_hash(x)
			r[:data] = ''

			if x.path and ::File.exist?(x.path)
				::File.open(x.path, "rb") do |fd|
					r[:data] << fd.read(fd.stat.size)
				end
			end
			r	
		}
	end
	
private

	def _find_report(report_id)
		::ActiveRecord::Base.connection_pool.with_connection {
			r = ::Mdm::Report.find(report_id.to_i)
			error(500, "Invalid Report ID") if not r
			r
		}
	end
	
	def _report_to_hash(x)
		r = {}
		r[:id] = x.id
		r[:workspace] = x.workspace.name
		r[:created_by] = x.created_by
		r[:rtype] = x.rtype
		r[:path] = x.path
		r[:size] = ::File.size(x.path) rescue 0
		r[:options] = x.options
		r[:created_at] = x.created_at.to_i
		r[:updated_at] = x.updated_at.to_i
		r[:downloaded_at] = x.downloaded_at.to_i
		r
	end

end
end
end
end
