module Pro
module RPC
module Stubs
module Tasks

	# List all live tasks on all workspaces
	# XXX: rewrite to use the database
	def rpc_task_list
		ret = {}
		self.tasks.each_pair do |task_id, task|
			ret[task_id] = {
				'status'      => task.status.to_s,
				'error'       => task.error || '',
				'created_at'  => task.created_at.to_i,
				'progress'    => task.progress,
				'description' => task.description,
				'info'        => task.info,
				'workspace'   => task.workspace,
				'username'    => task.username,
				'result'      => task.result || '',
				'path'        => task.path || '',
				'size'        => ( ::File.size(task.path) rescue 0 )
			}
		end
		ret
	end

	# Return the status for a specific task
	# XXX: rewrite to use the database
	def rpc_task_status(task_id)
		ret = {}

		task_id = task_id.to_s
		task = self.tasks[task_id]
		if(task)
			ret[task_id] = {
				'status'      => task.status.to_s,
				'error'       => task.error || '',
				'created_at'  => task.created_at.to_i,
				'progress'    => task.progress,
				'description' => task.description,
				'info'        => task.info,
				'workspace'   => task.workspace,
				'username'    => task.username,
				'result'      => task.result || '',
				'path'        => task.path || '',
				'size'        => ( ::File.size(task.path) rescue 0 )
			}
		else
			ret[task_id] = {'status' => 'invalid', 'error' => '' }
		end
		ret
	end

	# Forcibly stop a specific task
	def rpc_task_stop(task_id)
		task_id = task_id.to_s
		ret = { 'task' => task_id }
		if(self.tasks[task_id])
			self.tasks[task_id].stop if self.tasks[task_id]
			self.tasks.delete(task_id)
			ret['status'] = 'stopped'
		else
			ret['status'] = 'invalid'
		end

		ret
	end

	# Delete the data file corresponding to a task
	# XXX: Fix callers
	def rpc_task_delete_log(task_id)
		r = _find_task(task_id)
		::FileUtils.rm_f(r.path)
		{ 'status' => 'success'}
	end


	# Download the task log for a specific task
	def rpc_task_log(task_id)
		x = _find_task(task_id)
		r = _task_to_hash(x)

		r[:log] = ''
		if x.path and ::File.exist?(x.path)
			::File.open(x.path, "rb") do |fd|
				r[:log] << fd.read(fd.stat.size)
			end
		end
		r
	end

	def rpc_start_discover(conf={})
		_start_module_task(conf, "pro/discover", "Discovering")
	end

	def rpc_start_webscan(conf={})
		_start_module_task(conf, "pro/webscan", "Scanning")
	end

	def rpc_start_webaudit(conf={})
		_start_module_task(conf, "pro/webaudit", "Auditing")
	end

	def rpc_start_bruteforce(conf={})
		_start_module_task(conf, "pro/bruteforce", "Bruteforcing")
	end

	def rpc_start_exploit(conf={})
		_start_module_task(conf, "pro/exploit", "Exploiting")
	end

	def rpc_start_websploit(conf={})
		_start_module_task(conf, "pro/websploit", "Exploiting")
	end

	def rpc_start_import(conf={})
		_start_module_task(conf, "pro/import", "Importing")
	end

	def rpc_start_import_creds(conf={})
		_start_module_task(conf, "pro/import_creds", "Importing Creds")
	end

	def rpc_start_cleanup(conf={})
		_start_module_task(conf, "pro/cleanup", "Cleaning Up")
	end
	
	def rpc_start_upgrade_sessions(conf={})
	 	_start_module_task(conf, "pro/upgrade_sessions", "Upgrading Sessions")
  end

	def rpc_start_collect(conf={})
		_start_module_task(conf, "pro/collect", "Collecting")
	end

	def rpc_start_single(conf={})
		_start_module_task(conf, "pro/single", "Launching")
	end

	def rpc_start_download(conf={})
		_start_module_task(conf, "pro/download", "Downloading")
	end

	def rpc_start_upload(conf={})
		_start_module_task(conf, "pro/upload", "Uploading")
	end

	def rpc_start_nexpose(conf={})
		_start_module_task(conf, "pro/nexpose", "Nexpose")
	end

	def rpc_start_report(conf={})
		_start_module_task(conf, "pro/report", "Reporting")
	end

	def rpc_start_campaign(conf={})
		_start_module_task(conf, "pro/campaign", "Campaign")
	end

	# Start a new-style social engineering campaign
	def rpc_start_se_campaign(conf={})
		_start_module_task(conf, "pro/social_engineering/campaign_commander", "Social Engineering Campaign")
	end

	def rpc_start_nexpose_asset_group_push(conf={})
		_start_module_task(conf, "pro/nexpose_asset_group_push", "Nexpose Asset Group Push")
	end

	def rpc_start_nexpose_exception_push(conf={})
		_start_module_task(conf, "pro/nexpose_exception_push", "Nexpose Exception Push")
	end

	def rpc_generate_exe(conf={})
		_start_module_task(conf, "pro/phish_generate_exe", "Generate Executable")
	end

	def rpc_portable_file_generate(conf={})
		_start_module_task(conf, "pro/social_engineering/portable_file_generate", "Generate File")
	end

	def rpc_start_tunnel(conf={})
		_start_module_task(conf, "pro/tunnel", "Tunneling")
	end

	def rpc_start_listener(conf={})
		_start_module_task(conf, "pro/listener", "Listening")
	end

	def rpc_start_replay(conf={})
		_start_module_task(conf, "pro/replay", "Replaying")
	end

private

	def _find_task(task_id)
		::ActiveRecord::Base.connection_pool.with_connection {
			r = ::Mdm::Task.find(task_id.to_i)
			error(500, "Invalid Task ID") if not r
			r
		}
	end

	def _task_to_hash(x)
		r = {}
		r[:id] = x.id
		r[:workspace] = x.workspace.name
		r[:created_by] = x.created_by
		r[:module] = x.module
		r[:completed_at] = x.completed_at.to_i
		r[:path] = x.path
		r[:size] = ::File.size(x.path) rescue 0
		r[:info] = x.info
		r[:description] = x.description
		r[:progress] = x.progress
		r[:options] = x.options
		r[:error] = x.error
		r[:created_at] = x.created_at.to_i
		r[:updated_at] = x.updated_at.to_i
		r[:result] = x.result
		r[:module_uuid] = x.module_uuid
		r
	end


  # conf - config hash passed to RPC call
  # mod  - path to module file (normal MSF loading)
  # desc - description of the module being used by the task
	def _start_module_task(conf,mod,desc)
		::ActiveRecord::Base.connection_pool.with_connection {

			conf    ||= {}
			wspace    = conf['workspace'] || self.framework.db.workspace.name
			username  = conf['username']  || "unknown"
	
			workspace = self.framework.db.find_workspace(wspace)
			error("500", "Invalid Workspace") if not workspace
	
			trecord   = Mdm::Task.create(
				:workspace_id => workspace[:id],
				:created_by   => username,
				:module       => mod,
				:description  => desc,
				:options      => conf,
				:progress     => 0
			)

			oname = "task_#{mod.gsub("/", ".")}_#{trecord[:id]}.txt"
			opath = File.expand_path(File.join(_base_directory('task'), oname))

			trecord.path = opath
			trecord.save!
	
			output_file = Rex::Ui::Text::Output::File.new(trecord.path)

			disco = Proc.new do |task,args|

				Thread.current.priority = -10

				m = self.framework.auxiliary.create(mod)
				if(not m)
					raise RuntimeError, "Module #{mod} could not be loaded"
				end

				m = self.framework.modules.reload_module(m)
				if(not m)
					raise RuntimeError, "Module #{mod} could not be reloaded"
				end
	
				m.init_ui(nil, output_file)
	
				m[:conf] = conf
				m[:task] = task
				m[:args] = args
		
				# Not sure what context this Proc will end up running in, so go ahead and
				# make sure it checks in the database connection when it's done.
				::ActiveRecord::Base.connection_pool.with_connection {
					Mdm::Task.update(trecord[:id], { :module_uuid => m.uuid } )
				}
	
				if(conf['ips'])
					m.datastore['RHOSTS'] = conf['ips'].join(" ")
				end
	
				m.datastore['TimestampOutput'] = true
				m.datastore['WORKSPACE'] = conf['workspace']
				m.datastore['PROUSER']   = conf['username']
				conf.each_key do |k|
					if(k =~ /^DS_/)
						m.datastore[k.sub(/^DS_/, '')] = conf[k]
					end
				end
	
				m.run_simple do |ref|
					# Override with our replicant instance
					m = task.module = ref
					# Handle task stop events cleanly
					task.on_stop = Proc.new do |task|
						task.module.stop_task if task.module.respond_to?('stop_task')
					end					
				end

				if m.error
					# Set the task-level error
					trace = []
					m.error.backtrace.each do |line|
						break if line =~ /lib.msf.base.simple.auxiliary.rb/
						trace << line.gsub!(/.*\/pro\//, '/pro/')
					end
					task.error = "Module Exception: #{m.error} #{trace.join("\n")}"
				end
			end
	
			tid = self.tasks.create(trecord, disco)
			self.tasks[tid].description = conf['description'] || desc
			self.tasks[tid].workspace   = conf['workspace']   || ''
			self.tasks[tid].username    = conf['username']    || ''
		
			ret = self.framework.esnecil_invalid?
			if ret and mod !~ /\/(cleanup|collect|report)/
				self.tasks[tid].error = ret
				self.tasks[tid].proc  = Proc.new{|*args|}
			end
			
			self.tasks[tid].start
			Thread.current.priority = 10
			{ 'task_id' => tid }
		}
	end

end
end
end
end

