module Pro

# Basic hash container for managing tasks
class TaskContainer < Hash

	attr_accessor :monitor

	def initialize(*args)
		super(*args)
		start_monitor
		@tcnt = 0
	end

	def start(proc, args={})
		tid = create(proc, args)
		self[tid].start
		tid
	end

	def create(record, proc, args={})
		i = record[:id]
		t = ProTask.new(proc, args)
		t.task_id = i.to_s
		t.record  = record
		t.path    = record.path
		self[t.task_id] = t
		t.task_id
	end

	def start_monitor

		self.monitor = Rex::ThreadFactory.spawn("ProRPCTaskMonitor", true) do

			while true

				begin
					self.keys.each { |t| self[t].sync }
					self.keys.each { |t| self.delete(t) if t.status == "done" }
				rescue ::Exception => e
				end

				Kernel.select(nil, nil, nil, 1.0)
			end
		end
	end
end

# Wrapper object for the task
class ProTask
	attr_accessor :proc, :args, :thread, :task_id, :error, :progress, :created_at, :completed_at
	attr_accessor :description, :info, :module, :on_stop, :modname, :record
	attr_accessor :workspace, :username, :result, :path

	def initialize(proc, args)
		self.proc        = proc
		self.args        = args
		self.progress    = 0
		self.created_at  = ::Time.now.utc
		self.info        = ""
		self.description = ""
		self.result      = ""
	end

	def status
		if(not self.thread)
			return 'new'
		end

		if(not self.thread.alive?)
			return 'done'
		end

		return 'running'
	end

	# Stop the current task
	def stop
		if self.on_stop
			self.on_stop.call(self) rescue nil
		end

		# Tell the module to cleanup first
		if self.module
			self.module.cleanup rescue nil
			Kernel.select(nil, nil, nil, 1.0)
		end

		# The thread can take a second to die
		if self.thread
			self.thread.kill
			while self.status == 'running'
				Kernel.select(nil, nil, nil, 0.10)
			end
		end

		self.cleanup
		self.sync
	end

	# Sync the database object to the current task info
	def sync
	::ActiveRecord::Base.connection_pool.with_connection {

		# If there's no matching DB record, then it's a ProConsole task.
		db_task = Mdm::Task.find_by_id(self.record[:id])
		return unless(db_task and db_task.id == self.record[:id])

		done = false
		if ! self.record.completed_at and self.status == 'done'
			done = ::Time.now.utc
		end

		if done or
			(self.record.info != self.info) or
			(self.record.description != self.description) or
			(self.record.progress != self.progress) or
			(self.record.error != self.error) or
			(self.record.result != self.result)

			# Update just the attributes this code monitors
			update_info = {
				:info        => self.info,
				:description => self.description,
				:progress    => self.progress,
				:error       => self.error,
				:result      => self.result
			}

			# Mark the compeletion time if necessary
			update_info[:completed_at] = ::Time.now.utc if done

			# Update just the modified fields (dont conflict with UI DB fields like :settings)
			Mdm::Task.update(self.record[:id], update_info)

			# Update the record with the new data
			self.record = Mdm::Task.find(self.record[:id])
		end
	}
	end

	def start
		self.thread = Rex::ThreadFactory.spawn("ProRPCTask-#{self.task_id}", false) do
			begin
				Thread.current[:task] = self
				self.proc.call(self, self.args)
				self.progress = 100
			rescue ::Exception => e
				self.error = "#{e.class} #{e}"
				if self.module and self.module.user_output
					 self.module.user_output.print_error("Task Exception: #{e.class} #{e} #{e.backtrace}")
				end
			end

			self.cleanup
			self.completed_at = ::Time.now.utc
			self.sync
		end
	end

	def cleanup
		self.module.user_output.close if (self.module and self.module.user_output)
		self.module.user_output = nil if (self.module and self.module.user_output)
		self.module       = nil
		GC.start
	end

end
end

