#!/usr/bin/env ruby


$:.unshift(File.expand_path(File.join(File.dirname(__FILE__))))
require 'tool_dependency'


def usage
	$stderr.puts "Usage: #{$0} <Project Name> <Output Filename> <PDF,XML,ZIP,WORD>"
	$stderr.puts " Active Projects:"
	@pro.call("pro.workspaces").each_pair do |k,v|
		$stderr.puts " \t#{k}"
	end
	$stderr.puts ""

	exit(1)
end

@pro = Pro::Client.new(
	'ServerHost' => '127.0.0.1',
	'ServerPort' => 3790,
	'URL'  => '/api/1.0',
	'SSL'  => true
)

wspace = ARGV.shift || usage()
fname  = ARGV.shift || usage()
rtype  = ARGV.shift || "PDF"
user   = @pro.call("pro.default_admin_user")['username']

task = @pro.start_report(
	'DS_WHITELIST_HOSTS'        => "",
	'DS_BLACKLIST_HOSTS'        => "",
	'workspace'                 => wspace,
	'username'                  => user,
	'DS_MaskPasswords'          => false,
	'DS_IncludeTaskLog'         => false,
	'DS_JasperDisplaySession'   => true,
	'DS_JasperDisplayCharts'    => true,
	'DS_LootExcludeScreenshots' => false,
	'DS_LootExcludePasswords'   => false,
	'DS_JasperTemplate'         => "msfxv3.jrxml",
	'DS_REPORT_TYPE'            => rtype.upcase,
	'DS_UseJasper'              => true,
	'DS_UseCustomReporting'     => true,
	'DS_JasperProductName'      => "Metasploit Pro",
	'DS_JasperDbEnv'            => "production",
	'DS_JasperLogo'             => '',
	'DS_JasperDisplaySections'  => "1,2,3,4,5,6,7,8",
	'DS_EnablePCIReport'        => true,
	'DS_EnableFISMAReport'      => true,
	'DS_JasperDisplayWeb'       => true,
)


if not task['task_id']
	$stderr.puts "[-] Error generating the report: #{task.inspect}"
	exit(0)
end

puts "[*] Report is generating with Task ID #{task['task_id']}..."
while true
	select(nil, nil, nil, 0.50)
	stat = @pro.call("pro.task_status", task['task_id'])
	if stat['status'] == 'invalid'
		$stderr.puts "[-] Error checking task status"
		exit(0)
	end

	info = stat[ task['task_id'] ]

	if not info
		$stderr.puts "[-] Error finding the task"
		exit(0)
	end

	if info['status'] == "error"
		$stderr.puts "[-] Error generating report: #{info['error']}"
		exit(0)
	end

	break if info['progress'] == 100
end

report = @pro.call('pro.report_download_by_task', task['task_id'])
if report and report['data']
	::File.open(fname, "wb") do |fd|
		fd.write(report['data'])
	end
	$stderr.puts "[-] Report saved to #{::File.expand_path(fname)}"
else
	$stderr.puts "[-] Error downloading report: #{report.inspect}"
end

