#!/usr/bin/env ruby
require 'fileutils'
require 'pathname'

#
# Load our own library path and the standard Metasploit Framework path
#
engine_root_path = File.dirname(__FILE__) + '/../engine/'
engine_root_pathname = Pathname.new(engine_root_path).expand_path
engine_lib_path = engine_root_pathname.join('lib').to_s
$LOAD_PATH.unshift(engine_lib_path)

require 'metasploit/pro/engine'

Metasploit::Pro::Engine.initialize!

# Load any required hooks/monkeypatches
::Pro::Hooks::Loader.start(Metasploit::Pro::Engine.framework)

# needs to be called as early as possible to stop the UI from booting before the migrations are run
Metasploit::Pro::UI.initialize_database do
  # Load gem deps they share
  carrierwave_initializer_pathname = Metasploit::Pro::UI.root.join('config', 'initializers', 'carrierwave')
  carrierwave_initializer_path = carrierwave_initializer_pathname.to_s
  require carrierwave_initializer_path

  #
  # Load specific patches for system libraries
  #
  if Metasploit::Pro::Engine.configuration.platform.win32?
    require 'win32/registry'
    require 'patches/win32_registry'
  end

  # Pro API
  # Monkey patches Msf::Config
  require 'pro/config'

  # Pro Mixins
  require 'pro/mixins'

  Metasploit::Pro::Engine.expand_file_limit
  Metasploit::Pro::Engine.configure_environment
  Metasploit::Pro::Engine.parse_arguments

  # parse arguments before ensuring root user because root user is only necessary in production environment
  Metasploit::Pro::Engine.ensure_root_user!

  # Permissive umask for task logs
  File.umask(022)

  Metasploit::Pro::Engine.synchronize_token
  Metasploit::Pro::Engine.run_flash_policy_server
  Metasploit::Pro::Engine.write_pid_file

  #
  # Trap the TERM signal and exit
  #
  Signal.trap("TERM") do
    exit(0)
  end

  $0 = 'prosvc'

  if Metasploit::Pro::Engine.configuration.ssl?
    ssl_label = 'SSL'
  else
    ssl_label = 'NO SSL'
  end

  server_configuration = Metasploit::Pro::Engine.configuration.server
  server_label = "#{server_configuration.host}:#{server_configuration.port}"

  $stdout.sync = true unless ENV['RAILS_ENV'] == 'production'

  $stdout.puts "[*] Metasploit Pro Service (#{server_label} (#{ssl_label}))"
  $stdout.puts "[*] URI: #{Metasploit::Pro::Engine.configuration.uri}"

  Metasploit::Pro::Engine.merge_meterpreter_extensions
  Metasploit::Pro::Engine.configure_file_permissions

  #
  # Initialize the NginX server
  #
  nginx = Pro::Nginx.new
  nginx.start()

  Metasploit::Pro.load_modules(Metasploit::Pro::Engine.framework)
  Metasploit::Pro::Engine.connect_to_database
end

# create the profile if it does not exist so that the smtp configuration can be loaded from it
Metasploit::Pro::Engine.find_or_create_default_profile!
# Load smtp configuration
Mdm::Profile.load_latest_smtp_configuration

# Enable timestamps on all output
Metasploit::Pro::Engine.framework.datastore['TimestampOutput'] = 'true'


## DEVELOPMENT-only helper stuff
if Metasploit::Pro::Engine.configuration.environment.development?
  require 'pry'
end


# Clean up any old / defunct tasks
Metasploit::Pro::Engine.update_defunct_tasks!
SocialEngineering::CampaignsCleaner.cleanup

$f = Metasploit::Pro::Engine.framework
