jQueryInWindow ($) ->
  QUICK_PENTEST_URL = '/quick_pentest'
  QUICK_PENTEST_IS_FIRST_TASK_READY_URL = '/quick_pentest/first_task_ready'
  QUICK_PENTEST_REDIRECT_AFTER_SUCCESS_URL = '/quick_pentest/redirect_after_success'

  class @QuickPentestModal extends @TabbedModalView
    initialize: ->
      super
      @setTitle 'Quick Penetration Test'
      @setDescription 'Quickly set up a test that automatically gathers '+
                      'information about the target network, launches attacks '+
                      'against the targets, and builds a report of test findings.'
      @setTabs [
        {name: 'Target Settings'},
        {name: 'Configure Scan'},
        {name: 'Run Exploits', checkbox: true},
        {name: 'Generate Report', checkbox: true}
      ]
      @setButtons [
        {name: 'Cancel', class: 'close'},
        {name: 'Start Test', class: 'btn primary'}
      ]
      # immediately send an AJAX request to grab the form
      @loadForm()
      # move #modals to be last child of <body>
      $("#modals").appendTo($('body'))

    showLoadingDialog: ->
      @_loaderDialog = $('<div class="loading">').dialog(
        modal: true
        closeOnEscape: false
        title: 'Loading...'
      )
      @_loaderDialog.parents('.ui-dialog').addClass('white')
      @_loaderDialog.append($('<p class="dialog-msg center">').text('Loading...'))

    hideLoadingDialog: ->
      @_loaderDialog.dialog('close').remove()

    setDialogTitle: (title) ->
      $('.dialog-msg', @_loaderDialog).text title

    events: _.extend({
      'click a.advanced': 'toggleAdvanced',
      'click input#tab_run_exploits': 'toggleExploits',
      'click input#tab_generate_report': 'toggleGenerateReport',
      'click a.exclude_toggle': 'toggleExcludeIPs',
      'change input[name="quick_pentest[scan_type]"]': 'scanTypeChanged',
      'change select#quick_pentest_report_type': 'reportTypeChanged',
      'click .modal-actions .btn.primary': 'startScanClicked',
      'change select': 'someInputChanged',
      'change input, textarea': 'someInputChanged',
      'keydown input, textarea': 'someInputChanged'
    }, TabbedModalView.prototype.events)

    # ivar to contain step_name, url pairs: [[:create_project, '/quick_pentest/c_p']]
    # will be loaded in formLoadedSuccessfully
    _steps: []

    # load the entire QuickPentest form into our @content() div
    loadForm: ->
      # disable the tabs until we're loaded.
      _.each @tabs(), (t) -> $(t).addClass('ui_disabled')
      # show loader in content pane
      @content().addClass('loading').html('')
      $.ajax
        url: QUICK_PENTEST_URL
        method: 'get'
        success: @formLoadedSuccessfully
        error: ->
          # display error message
          @content().removeClass('loading').addClass('error')
            .text("Unable to reach Metasploit Pro. Please ensure you" +
            " are connected and the Metasploit Pro service is running.")
          # if modal still present, retry connection in 5s
          _.delay((=>@content().is(':visible') && @loadForm()), 5000)

    # generic handler for toggling the div next to <a.advanced>
    toggleAdvanced: (e) =>
      e.preventDefault()
      $(e.currentTarget).toggleClass('not_advanced')
      $('~div.advanced', e.currentTarget).toggle()

    toggleExploits: (e) =>
      checked = $(e.currentTarget).is(':checked')
      $('h3.enabled>span', @$modal).removeClass('disabled enabled')
      $('.configure_exploits h3.enabled>span', @$modal).removeClass('disabled enabled')
      if checked
        $('.configure_exploits h3.enabled>span', @$modal).text("enabled").addClass('enabled')
        # toggle input value for report_enabled (hidden)
        $("[name='quick_pentest[exploit_enabled]']", @$modal).attr('value', '1')
      else
        $('.configure_exploits h3.enabled>span', @$modal).text("disabled").addClass('disabled')
        # toggle input value for report_enabled (hidden)
        $("[name='quick_pentest[exploit_enabled]']", @$modal).removeAttr('value')

    # checkbox on tab for report generation
    toggleGenerateReport: (e) =>
      checked = $(e.currentTarget).is(':checked')
      $('.generate_report h3.enabled>span', @$modal).removeClass('disabled enabled')
      if checked
        $('.generate_report h3.enabled>span', @$modal).text("enabled").addClass('enabled')
        # toggle input value for report_enabled (hidden)
        $("[name='quick_pentest[report_enabled]']", @$modal).attr('value', '1')
      else
        $('.generate_report h3.enabled>span', @$modal).text("disabled").addClass('disabled')
        # toggle input value for report_enabled (hidden)
        $("[name='quick_pentest[report_enabled]']", @$modal).removeAttr('value')

    # user clicks Exclude IP Addresses...
    toggleExcludeIPs: (e) =>
      e.preventDefault()
      $('.excluded_addresses', @$modal).show()
      $('.exclude_toggle_wrap', @$modal).hide()
      $('.create_project .advanced', @$modal).css(top: '182px')

    # user toggles between Regular Scan, Nexpose Scan, and Import
    scanTypeChanged: (e) =>
      toggle_divs = ['toggle_regular_scan', 'toggle_nexpose_scan', 'toggle_import']
      _.each toggle_divs, (div) -> $(".#{div}", @$modal).hide()
      $(".toggle_#{$(e.target).val()}", @$modal).show()

    reportTypeChanged: (e) =>
      format = $('select#quick_pentest_report_type option:selected', @$modal).val()
      # add some dynamicness to the report form
      $(".report_formats>div", @$modal).hide().removeClass('active_report_option')
      $(".report_sections>div", @$modal).hide().removeClass('active_report_option')
      $(".report_options>div", @$modal).hide().removeClass('active_report_option')
      $(".report_formats #report_format_for_#{format}", @$modal).show().addClass('active_report_option')
      $(".report_sections #report_section_for_#{format}", @$modal).show().addClass('active_report_option')
      $(".report_options #report_option_for_#{format}", @$modal).show().addClass('active_report_option')

    # user clicked the "Primary" submit button.
    # try to do an actual CREATE call.
    startScanClicked: (e) =>
      e.preventDefault()
      @resetErrors()
      $form = $('form', @$modal).first()
      $('.hasErrors', @$modal).hide()
      $page = $('div.content form .page').eq(@_tabIdx) #.addClass('page-loading')
      @showLoadingDialog()
      # disable all the other tabs
      $primaryBtn = $('a.btn.primary', @$modal).addClass('disabled')
      $closeBtn = $('.modal-actions a.close', @$modal).addClass('ui-disabled')

      @submitFormAjax $form,
        url: QUICK_PENTEST_URL
        method: 'POST'
        error: (e) =>
          @hideLoadingDialog()
          $page.removeClass('page-loading')
          $primaryBtn.addClass('disabled')
          $closeBtn.removeClass('ui-disabled')
          @handleErrors.apply(@, arguments)
        success: (data) =>
          # saved! woot
          # $page.removeClass('page-loading')
          # so we've saved the TaskChain. Now we need to wait until Pro
          #   kicks off the first Task, and then we will redirect. So,
          #   poll this endpoint indefinitely
          @setDialogTitle('Waiting for first task to start...')
          taskChainQuery = "?task_chain_id=#{data.task_chain_id}"
          cancelMe = window.setInterval((=>
            $.ajax
              url: QUICK_PENTEST_IS_FIRST_TASK_READY_URL+taskChainQuery
              dataType: 'json'
              success: (data) =>
                if data.success
                  window.clearInterval(cancelMe)
                  window.location = QUICK_PENTEST_REDIRECT_AFTER_SUCCESS_URL+taskChainQuery
          ), 3000)
          

    resetErrors: (scope=@$modal) =>
      $('p.inline-error', scope).remove()
      $('li.error', scope).removeClass('error')

    resetErrorsOnTab: (tabIdx=@_tabIdx) =>
      # broken, need to grab specific 
      @resetErrors $('form.quick_pentest>div.page', @$el).eq(tabIdx)
      # turn asterisk into loading
      @tabAt(tabIdx).find('.hasErrors').hide()

    someInputChanged: (e) =>
      # enable the submit btn
      $('a.btn.primary', @$modal).removeClass('disabled')
      # save into a pvt var for determining whether ESC closes immediately
      @_somethingHasChanged = true

    handleErrors: (errorsHash) =>
      # render the errors on to the formtastic form
      window.eh = errorsHash #todo: remove (for debuggin)
      $('.hasError', @$modal).hide()

      # transition map to figure out which tab a nested model is on.
      # this will probably need to be abstracted into a class var.
      modelsToTabs = {
        workspace: 0,
        import: 1,
        scan_task: 1,
        nexpose_task: 1,
        exploit_task: 2,
        report_task: 3
      }
      # reset the Reports tab
      _.each errorsHash.errors, (modelErrors, modelName) =>
        tabIdx = modelsToTabs[modelName]
        $page = @pageAt(tabIdx)
        if typeof modelErrors == 'string' && $.trim(modelErrors).length > 0
          # one error per modal, due to a limitation in old-school TaskConfig model
          #  that I just can't bring myself to fix
          $tab = @tabAt(tabIdx)
          $tab.find('.hasErrors').show()   # show red *
          $page.find('p.error-desc').remove()
          $page.prepend($('<p>').addClass('error-desc').text(modelErrors)) # dump msg up top
        else if typeof modelErrors == 'object' && _.keys(modelErrors).length > 0
          $page.find('p.error-desc').remove() #  remove possible stale messages
          # when we get an object back, that means we have normal ActiveModal/Formtastic
          #   Errors hashes, hooray! Put the messages where they belong.
          @tabAt(tabIdx).find('.hasErrors').show()
          _.each modelErrors, (attrErrors, attrName) ->
            inputName = "#{modelName}[#{attrName}]"
            $li = $("[name='#{inputName}']", @$modal).parents('li').first().addClass('error')
            _.each attrErrors, (errorMsg) ->
              $("<p>").addClass('inline-error').appendTo($li).text(errorMsg)

    pageAt: (idx) =>
      $('>form>.page', @content()).eq(idx)

    # we load the form from the wicked quick_pentest_controller route
    formLoadedSuccessfully: (html) =>
      return unless @content().is(':visible')
      # update content of modal to show entire class
      @content().removeClass('loading').html(html)
      # re-enable tabs
      _.each @tabs(), (t) -> $(t).removeClass('ui_disabled')
      # toggle checkbox in tab depending on this value
      if $("[name='quick_pentest[report_enabled]']", @$modal).val()
        $("#tab_generate_report", @$modal).attr('checked', true)
      if $("[name='quick_pentest[exploit_enabled]']", @$modal).val()
        $("#tab_run_exploits", @$modal).attr('checked', true)
      # only show the first div
      $('form.quick_pentest>div.page', @$el).hide().first().show()
      @reportTypeChanged() # load the initial selection
      # the FORs of the labels get set incorreclty which fucks up label clicking
      $('#report_format_for_audit ol li label', @$modal).removeAttr('for')
      # load steps into constant
      @_steps = $.parseJSON($('meta[name=steps]', @$modal).attr('content'))
      # init select2 on <select> dropdowns # removed because fuck select2 thats why
      # $('select', @$modal).select2(DEFAULT_SELECT2_OPTS)
      # add the page overlays (todo: move this to template)
      $('div.content form .page', @$modal).each ->
        $(this).append($('<div>').addClass('page-overlay'))
      # todo: punt this out to a library for scripting :FILE inputs
      $('input:file', @$modal).each ->
        $label = $(this).prev()
        $(this).attr('size', '50').css(overflow: 'hidden')
        $p = $('<p>').text('Choose a file')
        $span = $('<span>').text($label.text())
        $label.html('').append($p).append($span)
        $(this).change ->
          path = $(this).val() || ''
          # the W3C spec tells browsers to append "C:\\fakepath\\" in front
          #   of the file. kind of ridiculous. basename that shit.
          path = path.replace(/.*(\\|\/)/g, '')
          $p.text(path)
      # recalculate layout after stack pops
      _.defer @layout
      # auto-select the first input
      _.defer => $('form :text:visible:first', @content()).focus()


    # hook for tab change
    index: (idx) ->
      # run validations on tab we just left
      $form = $('form.quick_pentest', @$el)
      $allTabs = $('form.quick_pentest>div.page', @$el)
      $currTab = $allTabs.eq(idx)
      @_somethingHasChanged = false
      unless @_tabIdx < 0 || idx == @_tabIdx # initial state @_tabIdx=-1
        # clear errors on prev tab
        oldTabIdx = @_tabIdx
        @resetErrorsOnTab(oldTabIdx)
        # show a spinner on the current tab & validate
        $li = $('ul.tabs>li', @$modal).eq(@_tabIdx).addClass('tab-loading')
        $page = $('div.content form .page').eq(@_tabIdx).addClass('page-loading')
        $page.find('p.error-desc').remove() #  remove possible stale messages
        @submitFormAjax $form, {
          noFiles: true # don't push up FILES until the very end :)
          success: =>
            # remove spinner
            $li.removeClass('tab-loading ui-disabled')
            $page.removeClass('page-loading')
            # we should probably force ordering of callbacks here, but since rails
            #   blocks we can pretty much assume they will come back in the right order
            unless $('.hasErrors:visible', @$modal).length > 0
              # if we're the last blocker, enable submission
              $('a.btn.primary', @$modal).removeClass('disabled')
            # highlight? code would go here.
          error: =>
            $li.removeClass('tab-loading ui-disabled')
            $page.removeClass('page-loading')
            $('a.btn.primary', @$modal).addClass('disabled')
            @handleErrors.apply(@, arguments)
        }
      # show/hide the correct tab content
      $allTabs.hide().eq(idx).show()
      super

    submitFormAjax: ($form, opts={}) ->
      # set Rails' _method parameter. defaults to PUT.
      $('input[name=_method]', $form).not(':first').remove()
      $('input[name=_method]', $form).val(opts.method || 'PUT')
      # we don't want to push up FILEs during validations, only on
      #   final submit. it would add too much overhead to the validation calls.
      $('input[name=no_files]', $form).val((opts.noFiles && '1') || '')
      # we only want the VISIBLE options on the report_task page.
      #   otherwise we will end up kicking off 15 reports.
      data = $(':input', $form).not(':file')
        .not('[type=checkbox]:not(:checked)')  # remove unchecked checkboxes (why do we have to do this?)
        .not(".report_formats>div:not(.active_report_option) *")  # strip hidden report options as they
        .not(".report_sections>div:not(.active_report_option) *") #   will fuck us over later
        .not(".report_options>div:not(.active_report_option) *")
      $.ajax
        url: opts['url'] || @_steps[@_tabIdx][1]
        type: 'POST'
        data: data.serializeArray()
        processData: false
        iframe: true
        files: if opts['noFiles'] then $() else $(':file', $form) # see above comment
        dataType: 'json'
        complete: (resp) =>
          try
            data = $.parseJSON(resp.responseText)
          catch e
            alert('Error processing response from server. Please ensure Metasploit Pro is running.')
            return

          if !data || !data['success']
            opts['error'].call(@, data) if opts['error']
          else
            # worked great! redirect to task chain page
            opts['success'].call(@, data) if opts['success']



