jQueryInWindow ($) ->
  class @CampaignSlideFactsView extends CampaignFactsView
    @EMAILS_SENT_VIEW: 0
    @EMAIL_OPENINGS_VIEW: 1
    @VISITS_VIEW: 2
    @FORM_ACTIVITY_VIEW: 3
    @PHISHING_RESULTS_VIEW: 4
    @SESSIONS_VIEW: 5


    initialize: ->
      @hideSubCircles = false
      @showTray = true
      @selIdx = -1
      super
      _.bindAll(this, 'scrollToCircle', 'updateDataTable')
      $(@tabView).bind 'scrollToCircle', @scrollToCircle
      $(@tabView).bind 'scrollToSubCircle', @scrollToSubCircle
      @campaignSummary.bind 'change', @updateDataTable

    dataTableColumns: ->
      columns = [
        { mDataProp: 'email_address', display:   'Email Address' },
        { mDataProp: 'first_name',    display:   'First Name'    },
        { mDataProp: 'last_name',     display:   'Last Name'     }
      ]

      renderPhishingResultLink = (row) =>
        id = row.aData.phishing_result_id
        url = "/workspaces/#{WORKSPACE_ID}/social_engineering/campaigns/#{@campaignSummary.id}/phishing_results/#{id}"
        "<a href='#{url}' target='_blank' data-pr-id='#{id}' class='anonymous'>Anonymous</a>"

      renderHumanTargetLink =  (row) =>
        email_address = row.aData.email_address
        id = row.aData.human_target_id
        url = "/workspaces/#{WORKSPACE_ID}/social_engineering/human_targets/#{id}"
        "<a href='#{url}' target='_blank' data-ht-id='#{id}'>#{email_address}</a>"


      if @selIdx == CampaignSlideFactsView.PHISHING_RESULTS_VIEW
        columns[0]['fnRender'] = (row) =>
          email_address = row.aData.email_address
          if email_address is null
            renderPhishingResultLink(row)
          else 
            renderHumanTargetLink(row)
        columns.push({ mDataProp: 'web_page_name', display: 'Web Page' }) 
        columns.push({ mDataProp: 'created_at', display: 'Submitted on', sType: 'title-string', fnRender: dataTableDateFormat })

      else if @selIdx == CampaignSlideFactsView.EMAIL_OPENINGS_VIEW
        columns[0]['fnRender'] = (row) =>
          renderHumanTargetLink(row)
        columns.push({ mDataProp: 'created_at', display: 'Opened on', sType: 'title-string', fnRender: dataTableDateFormat })

      else if @selIdx == CampaignSlideFactsView.VISITS_VIEW
        columns[0]['fnRender'] = (row) =>
          renderHumanTargetLink(row)
        columns.push({ mDataProp: 'created_at', display: 'Clicked on', sType: 'title-string', fnRender: dataTableDateFormat })

      else if @selIdx == CampaignSlideFactsView.SESSIONS_VIEW 
        columns = [
          { 
            mDataProp: 'id', 
            display: 'Name',
            sWidth: '100px', 
            fnRender: (row) ->
              "<a target='_blank' href='/workspaces/#{WORKSPACE_ID}/sessions/#{row.aData.id}'>Session #{row.aData.id}</a>"
          }
          { mDataProp: 'host_name', display:   'Host' }
          { mDataProp: 'desc', display:   'Description' }
          { mDataProp: 'stype', display:  'Session Type' }
          { mDataProp: 'via_exploit', display:   'Attack Module' }
          { mDataProp: 'os', display:   'OS', sWidth: '150px' }
          { mDataProp: 'platform', display:   'Platform' }
          { mDataProp: 'opened_at', display:   'Opened', sWidth: '150px' }
        ]
      columns
    updateDataTable: ->
      @dataTable.fnReloadAjax() if @dataTable

    template: _.template($('#campaign-facts').html())

    currentCircleUrl: ->
      id = @campaignSummary.get('id')
      "/workspaces/#{WORKSPACE_ID}/social_engineering/campaigns/#{id}/#{@actions[@selIdx]}"

    exportButtonClicked: ->
      window.location.href = "#{@currentCircleUrl()}.csv"

    events:
      _.extend({
        'click .tray .export': 'exportButtonClicked'
      }, CampaignFactsView.prototype.events)
    
    circleClicked: (e) ->
      $circle = $(e.target).parent('.circle')
      $circle = $(e.target) unless $circle.size()
      idx = $circle.parents('.cell').first().prevAll().size()
      @scrollToCircle(e, $circle, idx)

    render: -> # make sure circle at @selIdx stays selected
      super
      if @selIdx > -1
        $circles = $('.circle', @el)
        $circles.each -> $(this).parent('.cell').removeClass('selected')
        $circles.eq(@selIdx).addClass('selected').parent('cell').addClass('selected')
      else # auto select first circle
        $circle = $('.large-circles .cell:not(.hidden-cell) .circle', @el).first()
        @scrollToCircle(null, $circle, $circle.parents('.cell').first().index())

      $reportLink = $('.report-btn a', @el)
      href = $reportLink.attr('href')
      newUrl = href+"?campaign_id=#{@campaignSummary.id}"
      $reportLink.attr('href', newUrl) unless href.match /\?/

    scrollToCircle: (e, circle, idx) -> # overriden
      # determine delay based on if user has to wait for tab view to push over
      currPageClicked = ($(circle).parents().index(@$el) > -1)
      delay = if currPageClicked then 75 else 200
      if idx != @selIdx || !currPageClicked
        @selIdx = idx
        circle = $('.circle', @el)[idx]
        $cell = $(circle).parents('.row > div').first()
        # select the cell at that index
        $circles = $('.circle', @el)
        $circles.each -> $(this).parent('.cell').removeClass('selected')
        $circles.removeClass('selected').eq(idx).addClass('selected')
          .parent('.cell').addClass('selected')
        $('.tray', @el).html('').addClass('loading')
        _.delay (=>
          # page has slid over, now slide down the tray
          return unless @visibleCircles().length > 0
          visibleIdx = @visibleCircles().index($('.circle', $cell))
          left = $cell.position().left + parseInt($cell.css('margin-left'))
          cWidth = parseInt($cell.width())
          totalW = $('.large-circles').width()
          $('.shadow-arrow', @el).css(
            left: (left+cWidth/2-20)/totalW*100+'%', # arrow is 40px wide
            bottom: '-7px',
            top: 'auto'
          )
          $('.tray', @el).css(height: 'auto').show()
          $('.campaign-facts.border-box', @el).css('padding-bottom': 0).removeClass 'rd-shadow'
          $('.shadow-arrow-row', @el).css(height: '26px')
          $('.shadow-arrow,.shadow-arrow-row', @el).show()
          @renderPie(i) for i in [0..@actions.length]
          # columns is the array of column data that we pass as aoColumns to datatables
          columns = @dataTableColumns()
          thead = ''
          thead += "<th>#{col.display}</th>" for col in columns
          # refresh the datatable here
          $('.tray', @el).html(_.template($('#targets-table').html()).call())
          $('.tray table>thead>tr', @el).html(thead)
          @dataTable = $('.tray table', @el).dataTable {
            oLanguage: {
              sEmptyTable: "No data has been recorded."
            },
            bServerSide: true,
            sAjaxSource: @currentCircleUrl(),
            aoColumns: columns,
            bProcessing: true,
            sPaginationType: "full_numbers"
          }
          $('.tray', @el).removeClass('loading')
        ), delay

      else # remove selection
        return null # don't let them deselect anymore!
        # $('.tray', @el).html('').removeClass('loading').css(height: '0')
        # _.delay (=>
        #   $('.shadow-arrow,.shadow-arrow-row', @el).hide()
        #   $('.tray', @el).hide()
        #   $circles = $('.circle', @el)
        #   $circles.each -> $(this).parent('.cell').removeClass('selected')
        #   $circles.removeClass('selected')
        #   $('.campaign-facts.border-box', @el).css('padding-bottom': '14px').addClass 'rd-shadow'
        #   @selIdx = -1
        # ), delay
