jQuery ($) ->
  $(document).ready ->
    # web-template body should enable syntax highlighting
    editor = CodeMirror.fromTextArea(document.getElementById("web_template_body"), {
      lineNumbers: true
      onKeyEvent: (i, e) ->
        # Hook into F11
        if ((e.keyCode == 122 || e.keyCode == 27) && e.type == 'keydown')
          e.stop()
          return toggleFullscreenEditing()
    })

    # Enable rich-text buttons.
    bindRichTextButton = ($button, tag) ->
      $button.click ->
        editor.replaceSelection("<#{tag}>#{editor.getSelection()}</#{tag}>")
        editor.focus()

    bindRichTextButton($('.rich-text-control-bar').find('.bold'), 'strong')
    bindRichTextButton($('.rich-text-control-bar').find('.italic'), 'em')
    bindRichTextButton($('.rich-text-control-bar').find('.underline'), 'u')
    bindRichTextButton($('.rich-text-control-bar').find('.strikeout'), 's')
    bindRichTextButton($('.rich-text-control-bar').find('.code'), 'code')
    bindRichTextButton($('.rich-text-control-bar').find('.pre'), 'pre')
    bindRichTextButton($('.rich-text-control-bar').find('.h1'), 'h1')
    bindRichTextButton($('.rich-text-control-bar').find('.h2'), 'h2')
    bindRichTextButton($('.rich-text-control-bar').find('.h3'), 'h3')

    # Preview button should open a preview in a new tab.
    $('.rich-text-control-bar').find('.prev').click ->
      $form = $('form.web_template')
      $methodInput = $('input[name=_method]')
      oldAction = $form.attr 'action'
      oldMethod = $methodInput.val()
      oldBody = editor.getValue()

      # Reverse the input to avoid XSS protections in Chrome.
      newBody = oldBody.split('').reverse().join('')
      editor.setValue newBody
      $form.attr 'action', $('.rich-text-control-bar').find('span.hidden').html()
      $form.attr 'target', '_web_template'
      $methodInput.val 'post'
      $form.submit()

      $form.attr 'action', oldAction
      $form.attr 'target', ''
      editor.setValue oldBody
      $methodInput.val oldMethod

    # Esc or F11 should toggle a fullscreen editor.
    toggleFullscreenEditing = ->
      editorDiv = $('.CodeMirror-scroll')
      if (!editorDiv.hasClass('fullscreen'))
        toggleFullscreenEditing.beforeFullscreen = { height: editorDiv.height(), width: editorDiv.width() }
        editorDiv.addClass('fullscreen')
        editorDiv.height('100%')
        editorDiv.width('100%')
        editor.refresh()
      else 
        editorDiv.removeClass('fullscreen')
        editorDiv.height(toggleFullscreenEditing.beforeFullscreen.height)
        editorDiv.width(toggleFullscreenEditing.beforeFullscreen.width)
        editor.refresh()

    cloneSite = (clonePath) =>
      $cloneButton = $('#clone-website')
      $cloneButton.addClass 'submitting'
      $.post $cloneButton.attr('href'), {clone_url: $('#clone_url').val()}, (data) ->
        editor.setValue(data)
        $cloneButton.removeClass 'submitting'
        #CodeMirror.fromTextArea(document.getElementById('web_template_body'), {value: data})
        #$('.CodeMirror-scroll').last().remove()

    # Clone button should clone an existing website.
    $('#clone-website').click (e) ->
      cloneSite()
      e.preventDefault()

    # Exploit select menu should allow search
    $('#web_template_exploit_name').chosen()

    # Changing exploit select should load new module content.
    $('#web_template_exploit_name').change ->
      $('#module_fields').css opacity: 0.3
      $.post $('#exploit_options_url').val(), {module_name: $(this).val()}, (data) ->
        $('#module_fields').html(data)
        $('#module_fields').css opacity: 1

    # Protocol should be added to clone URL field on focus.
    $('#clone_url').focus ->
      $(this).val('http://') if !$(this).val()

    # Pressing enter key in clone URL field should clone, not submit form.
    $('#clone_url').keypress (e) ->
      if e.which == 13
        cloneSite()
        e.preventDefault()
