class CampaignsController < ApplicationController
	before_filter :load_workspace, :only => [ :index ]
	before_filter :my_load_workspace, :except => [ :index ]
	before_filter :check_license
	before_filter :redirect_if_not_licensed, :except => [ :index ]


  def index
		@page = (params[:page] || 1).to_i
		@per_page = (params[:n] || 20).to_i
		@per_page = 20 if @per_page == 0

		@count = Campaign.count(:conditions => { :workspace_id => @workspace.id })

		# Don't allow pages past the end
		last_page = ((@count - (@count % @per_page)) / @per_page) + 1
		@page = last_page if @page > last_page

		# Negative pages don't make sense
		@page = 1 if @page < 1

		@num_pages = 1 + ((@count-1) / @per_page)
		@offset = (@page-1) * @per_page

    @campaigns = Campaign.where(:workspace_id => @workspace.id).offset(@offset).limit(@per_page)

    #
		# Setup the order clause in the following find
    #

    order_direction = params[:sort_dir]

    if order_direction == 'desc'
      order_direction_class = Arel::Nodes::Descending
    else
      order_direction_class = Arel::Nodes::Ascending
    end

    sort_by = params[:sort_by]

    case sort_by
      when 'name'
        order_column = Campaign.arel_table[:name]
      when 'status'
        order_column = Campaign.arel_table[:status]
      when 'started_at'
        order_column = Campaign.arel_table[:started_at]
      when 'email_template'
        @campaigns = @campaigns.includes(:email_template)
        order_column = EmailTemplate.arel_table[:name]
      when 'web_template'
        @campaigns = @campaigns.includes(:web_template)
        order_column = WebTemplate.arel_table[:name]
    end

    if order_column
      order_direction_node = order_direction_class.new(order_column)

      @campaigns = @campaigns.order(order_direction_node)
    end

    #
    # Derived attributes that can't be sorted in SQL
    #

    if sort_by == 'web_link'
			@campaigns.sort_by! { |campaign|
        campaign.web_link || ''
      }

      if order_direction == 'desc'
        @campaigns.reverse!
      end
		end

		@licensed = License.get.supports_campaigns?
	end

	def new
		@campaign = Campaign.new(:workspace_id => @workspace.id)
	end

	def edit
		@campaign = Campaign.with_emails_by_id(params[:id]).first
	end

	def create
		campaign_opts = params[:campaign]
		@campaign = Campaign.new(
			:name=>campaign_opts[:name],
			 :listener_lhost=>campaign_opts[:listener_lhost],
			 :macro_name=>campaign_opts[:macro_name],
			 :do_web=>campaign_opts[:do_web],
			 :web_uripath=>campaign_opts[:web_uripath],
			 :web_srvhost=>campaign_opts[:web_srvhost],
			 :web_srvport=>campaign_opts[:web_srvport],
			 :web_ssl=>campaign_opts[:web_ssl],
			 :do_exe_gen=>campaign_opts[:do_exe_gen],
			 :exe_lport=>campaign_opts[:exe_lport],
			 :exe_name=>campaign_opts[:exe_name],
			 :payload_lhost=>campaign_opts[:payload_lhost],
			 :payload_type=>campaign_opts[:payload_type],
			 :do_email=>campaign_opts[:do_email],
			 :smtp_server=>campaign_opts[:smtp_server],
			 :smtp_port=>campaign_opts[:smtp_port],
			 :smtp_ssl=>campaign_opts[:smtp_ssl],
			 :smtp_user=>campaign_opts[:smtp_user],
			 :smtp_pass=>campaign_opts[:smtp_pass],
			 :mailfrom=>campaign_opts[:mailfrom],
			 :display_from=>campaign_opts[:display_from],
			 :web_urihost=>campaign_opts[:web_urihost],
			 :workspace_id => @workspace_id)

		if (@campaign.smtp_server and !@campaign.smtp_port)
			@campaign.smtp_port = 25
		end
		if @campaign.save
			if (params[:campaign][:file]) # FIXME: Should be handled in the model
				EmailAddress.from_file(@campaign, params[:campaign][:file])
			end
			if (@campaign.do_exe_gen)
				generate_exe
				@campaign.save! if @campaign.changed?
			end
			flash[:notice] = "Campaign created"
			redirect_to workspace_campaign_url(@workspace, @campaign)
		else
			render :action => 'new'
		end
	end

  def update
    @campaign = Campaign.find(params[:id])

    if @campaign.update_attributes(params[:campaign])
      if (params[:file])
        Mdm::EmailAddress.from_file(@campaign, params[:file])
      end

      respond_to do |format|
        format.html {
          flash[:notice] = "Campaign updated"
          redirect_to workspace_campaign_path
        }
        format.json {
          render :json => { :success => true }
        }
      end
    else
      respond_to do |format|
        format.html {
          @email_templates = Mdm::EmailTemplate.where(:campaign_id => nil)
          @web_templates   = Mdm::WebTemplate.where(:campaign_id => nil)
          render :action => 'edit'
        }
        format.json {
          err_msg = []
          @campaign.errors.each { |k, v| err_msg << "#{k} #{v}" }
          render :json => { :success => false, :message => 'Error: '+err_msg.join(', ') }
        }
      end
    end
  end		


	def show
		@campaign = Campaign.find(params[:id])
		if @campaign.do_web and not @campaign.web_template
			flash[:notice] = "Create a web template for this campaign"
			t = WebTemplate.create(:campaign_id => @campaign.id)
			redirect_to edit_workspace_web_template_path(@workspace, t)
			return
		end
		if @campaign.do_email and not @campaign.email_template
			flash[:notice] = "Create an email template for this campaign"
			t = EmailTemplate.create(:campaign_id => @campaign.id)
			redirect_to edit_workspace_email_template_path(@workspace, t)
			return
		end
		if @campaign.do_email and @campaign.email_addresses.blank?
			flash[:notice] = "Create/import email addresses for this campaign"
			redirect_to mass_edit_workspace_campaign_email_addresses_path(@workspace, @campaign)
			return
		end

	end

	def status
		@campaign = Campaign.find(params[:id])

		status = @campaign.status
		@campaign.update_status!

		if @campaign.status == Campaign::Statuses[:error]
			flash[:error] = @campaign.error
		elsif status != @campaign.status
			# Then it changed, notify the user
			flash[:notice] = "New Status: #{@campaign.status_string}"
		end

		respond_to do |format|
			format.html
			format.js   {
        render "status_update"
			}
		end
	end

	def start
		@campaign = Campaign.find(params[:id])
		errors = []
		if @campaign.running?
			errors << "Campaign is already running"
		else
			# Clear out any errors that linger from a previous run
			if (@campaign.error)
				@campaign.error = nil
				@campaign.status = :created
			end
			campaign_task = CampaignTask.new({
				:username => current_user.username,
				:workspace => @workspace,
				:campaign => @campaign
			})
			task = campaign_task.start
			if not task
				errors << campaign_task.error
				@campaign.status = :error
				@campaign.error = errors.join("\n")
			else
			  @campaign.status = :started
				@campaign.prefs[:task] = task.id
			end
			@campaign.save!
		end

		if errors.empty?
			flash[:notice] = "Campaign task started"
		else
			flash[:notice] = nil
			flash[:error] = errors.join("\n")
		end

		respond_to do |format|
      format.html
      format.js {
        render 'status_update' 
      }
    end
	end

	def stop
		@campaign = Campaign.find(params[:id])
		@campaign.status = :completed
		unless @campaign.prefs[:task].blank?
			task = Mdm::Task.find(@campaign.prefs[:task])
			task.stop unless task.done?
			@campaign.prefs[:task] = nil
			@campaign.error = nil
		end
		@campaign.save!

		respond_to do |format|
      format.html
      format.js {
        render 'status_update' 
      }
    end
	end

	def destroy
		campaign_ids = params[:campaign_ids] || []
		if (campaign_ids.empty?)
			flash[:error] = "No campaigns selected to delete"
		else
			Campaign.destroy(campaign_ids)
			flash[:notice] = view_context.pluralize(campaign_ids.size, "campaign") + " deleted"
		end

		respond_to do |format|
      format.html
      format.js {
        render 'index_update'
      }
    end
	end

	def download
		@campaign = Campaign.find(params[:id])
		if (@campaign.attachment)
			send_data @campaign.attachment.data,
				:filename => @campaign.attachment.name,
				:type => @campaign.attachment.content_type
		else
			flash[:error] = "No executable to download"
			redirect_to workspace_campaign_path
		end
	end

protected
	def my_load_workspace
		if params[:workspace_id]
			load_workspace
		else
			@workspace = @campaign.workspace
		end
	end

	def update_web_template(id)
		begin
			wt = WebTemplate.find(id)
		rescue ::ActiveRecord::RecordNotFound
		end
		old_wt = @campaign.web_template
		if (wt and old_wt)
			old_wt.update_attributes(wt.attributes)
			@campaign.web_template = old_wt
		elsif (wt)
			new_wt = wt.clone
			new_wt.campaign_id = @campaign.id
			new_wt.save
		end
	end

	def update_email_template(id)
		begin
			master_et = EmailTemplate.find(id)
		rescue ::ActiveRecord::RecordNotFound
			# If we have no master to clone from, then we are not making any
			# changes to the current template (which may be empty).
			return
		end
		old_et = @campaign.email_template
		# doesn't make sense to update a template with itself
		return if old_et == master_et

		if (old_et)
			# Then we're overwriting the attributes of the current template
			# with those of a new master.

			# First we have to remove the associated attachments.  This does
			# not delete the attachments themselves, just the row in the join
			# table.
			# XXX need to delete attachments that don't have any other
			# templates
			old_et.attachments.clear

			# Now update the values
			old_et.update_attributes(master_et.attributes)
			old_et.parent_id = master_et.id
			@campaign.email_template = old_et
		else
			# Then we're just cloning from a master.
			new_et = master_et.clone
			new_et.attachments = master_et.attachments
			new_et.campaign_id = @campaign.id
			new_et.parent_id = master_et.id
			new_et.save!
		end
	end

	def generate_exe
		generate_task = GenerateExeTask.new({
			:username => current_user.username,
			:workspace => @workspace,
			:campaign => @campaign,
			:lhost => @campaign.payload_lhost,
			:lport => @campaign.exe_lport,
			:filename => @campaign.exe_name,
			:ptype => @campaign.payload_type
		})
		task = generate_task.start
		if not task
			@campaign.status = :error
			@campaign.error = generate_task.error
		else
			@campaign.prefs[:task] = task.id
		end
	end

	def check_license
		@licensed = License.get.supports_campaigns?
	end

	def redirect_if_not_licensed
		unless @licensed
			render :text => "Unlicensed", :layout => "forbidden", :status => 403 # Forbidden
		end
	end
end

