class EmailAddressesController < ApplicationController
	before_filter :load_workspace
	before_filter :load_campaign
	before_filter :load_address, :only => [ :show, :edit, :update ]

  ORDER_COLUMN_BY_SORT_BY = {
      'address' => :address,
      'first_name' => :first_name,
      'id' => :id,
      'last_name' => :last_name
  }

	def index
		@page      = (params[:page] || 1).to_i
		@page      = 1 if @page < 1
		@per_page  = (params[:n] || 20).to_i
		@per_page  = 20 if @per_page == 0
		@count     = @campaign.email_addresses.size
		@num_pages = 1 + ((@count-1) / @per_page)
		@offset    = (@page-1) * @per_page

    @email_addresses = @campaign.email_addresses.offset(@offset).limit(@per_page)

    sort_by = params[:sort_by]
    # restrict valid sort_by values to prevent SQL injection warnings from Brakeman
    order_column = ORDER_COLUMN_BY_SORT_BY[sort_by]

    if order_column
      order_attribute = EmailAddress.arel_table[order_column]

      if params[:sort_dir] == 'desc'
        order_node_class = Arel::Nodes::Descending
      else
        order_node_class = Arel::Nodes::Ascending
      end

      order_node = order_node_class.new(order_attribute)
      @email_addresses = @email_addresses.order(order_node)
    end
	end

	def new
		render_popup("Create New Email Address", "new")
	end

	def create
		email_opts = params[:email_address]
		@email_address = @campaign.email_addresses.build(
			:address => email_opts[:address],
			:first_name => email_opts[:first_name],
			:last_name => email_opts[:last_name])
		if @email_address.save
			flash[:notice] = "Added #{@email_address.address}"
			redirect_to workspace_campaign_email_addresses_url(@workspace, @campaign)
		else
			render :action => 'new'
		end
	end

	def mass_edit
		@email_addresses = @campaign.email_addresses.all
		render "edit"
	end

	def mass_update
		if params[:campaign] and params[:campaign][:email_addresses_attributes]
			# Since the only attributes we're updating here are the email
			# addresses, merge in the campaign's existing attributes to avoid
			# replacing them with defaults. see #2753
			@campaign.update_attributes(params[:campaign].merge(@campaign.attributes))
			@campaign.save
		end
		
		redirect_to workspace_campaign_url(@workspace, @campaign)
	end

	def mass_delete
		count = 0
		if params[:email_address_ids]
			destroyed = EmailAddress.destroy_all(:id => params[:email_address_ids])
			count = destroyed.length
		end

		if count.zero?
			flash[:error] = "No addresses selected to delete"
		else
			flash[:notice] = "#{view_context.pluralize(count, "address")} deleted"
		end
		redirect_to workspace_campaign_email_addresses_url(@workspace, @campaign)
	end

	def new_import
		render_popup("Import New Email Addresses", "import")
	end

	def import
		if (params[:file])
			count = EmailAddress.from_file(@campaign, params[:file])
			flash[:notice] = "Imported #{count} new addresses (ignoring duplicates)"
			redirect_to workspace_campaign_url(@workspace, @campaign)
		else
			flash[:error] = "No file given"
			redirect_to workspace_campaign_email_addresses_url(@workspace, @campaign)
		end
	end

	def show
	end

protected
	def load_campaign
		id = params[:campaign_id].to_i
		@campaign = Campaign.find(id)
	end
	def load_address
		if params[:email_address_id]
			id = params[:email_address_id].to_i
			@email_address = EmailAddress.find(id)
		end
	end

end

