class HostsController < ApplicationController
  include AnalysisSearchMethods
  HOST_TAB_ACTIONS = [:services, :sessions, :vulns, :shares, :loots, :notes, :creds, :tags, :attempts, :details, :exploits]
  before_filter :load_workspace, :except => [:show, :edit, :update] + HOST_TAB_ACTIONS
  before_filter :load_host, :only => [:show] + HOST_TAB_ACTIONS
  before_filter :load_workspace_from_host, :only => [:show] + HOST_TAB_ACTIONS

  require 'shellwords'

  def index
    @stats = @workspace.stats

    sort_by = params[:sort_by] || 'address'
    sort_dir = params[:sort_dir] || 'asc'

    if request.format.json?
      @hosts = pro_search(:hosts,false) 
      @hosts_total_display_count = @hosts.length

      if params[:iDisplayLength] == '-1'
        @hosts_per_page = nil
      else
        @hosts_per_page = (params[:iDisplayLength] || 20).to_i
      end
      @hosts_offset = (params[:iDisplayStart] || 0).to_i


      @hosts_limited = @hosts.
        limit(@hosts_per_page).
        offset(@hosts_offset).
        order(hosts_sort_option)


      @hosts = @hosts_limited
      @hosts_total_display_count  ||= @hosts_total_count

    end

    #@hosts = @hosts.reject { |h| h.status.to_s != params[:status_filter] } if params[:status_filter]
    @hosts = sort_hosts(@hosts) if params[:iSortCol_0] == '11'

    respond_to do |format|
      format.html
      format.json { render :partial => 'hosts', :hosts => @hosts }
    end
  end

  def edit
    @host = Mdm::Host.find(params[:id])
    @workspace = @host.workspace
  end

  def new
    @host = @workspace.hosts.build
  end

  def create
    @host = Mdm::Host.new(params[:host])
    @host.state = 'alive'
    if @host.save
      flash[:notice] = "Host created"
      redirect_to @host
    else
      render :action => 'new'
    end
  end

  def map
    if not License.get.supports_map?
      redirect_to hosts_path(@workspace)
      return
    end

    tag_search = false
    if params[:search].blank? 
      @hosts = @workspace.hosts.includes(:notes)
    else
      @search = params[:search].to_s
      search_terms = ::Shellwords.shellwords(@search) rescue @search.split(/\s+/)

      @hosts = @workspace.hosts.alive

      search_terms.each do |term|
        if term =~ /^#(.*)/ and License.get.supports_tags?
          @hosts = @hosts.tag_search($1)
          tag_search = true	
        else
          @hosts = @hosts.search(term)
        end
      end
    end

    @total_nodes = 0

    @traceroutes = {"ip" => "localhost", "os_name"=>"localhost", "name"=>"Metasploit Pro", "depth"=>0,"children" => []}

    traceroute = false
    @hosts.each do |host|
      host.notes.each do |note|
        if note.ntype == "host.nmap.traceroute"
          add_to_routes(note.data["hops"])
          @total_nodes = @total_nodes + note.data["hops"].length
          traceroute = true
        end
      end
      if (not traceroute and !params.has_key?(:trace_hosts)) or tag_search
        @traceroutes["children"].push({"ip"=>host.address,"no_traceroute"=>true,"depth"=>1,"children"=>[]})
        @total_nodes = @total_nodes + 1
      end

      traceroute = false
      add_route_data(host,@traceroutes)
    end

    if @total_nodes > 500
      flash[:nodes] = "The total number of nodes is greater than 500 which can cause the page to slow down. "
      flash[:nodes] << "Click CONTINUE below to load the map with all nodes or use the search functionality"
      flash[:nodes] << " (e.g. 10.0.0., printers, #imported) to lower the number of nodes rendered."
      flash[:nodes] << "  Current Total Nodes: #{@total_nodes}."
    else
      flash[:nodes] = nil
    end
    respond_to do |format|
      format.html
      format.json { render :partial => 'maps.json', :maps => @traceroutes, :total_nodes=>@total_nodes }
    end
  end


  # Mdm::Note that host tags should be uniq by name and inclusion in this
  # workspace, which is why this doesn't call Mdm::Tag.new directly, but
  # first looks for a tag associated with a host in this workspace,
  # and if that fails, Mdm::Tag.new. This would be easier with a workspace_id
  # on the Mdm::Tag itself. The upside is that it removes a constraint (the
  # workspace_id), and might make tag searching cross-workspace a little
  # easier -- right now tag searching is intentionally restrained to 
  # be limited to a workspace.
  def _create_or_update_tag(args={})
    host_id = args[:host_id]
    @host = Mdm::Host.find(host_id)
    unless @host
      flash[:error] = "Host #{host_id} not found."
    end
    workspace_id = @host.workspace_id
    possible_tag = Mdm::Tag.where("hosts.workspace_id = ? and tags.name = ?", workspace_id, args[:name]).includes(:hosts).first
    @tag = possible_tag || Mdm::Tag.new
    @tag.name = args[:name]
    @tag.user_id = args[:user_id]
    @tag.report_summary = args[:report_summary] if !args[:report_summary].nil?
    @tag.report_detail = args[:report_detail] if !args[:report_detail].nil?
    @tag.critical = args[:critical] if !args[:critical].nil?
    if !args[:desc].nil? && !args[:desc].strip.empty?
      @tag.desc = args[:desc]
    end
    @tag.add_host_by_id(host_id)
    @tag.save
    return @tag
  end

  def create_or_update_tag
    case params[:commit]
    when /^(Save|Update)/
      args = {
        :host_id => params[:id],
        :name => params[:tag][:name],
        :desc => params[:tag][:desc],
        :user_id => @current_user.id
      }
      args[:report_summary] = !!(params[:tag][:report_summary] == "1")
      args[:report_detail] =  !!(params[:tag][:report_detail]  == "1")
      args[:critical] =       !!(params[:tag][:critical]       == "1")
      tag = _create_or_update_tag(args)
      unless tag.errors.empty?
        flash[:error] = "%s %s" % [tag.errors.first[0], tag.errors.first[1]]
      else
        flash[:notice] = "Tag ##{h tag.name} %s." % [tag.updated? ? 'updated' : 'created']
      end
    when /^(Remove|Delete)/
      this_tag = Mdm::Tag.find(params[:tag_id])
      this_tag.hosts.delete (host = Mdm::Host.find(params[:id]))
      flash[:notice] = "Removed ##{this_tag.name} from #{host.name || host.to_s}"
      unless this_tag.save
        flash[:error] = @tag.errors
      end
    end
    redirect_to host_path(params[:id]) + "#tags"
  end

  # Finally refactored!
  def quick_multi_tag
    hosts = params[:host_ids].collect { |id| Mdm::Host.find(id.to_i) }
    tags_invalid = false
    saved_tags = {} # save a tag after you add it, and reuse on next host
    if params[:new_host_tags] && params[:host_ids]
      if params[:host_ids].size == 1 
        # if only one host specified, make host.tags an *exact* replica of new_host_tags,
        # so clear host.tags before appending 
        hosts.first.tags.clear
      end  
      hosts.each do |host|
        names = params[:new_host_tags].split(',').uniq
        names.each { |name|
          if saved_tags[name].present?
            host.add_tag saved_tags[name]
          else
            saved_tags[name] = host.add_tag_by_name name
          end
          tags_invalid = true unless saved_tags[name].valid?
        }
      end
    end

    respond_to do |format|
      format.html { redirect_back }
      format.js do
        if tags_invalid
          # convert name=>tag hash to array of tags
          tags = saved_tags.flatten.select { |t| t.class <= Mdm::Tag }
          # generate an error message for it
          error_message = tags.select { |t| !t.valid? }.first.errors.full_messages.join ', '
          render :text => error_message, :status => 403
        else
          render :nothing => true, :status => 200
        end
      end
    end
  end

  def multi_tag
    if params[:host_ids]
      args = {
        :user_id => @current_user.id,
        :name => params[:name],
        :desc => params[:desc],
      }
      if params[:tag_attributes]
        args[:report_summary] = params[:tag_attributes].include? "report_summary"
        args[:report_detail] =  params[:tag_attributes].include? "report_detail"
        args[:critical] =       params[:tag_attributes].include? "critical"
      end
      successful_hosts = []
      params[:host_ids].each do |host_id|
        tag = _create_or_update_tag(args.merge(:host_id => host_id.to_i))
        unless tag.errors.empty?
          flash[:error] = tag.errors
          break
        end
        successful_hosts << Mdm::Host.find(host_id).address
      end
      if successful_hosts.size == params[:host_ids].size
        flash[:notice] = "Applied tag ##{h args[:name]} to #{ActionController::Base.helpers.pluralize(successful_hosts.size, 'host')}"
      end
    else
      flash[:error] ||= "No hosts were selected to tag."
    end
    render_js_redirect(hosts_path(@workspace))
  end

  def remove_tag
    host_id = params[:remove_tag][:host_id]
    begin
      @host = Mdm::Host.find(host_id)
    rescue
      redirect_to hosts_path(params[:workspace_id])
    end
    if params[:tag_ids].nil? || params[:tag_ids].empty?
      flash[:error] = "No tags were selected to remove."
    else
      params[:tag_ids].each do |tid|
        @tag = Mdm::Tag.find_by_id(tid)
        @tag.hosts.delete @host
        @tag.destroy if @tag.hosts.empty?
      end
    end
    redirect_to host_path(@host) + "#tags"
  end

  def update
    @host = Mdm::Host.find(params[:id])
    orig = @host.attributes.dup
    orig_services = @host.services.clone
    @workspace = @host.workspace
    if @host.update_attributes(params[:host])
      diff = @host.attributes.reject {|k,v| (k == "updated_at") || (orig[k] == v) }
      services_match = (orig_services == @host.services && orig_services.size == @host.services.count)
      if !diff.empty? || !services_match
        diff.each_pair do |k,v|
          note = @host.notes.find_or_initialize_by_ntype_and_host_id("host.updated.#{k}",@host.id)
          note.workspace_id = @workspace.id
          note.data = {:user => current_user.username,
            :value => v,
            :locked => (params[:host][:lock_attrs] == "1" ? true : false)
          }
          note.save!
        end
        flash[:notice] = "Host updated"
      else
        flash[:notice] = "No updates"
      end
      redirect_to @host
    else
      render :action => 'edit'
    end
  end

  def show
    # TODO: These should probably be block helpers. Doesn't need
    # to be in the controller.
    @has_services = ( @host.services.size > 0 )
    @has_sessions = ( @host.sessions.size > 0 )
    @has_shares   = ( (@host.smb_shares.size + @host.nfs_exports.size) > 0 )
    @has_loot     = ( @host.loots.size > 0 )
    @has_notes    = ( @host.notes.size > 0 )
    @has_creds    = ( @host.creds.size > 0 )

    @workspace = @host.workspace
    respond_to do |format|
      format.html
      format.xml { render :xml => @host }
    end
    @host.clear_flags
  end

  def services
    respond_to do |format|
      format.js { render :partial => "services_tab" }
    end
  end

  def sessions
    respond_to do |format|
      format.js { render :partial => "sessions" }
    end
  end

  def vulns
    respond_to do |format|
      format.js { render :partial => "vulns_tab"}
    end
  end

  def shares
    respond_to do |format|
      format.js { render :partial => "shares_tab"}
    end	
  end

  def loots
    respond_to do |format|
      format.js { render :partial => "loots", :locals => { :loots => @host.loots }}
    end
  end

  def notes
    respond_to do |format|
      format.js { render :partial => "notes", :locals => { :notes => @host.notes }}
    end
  end

  def creds
    respond_to do |format|
      format.js do
        render :partial => "generic/auths",
          :locals => {:show_timestamps => true, :detailed => true, :password_mask_bool => false, :host_record => true},
          :object => @workspace.creds.select {|cred| cred.service.host == @host}
      end
    end
  end

  def tags
    if License.get.supports_tags?
      respond_to do |format|
        format.js { render :partial => "tags" }
      end
    end
  end

  def attempts
    @exploit_attempts = @host.exploit_attempts.
      select("exploit_attempts.*, module_details.name AS module_name").
      from("exploit_attempts,module_details").
      where("exploit_attempts.module = module_details.fullname")
    respond_to do |format|
      format.js { render :partial => "exploit_attempts_tab", :locals => { :host => @host }}
    end
  end

  def details
    respond_to do |format|
      format.js { render :partial => "host_details_tab", :locals => { :host => @host }}
    end
  end

  def exploits
    respond_to do |format|
      format.js { render :partial => "host_exploits_tab", :locals => { :host => @host }}
    end
  end

  def destroy
    host_ids = [params[:id]]       if params[:id]
    host_ids = params[:host_ids]   if params[:host_ids]

    hosts = Mdm::Host.destroy(host_ids)

    if hosts.length.zero?
      flash[:error] = "No hosts selected to delete"
    else
      flash[:notice] = view_context.pluralize(hosts.length, "host") + " deleted"
    end

    render_js_redirect(hosts_path(@workspace))
  end

  private

  # Generate a SQL sort by option based on the incoming DataTables paramater.
  #
  # Returns the SQL String.
  def hosts_sort_option
    column = case params[:iSortCol_0]
             when '1'
               'inet(hosts.address)'
             when '2'
               'hosts.name'
             when '3'
               'hosts.os_name'
             when '4'
               'hosts.virtual_host'
             when '5'
               'hosts.purpose'
             when '6'
               'service_count'
             when '7'
               'vuln_count'
             when '8'
               'exploit_attempt_count'
             when '9'
               'tag_count'
             when '10'
               'hosts.updated_at'
             when '11'

             end
    column + ' ' + (params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc') if column
  end


  def sort_hosts(hosts)
    desc = params[:sSortDir_0] == 'desc'

    @stats ||= @workspace.stats
    sort_hosts_by(hosts, desc) { |h| h.status(@stats) }
  end

  # sort the hosts using the given block, always putting blanks at the end
  def sort_hosts_by(hosts, descending, &block)
    blank, non_blank = hosts.partition do |h|
      val = yield(h)
      val.nil? or val == "" or val == 0
    end

    if descending
      non_blank.sort { |a,b| yield(a) <=> yield(b) } + blank
    else
      non_blank.sort { |d,c| yield(c) <=> yield(d) } + blank
    end
  end

  def create_tuples(hops)
    tuples = []
    for pos in (0..hops.length) do
      next if not hops[pos]
      if hops[pos]["ttl"] == "1"
        tuples.push([{"ip"=>"127.0.0.1"},{"ip"=>hops[pos]["ipaddr"]}, pos+1])
        if hops[pos+1]
          tuples.push([{"ip"=>hops[pos]["ipaddr"]},{"ip"=>hops[pos+1]["ipaddr"]}, pos+2])
        end
      else
        if hops[pos+1]
          tuples.push([{"ip"=>hops[pos]["ipaddr"]},{"ip"=>hops[pos+1]["ipaddr"]}, pos+3])
        end
      end
    end
    return tuples
  end

  def add_to_routes(hops)
    tuples = create_tuples(hops)

    for pos in (0..tuples.length) do
      next if not tuples[pos]
      next if tuples[pos][1]["ip"] == "0.0.0.0"
      current_child = check_children(@traceroutes,tuples[pos][1]["ip"])
      next if tuples[pos][0]["ip"] == "127.0.0.1" and current_child

      if(@traceroutes["children"].length == 0)
        @traceroutes["children"].push({"ip"=>tuples[pos][1]["ip"],"depth"=>tuples[pos][2],"children"=>[]})
      elsif(tuples[pos][0]["ip"] == "127.0.0.1" and !current_child)
        @traceroutes["children"].push({"ip"=>tuples[pos][1]["ip"],"depth"=>tuples[pos][2],"children"=>[]})
      else
        @traceroutes = find_location(tuples[pos],@traceroutes)
      end
    end
  end

  def find_location(tuple,root)
    return root if not root["children"]
    for pos in (0..root["children"].length)
      next if not tuple[0] or not root["children"][pos]
      if tuple[0]["ip"] == root["children"][pos]["ip"]
        is_child = check_children(root["children"][pos],tuple[1]["ip"])
        if !is_child
          grandchild = {"ip"=>tuple[1]["ip"],"depth"=>root["children"][pos]["depth"].to_i + 1,"children"=>[]}
          root["children"][pos]["children"].push(grandchild)
          return root
        end
      else
        root["children"][pos] = (find_location(tuple,root["children"][pos]))
      end
    end
    return root
  end

  def check_children(root,ip)
    return false if not root["children"]
    root["children"].each do |child|
      next if not child
      if child["ip"] == ip
        return true
      end
    end
    return false
  end

  def add_route_data(host,root)
    return root if not root["children"]
    for pos in (0..root["children"].length)
      next if not root["children"][pos]
      if host.address == root["children"][pos]["ip"]
        root["children"][pos]["os_name"] = host.os_name
        root["children"][pos]["os_flavor"] = host.os_flavor
        root["children"][pos]["mac"] = host.mac
        root["children"][pos]["comments"] = host.comments
        root["children"][pos]["name"] = host.name
        root["children"][pos]["id"] = host.id
        root["children"][pos]["status"] = host.status	
        return root
      else    
        root["children"][pos] = add_route_data(host,root["children"][pos])
      end
    end
    return root
  end

  def load_host
    @host = Mdm::Host.find(params[:id])
    @exploits = build_exploit_match
  end

  def load_workspace_from_host
    @workspace = @host.workspace
  end

  # Shoes on 'Modules' tab of the Host show view
  def build_exploit_match
    matches = Mdm::ModuleDetail.select("DISTINCT(module_details.*)").
      from("module_details, module_refs, vulns, vulns_refs, refs").
      where(
        "module_details.id = module_refs.module_detail_id AND vulns.host_id = ? AND " +
        "vulns_refs.vuln_id = vulns.id AND vulns_refs.ref_id = refs.id AND " +
        "refs.name = module_refs.name AND module_details.mtype IN (?)", 
        @host.id, ["exploit", "auxiliary"]
      ).
      group("module_details.id").
      order("module_details.mtype DESC, module_details.rank DESC").all.uniq
    
    # de-dupe module listing
    module_fullnames = []
    exploits = []
    matches.each do |m|
      unless module_fullnames.include? m.fullname
        module_fullnames << m.fullname
        exploits << m
      end
    end
    exploits
  end
end
