class LootsController < ApplicationController
	include AnalysisSearchMethods
	before_filter :load_workspace, :only => [:index, :destroy_multiple]
	before_filter :fix_datatables_search_parameters, :only => [:index]

	helper HostsHelper

	def add_loot
		if params[:loot][:data].blank?
			flash[:error] = "No File Provided"
		elsif params[:loot][:name].blank?
			flash[:error] = "No Name Provided"
		elsif params[:id].blank?
			flash[:error] = "No Host Selected"
		else
			file_contents = params[:loot][:data].tempfile.read
			params[:loot][:data] = file_contents
			params[:loot][:host_id] = params[:id]
			params[:loot][:ltype] = "user.added.loot"
			mycli = Pro::Client.get
			mycli.loot_create(params[:loot])
		end
		
		if flash[:error]
			redirect_to :action => "new"
		else
			redirect_to :controller => 'hosts', :action => "show"
		end
	end

	# TODO: Action should be called 'download'.
	def get
		@loot = Mdm::Loot.find(params[:id])
		@host = @loot.host

		ext   = "dat"
		dis   = "attachment"

		case @loot.content_type
		when /x-ms-bmp/
			ext = "bmp"
			dis = "inline"
		when /jpg|jpeg/
			ext = "jpg"
			dis = "inline"
		when /png/
			ext = "png"
			dis = "inline"
		when "text/plain"
			ext = "txt"
			dis = "inline"
		end

		fname = downloadable_fname(ext)
		File.open(@loot.path, 'rb') do |file|
			send_data file.read, 
			:type => @loot.content_type,
			:disposition => (params[:disposition] || dis),
			:filename => fname
		end
	end

	# It would be nice to have ext and dis be proper attributes of Loot.
	def downloadable_fname(ext)
		if @loot.name
			@host ? "#{@host.address}_#{@loot.name}" : @loot.name
		else
			fname = "#{@loot.ltype}.#{ext}"
			@host ? "#{@host.address}_#{fname}" : fname
		end
	end

	def index
		load_loots if request.format.json?

		session[:return_to] = workspace_loots_path(@workspace)

		respond_to do |format|
			format.html
			format.json { render :partial => 'loots', :notes => @loots }
		end
	end


	def destroy_multiple
		loots = Mdm::Loot.destroy params[:loot_ids]

		flash[:notice] = view_context.pluralize(loots.length, "piece") + " of evidence deleted"

		render_js_redirect(workspace_loots_path(@workspace))
	end

	private

	def load_loots
		if params[:iDisplayLength] == '-1'
			@loots_per_page = nil
		else
			@loots_per_page = ( params[:iDisplayLength] || 20 ).to_i
		end
		@loots_offset = ( params[:iDisplayStart] || 0 ).to_i
		loots_set = pro_search(:loots,false)
		@loots = loots_set.
					limit(@loots_per_page).
					offset(@loots_offset).
					order(loots_sort_option)

		@loots_total_display_count = loots_set.length
	end

	# Generate a SQL sort by option based on the incoming DataTables parameter.
	#
	# Returns the SQL String
	def loots_sort_option
		column = case params[:iSortCol_0]
		when '2'
			'ltype'
		when '3'
			'name'
		when '4'
			'size'
		when '5'
			'info'
		end
		column + ' ' + (params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc') if column
	end



end

