class NotesController < ApplicationController
	include AnalysisSearchMethods
	before_filter :load_workspace
	before_filter :set_pagination, :only => [:index, :combined]
	before_filter :fix_datatables_search_parameters, :only => [:index]

	helper HostsHelper

	def index
		load_notes if request.format.json?

		session[:return_to] = workspace_notes_path(@workspace)

		respond_to do |format|
			format.html
			format.json { render :partial => 'notes', :notes => @notes }
		end
	end

	def destroy_multiple
		notes = Mdm::Note.destroy params[:note_ids]
		
		flash[:notice] = view_context.pluralize(notes.length, "note") + " deleted"

		render_js_redirect(workspace_notes_path(@workspace))
	end

	def combined
		load_combined_notes if request.format.json?

		respond_to do |format|
			format.html
			format.json { render :partial => 'notes_combined', :notes => @notes }
		end
	end

	private

	# Determine the correct pagination arguments based on the incoming DT parameters.
	#
	# Returns nothing.
	def set_pagination
		if params[:iDisplayLength] == '-1'
			@notes_per_page = nil
		else
			@notes_per_page = (params[:iDisplayLength] || 20).to_i
		end
		@notes_offset = (params[:iDisplayStart] || 0).to_i
	end

	def load_notes
		notes_set = pro_search(:notes,false)
		@notes = notes_set.
					limit(@notes_per_page).
					offset(@notes_offset).
					order(notes_sort_option)
		@notes_total_display_count = notes_set.length 
	end

	def load_combined_notes
		notes_set = pro_search(:notes,true)
		combined_notes_group_option = "notes.ntype"
		@notes = notes_set.
					select("COUNT(notes.id) AS host_count, notes.ntype").
					group(combined_notes_group_option).
					limit(@notes_per_page).
					offset(@notes_offset).
					order(notes_combined_sort_option)
		@notes_total_display_count = notes_set.length 
	end

	# Generate a SQL sort by option based on the incoming DataTables parameter.
	#
	# Returns the SQL String
	def notes_sort_option
		column = case params[:iSortCol_0]
		when '2'
			'ntype'
		when '4'
			'created_at'
		end
		column + ' ' + (params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc') if column
	end

	# Generate a SQL sort by option based on the incoming DataTables parameter.
	#
	# Returns the SQL String
	def notes_combined_sort_option
		column = case params[:iSortCol_0]
		when '0'
			'ntype'
		when '1'
			'host_count'
		end
		column + ' ' + (params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc') if column
	end

end
