class QuickPentestController < ApplicationController
  # Wizard autogenerates the routes for each step of the wizard
  include Wicked::Wizard
  steps :create_project,
        :configure_scan,
        :configure_exploits,
        :generate_report

  layout nil         # don't wrap this in a layout, as it's AJAX'd in
  helper TasksHelper # we need the report_types helper

  # render entire blank form
  def show
    @steps = steps
    @current_step = step
    @quick_pentest = QuickPentestBuilder.new({}, current_user).set_defaults!.quick_pentest
    render :partial => 'form'
  end

  # form submission: depending on the step, we will either
  #   launch the scan or simply validate.
  def update
    # 'step' contains symbol with name of curr step
    @quick_pentest = QuickPentestBuilder.new(params, current_user).build
    validator = QuickPentestValidator.new(@quick_pentest)
    if validator.step_is_valid? step
      render_success_with_options
    else
      render_errors_from_validator(validator)
    end
  end

  def create
    @builder = QuickPentestBuilder.new(params, current_user)
    @quick_pentest = @builder.build
    @validator = QuickPentestValidator.new(@quick_pentest)
    if @validator.steps_are_valid? steps # run all stepz
      @task_chain = @builder.to_task_chain
      # save the workspace first since EVERYTHING relies on it
      @quick_pentest.workspace.save if @task_chain.valid?
      if @task_chain.save
        # pass the workspace ID back to the client so he can redirect to task log
        render_success_with_options(:redirect => true, :task_chain_id => @task_chain.id)
      else
        # handle task chain error? user is fucked. TODO: LOG THIS SITUATION
        render_errors_from_validator(@validator)
      end
    else
      # see comment in #update
      render_errors_from_validator(@validator)
    end
  end

  # client polls this endpoint until the first task is kicked off 
  def first_task_ready
    task_chain = TaskChain.find(params[:task_chain_id])
    # we aren't POSTing through an iframe here, just straight AJAX,
    #   so noneed to use the _pass_data workaround for IE
    render :json => { :success => !task_chain.workspace.tasks.empty? }
  end

  # since data is POSTed indirectly to an iframe, we can't just redirect the POST.
  # instead, we pass back :redirect => true, and the client side JS redirects the page
  #   to /quick_pentest/redirect_after_success?task_chain_id=1
  def redirect_after_success
    # I wish there was a half-decent TaskChain#show, for now just punt them out to the task
    task_chain = TaskChain.find(params[:task_chain_id])
    first_task = task_chain.workspace.tasks.first
    if first_task.nil?
      head :error
    else
      flash['success'] = 'Quick Pentest has been configured successfully.'
      redirect_to tasks_path(first_task.workspace)
    end
  end

  private
  def render_errors_from_validator(validator)
    # we use 'pass_data' partial to enable passing JSON back to the user.
    # the JSON is wrapped in <textarea> tags to fix an IE9
    #   annoyance related to doing AJAXy file uploads.
    # (unfortunately this IE bug also means we can't get a SUCCESS or ERROR condition
    #   client-side from http, so we use {success=>true} as a crappy workaround)
    render :partial => 'pass_data', :locals => { :data => {
      :success => false, :errors => validator.errors_hash
    }}
  end

  def render_success_with_options(options={})
    render :partial => 'pass_data', :locals => { :data => { :success => true }.merge(options) }
  end
end