class ReportsController < ApplicationController
  include Reporting
  include TasksSharedControllerMethods

  DEFAULT_TEMPLATE_FILENAME      = REPORTING_DEFAULT_JRXML
  SIMPLE_TEMPLATE_FILENAME       = "msfx-simple.jrxml"

	layout "report", :except => :index

	before_filter :load_workspace
	before_filter :process_report_params
	before_filter :require_admin, :only => [:report_template_delete]
	before_filter :gather_report_data, :only => [:index]

	after_filter :clear_flash_messages

	def index
		render :action => "index", :layout => "application"
	end

	def gather_report_data
		@workspaces = Mdm::Workspace.all
		@reports = @workspace.reports.all
		@licensed = License.get.supports_reports?
	end

	def report_view
		report_download(dis="inline")
	end

	def report_download(dis="attachment")
		@report = Mdm::Report.find(params[:report_id])

		ext   = "dat"
		typ   = "application/octet-stream"

		case @report.rtype
		when /-?HTML$/
			ext = "html"
			typ = "text/html"
		when /(-?PDF|Jasper|JASPER)$/ # Do we still have a JASPER type anywhere? Don't think so...
			ext = "pdf"
			typ = "application/pdf"
		when /-?RTF$/
			ext = "rtf"
			typ = "application/rtf"
		when /-?WORD$/
			ext = "docx"
			typ = "application/msword"
		when /-?XML$/
			ext = "xml"
			typ = "application/xml"
		when "PWDUMP" 
			ext = "txt"
			typ = "text/plain"
		when /-?ZIP$/
			ext = "zip"
			typ = "application/zip"
		when "REPLAY"
			ext = "replay.zip"
			typ = "application/zip"
		end

		unless caller[0] =~ /`report_view'/
			@report.downloaded_at = Time.now.utc.to_s
			@report.save!
		end

		send_data File.open(@report.path, "rb") { |f| f.read },
			:type => typ,
			:disposition => dis,
      :filename => File.split(@report.path).last
	end

	# Hard link to download the default template.
	def report_template_download_static
		case params[:static_id].intern
		when :default_template
			file_path = DEFAULT_TEMPLATE_FILENAME
			type = "application/xml"
			fname = "msfx-default.jrxml"
		when :simple_template
			file_path = SIMPLE_TEMPLATE_FILENAME
			type = "application/xml"
			fname = "msfx-simple.jrxml"
		end

		full_file_path = File.join(pro_report_directory, file_path)
		send_data File.open(full_file_path, "rb") { |f| f.read },
			:type => type,
			:disposition => "attachment",
			:filename => fname
	end

	def report_template_download
		@report_template = Mdm::ReportTemplate.find(params[:report_template_id])

		ext   = "dat"
		dis   = "attachment"
		typ   = "application/octet-stream"

		case @report_template.path
		when /\.(gif)/i
			ext = $1
			dis = "inline"
			typ = "image/gif"
		when /\.(jpg|jpeg)/i
			ext = $1
			dis = "inline"
			typ = "image/jpeg"
		when /\.(png)/i
			ext = $1
			dis = "inline"
			typ = "image/gif"
		when /\.(jrxml)/i
			ext = $1
			typ = "application/xml"
		when /\.(jrxml)/i
			ext = $1
		end

		send_data File.open(@report_template.path, "rb") { |f| f.read },
			:type => typ,
			:disposition => dis,
			:filename => File.split(@report_template.path).last
	end

	def reports_delete
		report_ids = params[:report_ids]
		if (report_ids.nil? || report_ids.empty?)
			flash[:notice] = nil
			flash[:error] = "No reports selected to delete"
		else
			deleted_count = 0
			report_ids.each do |rid|
				rep = Mdm::Report.find_by_id(rid)
				next unless rep
				@c ||= Pro::Client.get
				r = @c.call('pro.report_delete_data_file', rid)
				rep.destroy
				deleted_count += 1
			end
			flash[:error] = nil
			flash[:notice] = view_context.pluralize(deleted_count, "report") + " deleted"
		end
		gather_report_data
		
		render 'flash_message_update'
	end

	# XXX: Fix this too
	def report_template_delete
		@report_template = Mdm::ReportTemplate.find(params[:report_template_id])
		r_name = @report_template.name
		@c ||= Pro::Client.get
		r = @c.call('pro.report_template_delete_file', params[:report_template_id])
		@report_template.destroy
		flash[:notice] = "Deleted '#{r_name}'"
		redirect_to new_custom_report_path
	end

end

