class ServicesController < ApplicationController
	include AnalysisSearchMethods
	before_filter :load_workspace
	before_filter :set_pagination, :only => [:index, :combined]
	before_filter :fix_datatables_search_parameters, :only => [:index]

	helper HostsHelper

	def index
		load_services if request.format.json?

		session[:return_to] = workspace_services_path(@workspace)

		respond_to do |format|
			format.html
			format.json { render :partial => 'services', :services => @services }
		end
	end

	def combined
		load_combined_services if request.format.json?

		respond_to do |format|
			format.html
			format.json { render :partial => 'services_combined', :services => @services }
		end
	end

	def destroy_multiple
		services = Mdm::Service.destroy params[:service_ids]

		flash[:notice] = view_context.pluralize(services.length, "service") + " deleted"

		render_js_redirect(workspace_services_path(@workspace))
	end

	private

	# Determine the correct pagination arguments based on the incoming DT parameters.
	#
	# Returns nothing.
	def set_pagination
		if params[:iDisplayLength] == '-1'
			@services_per_page = nil
		else
			@services_per_page = (params[:iDisplayLength] || 20).to_i
		end
		@services_offset = (params[:iDisplayStart] || 0).to_i
	end

	def load_services 
		service_set = pro_search(:services,false)
		@services = service_set
					.limit(@services_per_page)
					.offset(@services_offset)
					.order(services_sort_option)
					.group('services.id, hosts.id')
					.select('hosts.name AS host_name, hosts.address, hosts.id AS host_id, ' +
						# okay, Mdm::Host has some gotchas to sorting:
						# 1. Generally, sort Hosts by #name (type:string)
						# 2. If Host#name is blank, default to sorting by #address (type:inet)
						# 3. If Host#name is EQUAL to Host#address, default to sorting by #address (type:inet)
						#       (otherwise the address will be alphabetically sorted, WRONG)
						'(CASE hosts.name WHEN HOST(hosts.address::inet)::text THEN NULL ELSE LOWER(hosts.name) END) ' +
						'AS host_name_sort')
						# HOST(hosts.address)::text returns e.g. '127.0.0.1', whereas
						#   CAST(hosts.address TO text) returns e.g. '127.0.0.1/32'

		@services_total_display_count = service_set.length
	end

	def load_combined_services
		service_set = pro_search(:services,true)
		@services = service_set.
					limit(@services_per_page).
					offset(@services_offset).
					order(services_combined_sort_option)
        @services_total_display_count = service_set.length
	end

	# Generate a SQL sort by option based on the incoming DataTables parameter.
	#
	# Returns the SQL String
	def services_sort_option
		sort_dir = params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc'
		case params[:iSortCol_0]
		when '1'
			"host_name_sort #{sort_dir}, hosts.address #{sort_dir}"
		when '2'
			"services.name #{sort_dir}"
		when '3'
			"services.proto #{sort_dir}"
		when '4'
			"services.port #{sort_dir}"
		when '5'
			"services.info #{sort_dir}"
		when '6'
			"services.state #{sort_dir}"
		else
			"services.updated_at #{sort_dir}"
		end
	end


	# Generate a SQL sort by option based on the incoming DataTables parameter.
	#
	# Returns the SQL String
	def services_combined_sort_option
		column = case params[:iSortCol_0]
		when '0'
			'services.name'
		when '1'
			'services.proto'
		when '2'
			'services.port'
		when '3'
			'services.info'
		when '4'
			'host_count'
		end
		column + ' ' + (params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc') if column
	end


end
