class TasksController < ApplicationController
  include Reporting
  include TasksSharedControllerMethods

  before_filter :load_workspace, :except => [:stop, :replay, :logs, :stop_all]

  before_filter :require_admin, :only => [:new_tunnel, :start_tunnel, :start_report_template, :delete_imported_creds]  # Should this include :stop_all ?

  before_filter :process_report_params, :only => [:start_report]
  before_filter :gather_workspace, :only => [:start_report]
  before_filter :gather_users, :only => [:start_report]
  before_filter :gather_report_templates, :only => [:new_custom_report, :new_report]
  before_filter :using_embedded_layout?
  layout :task_layout_selector


  def index
    @tasks = @workspace.tasks.includes(:workspace)
    respond_to do |format|
      format.html
      format.js {
        render 'index_update' # FIX ME: For some reason this renders as mime_type undefined!
      }
    end
  end

  def gather_report_templates
    @report_templates = @workspace.report_templates.all
    @report_templates_jasper = @workspace.report_templates_jasper
    @report_templates_logos = @workspace.report_templates_logos
  end

  def show
    @task = Mdm::Task.find(params[:id])
    path = @task.path
    if path and File.exist?(path) and
      @lines = File.read(path, File.size(path))
    else
      @lines = '(Task log is not available)'
    end

    @logs  = ''
    @lines.split("\n").each do |line|
      @logs << view_context.tasklog_prettify(line)
    end
    @logs << view_context.tasklog_cap(@task)

    respond_to do |format|
      format.html
      format.js {
        render 'show_update'
      }
    end
  end

  def logs
    @task = Mdm::Task.find(params[:id])
    path = @task.path
    from = (params[:line] || 0).to_i
    @logs = @task.log_lines(from).map do |line|
      view_context.tasklog_prettify(line)
    end.join + view_context.tasklog_cap(@task)
    data = {
      :header => render_to_string(:partial => 'task_header'),
      :log    => @logs
    }
    render :json => data.to_json
  end

  # Stops the specified task
  def stop
    @task = Mdm::Task.find(params[:id])
    @task.stop unless @task.nil?

    # render :update do |page|
    #   page.replace_html dom_id(@task), :partial => 'task', :object => @task
    # end
    @task_dom_id = dom_id @task
    render 'stop_update.js.erb'
  end

  # Replays the specified task
  def replay
    @task = Mdm::Task.find(params[:id])
    case @task.settings[:task_type]
    when 'ScanTask'
      redirect_to new_scan_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'NexposeTask'
      redirect_to new_nexpose_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'BruteforceTask'
      redirect_to new_bruteforce_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'ExploitTask'
      redirect_to new_exploit_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'CollectEvidenceTask'
      redirect_to new_collect_evidence_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'CleanupTask'
      redirect_to new_cleanup_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'ReportTask'
      redirect_to new_report_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'WebauditTask'
      redirect_to new_webaudit_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'WebscanTask'
      redirect_to new_webscan_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    when 'WebsploitTask'
      redirect_to new_websploit_path(:workspace_id => @task.workspace.id, :clone_task => @task.id)
    else
      render :text => "<h1>Unable to replay this task type</h1>"
    end
  end

  def stop_all
    Mdm::Task.running.each { |t| t.stop }

    #render :update do |page|
    #  page.reload
    #end
    render :js => 'location.reload(true);'
  end

  # Shows a form for creating a new Scan task
  def new_scan
    addresses = if params[:host_ids]
      if params[:host_ids].blank?
        @workspace.hosts.map(&:address)
      else
        @workspace.hosts.find(params[:host_ids]).map(&:address)
      end
    elsif params[:target_addresses]
      [params[:target_addresses]]
    else
      @workspace.addresses
    end
    @task = ScanTask.new( cloned_settings || {
      :addresses => addresses,
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Host and Service Discovery"
  end

  # Starts a new Scan task
  def start_scan
    process_task_config(:scan, params[:scan_task], true)
  end

  def start_campaign
    @licensed = License.get.supports_campaigns?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end
    campaign = Campaign.find(params[:campaign_id])
    @task = CampaignTask.new({
      :username => current_user.username,
      :workspace => @workspace,
      :campaign => campaign
    })
    if (t = @task.start)
      #redirect_to show_workspace_campaign_path(@workspace, campaign)
      redirect_to task_detail_path(t.workspace, t)
      #render_new_task(@task)
    else
      render_task_errors(@task)
    end
  end

  # Shows a popup with a form for creating a new Import task
  def new_import
    @task = ImportTask.new(
      :workspace => @workspace,
      :username  => current_user.username
    )
    @title = "Data Import"
  end

  # Starts a new Import task
  def start_import
    process_task_config(:import, params[:import_task], true)
  end

  # Shows a popup with a form for creating a new Exploit task
  def new_exploit
    hosts = _get_hosts_by_ids_or_search(params[:host_ids], params[:search])

    @task = ExploitTask.new( cloned_settings || {
      :whitelist => hosts.map(&:address),
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Automated Exploitation"
    @licensed = License.get.supports_exploit?
  end

  # Start a new Exploit task
  def start_exploit
    @licensed = License.get.supports_exploit?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end
    process_task_config(:exploit, params[:exploit_task], true)
  end


  # Shows a popup with a form for creating a new NexposeAssetGroupPush task
  def new_nexpose_asset_group_push
    hosts = _get_hosts_by_ids_or_search(params[:host_ids], params[:search])
    @task = NexposeAssetGroupPushTask.new( cloned_settings || {
      :tag_ids   => params[:tag_ids],
      :whitelist => hosts.map(&:address),
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Nexpose Asset Group Push"
    @licensed = License.get.supports_advanced_nexpose?
  end

  # Start a new Exploit task
  def start_nexpose_asset_group_push
    @licensed = License.get.supports_advanced_nexpose?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end
    process_task_config(:nexpose_asset_group_push, params[:nexpose_asset_group_push_task], true)
  end

  # Shows a popup with a form for creating a new NexposeExceptionPush task
  # XXX: Use VulnIDs not HostIDs
  def new_nexpose_exception_push
    @task = NexposeExceptionPushTask.new( cloned_settings || {
      :vuln_ids  => params[:vuln_ids],
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Nexpose Exception Push"
    @licensed = License.get.supports_advanced_nexpose?
  end

  # Start a new Exploit task
  def start_nexpose_exception_push
    @licensed = License.get.supports_advanced_nexpose?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end
    process_task_config(:nexpose_exception_push, params[:nexpose_exception_push_task], true)
  end

  # Shows a popup with a form for creating a new Replay
  def new_replay
    @licensed = License.get.supports_replay?
    @task = ReplayTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Automated Attack Replay"
  end

  # Start a new Replay task
  def start_replay
    @licensed = License.get.supports_replay?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end

    @replay_task = ReplayTask.new(params[:replay_task].merge({:username => current_user.username}))

    if (@task = @replay_task.start)
      redirect_to(task_detail_path(@task.workspace, @task.id))
    else
      @task = @replay_task
      render "new_replay"
    end
  end

  # Shows a popup with a form for creating a new Websploit task
  def new_websploit
    @licensed = License.get.supports_webapp_exploitation?

    hosts = nil
    if params[:host_ids]
      hosts = @workspace.hosts.find(params[:host_ids])
    end

    if params[:site_ids]
      hosts = Mdm::WebSite.find(params[:site_ids]).map{|w| w.service.host}.select{|host| host.workspace_id == @workspace[:id]}
    end

    hosts ||= @workspace.hosts

    @title = "Web Application Vulnerability Exploitation"
    @task  = WebsploitTask.new( cloned_settings || {
      :whitelist => hosts.map(&:address),
      :workspace => @workspace,
      :username  => current_user.username
    })
  end

  # Start a new Websploit task
  def start_websploit
    @licensed = License.get.supports_webapp_exploitation?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end

    @websploit_task = WebsploitTask.new(params[:websploit_task].merge({:username => current_user.username}))
    if (@task = @websploit_task.start)
      render_new_task(@task)
    else
      render_task_errors(@websploit_task)
    end
  end

  # Shows a popup with a form for creating a new Webscan task
  def new_webscan
    @licensed = License.get.supports_webapp_exploitation?

    hosts = nil
    if params[:host_ids]
      hosts = @workspace.hosts.find(params[:host_ids])
    end

    if params[:site_ids]
      hosts = Mdm::WebSite.find(params[:site_ids]).map{|w| w.service.host}.select{|host| host.workspace_id == @workspace[:id]}
    end

    hosts ||= @workspace.hosts

    @title = "Web Application Reconnaissance"
    @task  = WebscanTask.new( cloned_settings || {
      :whitelist => hosts.map(&:address),
      :targeted  => params[:host_ids] ? true : false,
      :workspace   => @workspace,
      :username  => current_user.username
    })
  end

  # Start a new Webscan task
  def start_webscan
    @licensed = License.get.supports_webapp_exploitation?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end
    process_task_config(:webscan, params[:webscan_task], true)
  end

  # Shows a popup with a form for creating a new Webaudit task
  def new_webaudit
    @licensed = License.get.supports_webapp_exploitation?

    hosts = nil
    if params[:host_ids]
      hosts = @workspace.hosts.find(params[:host_ids])
    end

    if params[:site_ids]
      hosts = Mdm::WebSite.find(params[:site_ids]).map{|w| w.service.host}.select{|host| host.workspace_id == @workspace[:id]}
    end

    hosts ||= @workspace.hosts.all

    @title = "Web Application Vulnerability Scanning"
    @task  = WebauditTask.new( cloned_settings || {
      :whitelist => hosts.map(&:address),
      :workspace => @workspace,
      :username  => current_user.username
    })
  end

  # Start a new Webaudit task
  def start_webaudit
    @licensed = License.get.supports_webapp_exploitation?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end

    @webaudit_task = WebauditTask.new(params[:webaudit_task].merge({:username => current_user.username}))
    if (@task = @webaudit_task.start)
      render_new_task(@task)
    else
      render_task_errors(@webaudit_task)
    end
  end

  # Shows a popup with a form for creating a new Bruteforce task
  def new_bruteforce
    hosts = _get_hosts_by_ids_or_search(params[:host_ids], params[:search])

    @imported_count = count_imported_creds
    @task = BruteforceTask.new( cloned_settings || {
      :whitelist => hosts.map(&:address),
      :workspace => @workspace,
      :username  => current_user.username
    })
    @inactive_count = 0
    @workspace.creds.each {|cred| @inactive_count += 1 unless cred.active}
    @title = "Authentication Bruteforce"
    @licensed = License.get.supports_bruteforce?
  end

  # Start a new Bruteforce task
  def start_bruteforce
    @licensed = License.get.supports_bruteforce?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end

    process_task_config(:bruteforce, params[:bruteforce_task], true)
  end

  # Shows a popup with a form for creating a new Nexpose task
  def new_nexpose
    addresses = if params[:host_ids]
                  if !params[:host_ids].blank?
                    @workspace.hosts.find(params[:host_ids]).map(&:address)
                  else
                    @workspace.hosts.map(&:address)
                  end
                else
                  @workspace.addresses
                end

    pthcred = ""
    @workspace.creds.each do |cred|
      case cred.ptype
      when "smb_hash"
        pthcred += "%s:%s\n" % [cred.user, cred.pass]
      end
    end

    @task = NexposeTask.new( cloned_settings || {
      :whitelist => addresses,
      :pthcred   => pthcred,
      :workspace => @workspace,
      :username  => current_user.username,
      :nexpose_creds_type => current_user.nexpose_creds_type,
      :nexpose_creds_user => current_user.nexpose_creds_user,
      :nexpose_creds_pass => current_user.nexpose_creds_pass
    })
    @title = "Nexpose Vulnerability Management"
  end

  # Start a new Nexpose task
  def start_nexpose
    process_task_config(:nexpose, params[:nexpose_task], true)
  end

  # Shows a popup with a form for creating a new CollectEvidence task
  def new_collect_evidence
    @sess_ids = (params[:sess_ids] || []).map{|x| x.to_i}
    @task = CollectEvidenceTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Compromised Host Evidence Collection"
    @licensed = License.get.supports_collect?		
  end

  def start_collect_evidence
    @licensed = License.get.supports_collect?		
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end
    process_task_config(:collect_evidence, params[:collect_evidence_task], true)
  end


  # Shows a popup with a form for creating a new Cleanup task
  def new_cleanup
    @sess_ids = (params[:sess_ids] || []).map{|x| x.to_i}
    @task = CleanupTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Compromised Host Cleanup"
  end

  def start_cleanup
    process_task_config(:cleanup, params[:cleanup_task], true)
  end
  
  def new_upgrade_sessions
    @sess_ids = (params[:sess_ids] || []).map{|x| x.to_i}
    @task = UpgradeSessionsTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Windows CMD Shell Upgrade"
  end

  def start_upgrade_sessions
    process_task_config(:upgrade_sessions, params[:upgrade_session_task], true)
  end
  

  def new_module_run
    @clone           = params[:clone_id]
    @target_hosts    = [params[:target_host] || nil].compact
    @options         = params[:options] || {}
    @sessions        = params[:sessions]

    if params[:host_ids]
      @target_hosts = Mdm::Host.find(params[:host_ids]).map{|host| host.address}
    end


    if @clone
      # CloneID is either a session_open or a module_run event ID
      ev = @workspace.events.find(@clone)

      if not (ev and ev.info and ev.info[:datastore])
        render :text => "<h1>Missing event information</h1>"
        return
      end

      @options = ev.info[:datastore]


      # This may be deprecated after #4140, not 100% sure tho.
      if ev.name == "session_open"
        @module          = MsfModule.find_by_fullname( ev.info[:via_exploit] )
        @target_hosts    << (ev.info[:target_host] || ev.info[:tunnel_peer] || '').split(':')[0]
      end

      if ev.name == "module_run"
        @module          = MsfModule.find_by_fullname( ev.info[:module_name] )
        @target_hosts    << @options['RHOST'] || @options['RHOSTS'] || ''
      end
    else
      @module = find_module_by_path
    end
    if @target_hosts.length == 0
      @target_hosts = @workspace.addresses
    end

    @options['RPORT'] = params[:target_port].to_i if params[:target_port]

    @task = ModuleRunTask.new( cloned_settings || {
      :module  => @module,
      :workspace => @workspace,
      :options => @options,
      :target  => @module.default_target,
      :sessions => @sessions
    })
  end

  def session_reopen
    @session = Mdm::Session.find(params[:id])

    @options = @session.datastore
    @module  = MsfModule.find_by_fullname(@session.via_exploit)
    @target_hosts = [@options['RHOST'] || @options['RHOSTS'] || '']

    @task = ModuleRunTask.new(
      :module  => @module,
      :workspace => @workspace,
      :options => @options,
      :target  => @module.default_target,
      :sessions => @sessions
    )

    render :action => 'new_module_run'
  end

  def start_module_run
    process_task_config(:module_run, params[:module_run_task], true)
  end

  # Shows a popup with a form for creating a new Report task
  def new_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Report Generator"
    @licensed = License.get.supports_reports?
  end


  # Shows a popup with a form for creating a new Mdm::Report task
  def new_audit_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Audit Report Generator"
    @licensed = License.get.supports_reports?
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_export
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Export Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_pci_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "PCI Findings Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_fisma_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "FISMA Findings Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_services_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Network Service Report Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_auth_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Authentication Tokens Report Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_compromised_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Compromised Hosts Report Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_custom_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Custom Report Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_webapp_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "WebApp Report Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_webapp_assessment_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Web Application Assessment Report Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_campaigns_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Campaigns Report Generator"
    @licensed = License.get.supports_reports?		
  end

  # Shows a popup with a form for creating a new Mdm::Report task
  def new_loot_report
    @task = ReportTask.new( cloned_settings || {
      :workspace => @workspace,
      :username  => current_user.username
    })
    @title = "Evidence Report Generator"
    @licensed = License.get.supports_reports?		
  end


  def start_report
    @licensed = License.get.supports_reports?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
    end
    process_task_config(:report, params[:report_task], true)
  end


  # Shows a popup with a form for uploading new credentials.
  def new_import_creds
    @imported_count = count_imported_creds
    @title = "Manage Credentials"
    @task = ImportCredsTask.new(
      :workspace => @workspace,
      :username => current_user.username
    )
  end

  def start_import_creds
    temp = nil
    path = nil
    orig_file_name = nil
    if params[:import_creds_task] and params[:import_creds_task][:file]
      # Write the uploaded file to a tempfile
      temp = ::Rex::Quickfile.new('import_creds')
      uploaded_io = params[:import_creds_task][:file]
      orig_file_name = uploaded_io.original_filename rescue nil
      begin
        while (buff = uploaded_io.read(1024*64))
          temp.write(buff)
        end
      rescue ::EOFError
      end
      temp.flush
    else
      temp = nil
    end
    task_attribs = params[:import_creds_task]
    task_attribs[:path] = (temp ? temp.path : nil)
    task_attribs[:orig_file_name] = orig_file_name
    task_attribs[:workspace] = @workspace
    task_attribs[:username] = current_user.username
    @import_creds_task = ImportCredsTask.new( task_attribs)
    temp = nil
    if (@task = @import_creds_task.start)
      # Hang for a sec while the Mdm::CredFile record gets saved.
      select(nil,nil,nil,1.1)
      flash[:notice] = "Credentials imported."
    else
      flash[:error] = @import_creds_task.error
    end
    if params[:task_chains_view]
      render :partial => "task_chains/cred_file_upload", :layout => false
      flash.clear
    else
      redirect_to :action => :new_bruteforce
    end
  end

  # Ends up doing a bunch of things, it's all mashed together
  # because todb is still terribad at editing routes. He's okay
  # at regular ruby so there's no excuse for all the copypasta.
  def update_cred_file
    if params["commit"] =~ /^Update\s/ # Updating info about an existing cred file
      update_params = params[:import_creds_task]

      if update_params
        begin
          cred_file = CredFile.find(update_params[:cred_file_id])
        rescue ActiveRecord::RecordNotFound => error
          flash[:error] = error.to_s
        else
          cred_file.name = update_params[:name]
          cred_file.desc = update_params[:desc]
          cred_file.ftype = update_params[:file_type]

          if cred_file.save
            flash[:notice] = "Updated #{cred_file.display_name}"
          else
            flash[:error] = "Could not update #{cred_file.display_name}"
          end
        end
      else
        flash[:error] = 'No import_creds_task given'
      end
    elsif params["commit"] =~ /^Delete\s/ # Delete an existing cred file
      update_params = params[:import_creds_task]

      if update_params
        begin
          cred_file = CredFile.find(update_params[:cred_file_id])
        rescue ActiveRecord::RecordNotFound => error
          flash[:error] = error.to_s
        else
          cred_file.delete
          ::File.unlink(cred_file.path) rescue nil

          flash[:notice] = "Deleted #{cred_file.display_name}"
        end
      else
        flash[:error] = 'No import_creds_task given'
      end
    elsif params["commit"] =~ /^Download\s/ # Download an existing cred file
      update_params = params[:import_creds_task]

      if update_params
        begin
          cred_file = CredFile.find(update_params["cred_file_id"])
        rescue ActiveRecord::RecordNotFound => error
          flash[:error] = error.to_s
        else
          data = ''

          begin
            File.open(cred_file.path, 'rb') do |f|
              data = f.read
            end
          rescue Errno::ENOENT => error
            flash[:error] = error.to_s
          else
            filename = File.basename(cred_file.path)

            send_data(
                data,
                :disposition => 'attachment',
                :filename => filename,
                :type => 'text/plan'
            )

            # return to skip redirect
            return
          end
        end
      else
        flash[:error] = 'No import_creds_task given'
      end
    else
      flash[:error] = "Unknown action: #{params["commit"]}"
    end

    redirect_to :action => :new_import_creds
  end

  # Shows a popup with a form for uploading a new Mdm::Report Template
  def new_report_template
    @task = ReportTemplateTask.new(
      :workspace => @workspace,
      :username  => current_user.username
    )
    render_popup("Upload a Custom Report Template", "new_report_template")
  end

  def reserved_filenames
    [
      REPORTING_DEFAULT_JRXML,
      ReportsController::SIMPLE_TEMPLATE_FILENAME,
    ] | %w{
        msfx_actv1.jasper
        msfx_actv1.jrxml
        msfx_auth_tokens.jrxml
        msfx_campaigns_details_email.jasper
        msfx_campaigns_details_email.jrxml
        msfx_campaigns_details.jasper
        msfx_campaigns_details.jrxml
        msfx_campaigns_details_targets.jasper
        msfx_campaigns_details_targets.jrxml
        msfx_campaigns_details_web.jasper
        msfx_campaigns_details_web.jrxml
        msfx_campaigns.jrxml
        msfx_compromised_hosts.jrxml
        msfx_compromised_vulns_table.jasper
        msfx_compromised_vulns_table.jrxml
        msfx_fismav1.jasper
        msfx_fismav1.jrxml
        msfx_fismav1_subreport_hosts.jasper
        msfx_fismav1_subreport_hosts.jrxml
        msfx_loot_images.jasper
        msfx_loot_images.jrxml
        msfx_loot.jrxml
        msfx_loot_texts.jasper
        msfx_loot_texts.jrxml
        msfx_pciv1.jasper
        msfx_pciv1.jrxml
        msfx_pciv1_subreport_hosts.jasper
        msfx_pciv1_subreport_hosts.jrxml
        msfx_services.jrxml
        msfxv3_compromised_hosts_table.jasper
        msfxv3_compromised_hosts_table.jrxml
        msfxv3_detail_creds.jasper
        msfxv3_detail_creds.jrxml
        msfxv3_detail_exploits.jasper
        msfxv3_detail_exploits.jrxml
        msfxv3_detail_services.jasper
        msfxv3_detail_services.jrxml
        msfxv3_detail_session_events.jasper
        msfxv3_detail_session_events.jrxml
        msfxv3_detail_sessions.jasper
        msfxv3_detail_sessions.jrxml
        msfxv3_details.jasper
        msfxv3_details.jrxml
        msfxv3_detail_tags.jasper
        msfxv3_detail_tags.jrxml
        msfxv3_detail_vulns.jasper
        msfxv3_detail_vulns.jrxml
        msfxv3_detail_web.jasper
        msfxv3_detail_web.jrxml
        msfxv3.jrxml
        msfxv3_webapp_detail_forms.jasper
        msfxv3_webapp_detail_forms.jrxml
        msfxv3_webapp_detail_pages.jasper
        msfxv3_webapp_detail_pages.jrxml
        msfxv3_webapp_details.jasper
        msfxv3_webapp_details.jrxml
        msfxv3_webapp_detail_vulns.jasper
        msfxv3_webapp_detail_vulns.jrxml
        msfxv3_webapp.jrxml
    }

  end

  # Validates the uploaded file. This skips the whole rpc_call procedure
  # since we're not going to be bothering with task ID's. As such, it's
  # temporary.
  # XXX: Really need to protect against cmd injection here.
  def validate_report_template_file()
    # Check the license.
    unless License.get.supports_custom_reporting?
      return {:error => "Custom reporting unavailable in this version of Metasploit."}
    end

    unless params[:report_template_task] and params[:report_template_task][:file]
      return {:error => "No template file specified."}
    end

    if reserved_filenames.include? params[:report_template_task][:file].original_filename
      return {:error => "Reserved default filename. Please rename your file before uploading."}
    end

    unless params[:report_template_task][:file].original_filename[/\.(gif|png|jpg|jpeg|jrxml|jasper)$/i]
      return {:error => "Unknown file extension. Must be one of: GIF, PNG, JPG, JPEG, JRXML, or JASPER."}
    end

    data = params[:report_template_task][:file].read
    # Check for serialized jasperreport data, jpg, png, gif, or unseralized jasper XML.
    # This is mostly to prevent mistakes and not prevent malicious uploads, since only
    # admins should be able to upload these anyway.
    filetype_regex = /^\xac\xed\x00\x05.{1,64}net\.sf\.jasperreports|^\xff\xd8\xff\xe0\x00\x10\x4a\x46\x49\x46\x00|^\x89PNG|^GIF8[97]|jasperreports\.sourceforge/
      unless data[filetype_regex]
        return {:error => "File is not an acceptable JasperReport collateral file type."}
      end

    return {:success => true}
  end

  # XXX: Fix this
  def start_report_template
    validate_result = validate_report_template_file()
    if validate_result[:success]
      # Add the file to the filesystem
      uploaded = params[:report_template_task][:file]
      fname = uploaded.original_filename
      # Bonus of making this a module: Can use a proper pro_data_directory
      # from the Locations.
      d = ::File.expand_path(::File.dirname(__FILE__))
      rt_path = File.join(d, "..","..","..","reports",fname)
      uploaded.rewind
      ::File.open(rt_path, "wb") { |f| f.write(uploaded.read) }
      # Figure out the descriptive name
      report_name = params[:report_template_task][:report_name]
      if report_name.nil? || report_name.empty?
        report_name = uploaded.original_filename
      end
      # Record the file in the database. Note that since we're not
      # kicking of a real task with a task_id, we can't actually
      # do any validation down in rpc/core#start_report_template.
      # This is kind of a bummer, and XXX should be fixed.
      t = ReportTemplateTask.new(params[:report_template_task].merge({
        :username => current_user.username,
        :rt_path => rt_path, :report_name => report_name}))
        t.start # Assume all is well, since we're not kicking off a real task.
        redirect_to new_custom_report_path
    else
      flash[:error] = validate_result[:error]
      redirect_to new_custom_report_path
    end
  end

  # Shows a popup with a form for creating a new Tunnel task
  def new_tunnel
    @licensed = License.get.supports_vpn_pivot?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end

    @interface_names = %W{ }
    @sess = params[:sess_id]
    interfaces = {}

    @c = Pro::Client.get

    perr = nil
    begin
      res = @c.call("pro.meterpreter_tunnel_interfaces", @sess)
      if res and res["result"] == "success" and res["interfaces"]
        interfaces = res["interfaces"]
      else
        perr = res["error"].to_s
      end
    rescue ::Exception => e
    end

    valid = {}
    interfaces.each_key do |k|
      int = interfaces[k]
      next if not (int['address'] and int['netmask'])
      next if int['address'] == '0.0.0.0'

      # Determine what source address we would use to route to this network
      saddr = Rex::Socket.source_address(int['address'])
      scidr = Rex::Socket.addr_atoc(int['netmask']) rescue nil
      next if not scidr

      # Does this address fall within the same subnet of the remote interface?
      range = Rex::Socket::RangeWalker.new("#{saddr}/#{scidr}")
      next if not range
      next if range.include?(int['address'])
      valid[k] = int
    end

    @task = TunnelTask.new(
      :workspace        => @workspace,
      :username       => current_user.username,
      :interfaces     => valid,
      :all_interfaces => interfaces,
      :error          => perr
    )
    render_popup("Network Tunnel Configuration", "new_tunnel")
  end

  def start_tunnel
    @licensed = License.get.supports_vpn_pivot?
    if not @licensed
      render :text => "<h1>Unlicensed</h1>"
      return
    end

    @tunnel_task = TunnelTask.new(params[:tunnel_task].merge({:username => current_user.username}))
    if (@task = @tunnel_task.start)
      render_new_task(@task)
    else
      render_task_errors(@tunnel_task)
    end
  end

  def count_imported_creds
    Mdm::ImportedCred.find_all_by_workspace_id(@workspace.id).size
  end

  def delete_imported_creds
    count = count_imported_creds
    if count.zero?
      flash[:notice] = "Deleted 0 credentials."
    else
      Mdm::ImportedCred.destroy_all(:workspace_id => @workspace.id)
      flash[:notice] = "Deleted #{count} credentials."
    end
    redirect_to :action => :new_bruteforce
  end

  def download_imported_creds(proj=nil)
    creds = Mdm::ImportedCred.find_all_by_workspace_id(@workspace.id)
    ext = "txt"
    dis ="attachment"
    typ = "text/plain"
    text_file = Rex::Quickfile.new("imported_creds")
    text_file.write "# Imported Credentials, exported #{Time.now} from '#{@workspace.name}'\r\n"
    text_file.write "#{'#' * 80}\r\n"
    creds.each do |cred|
      case cred.ptype
      when "smb_hash"
        text_file.write "%s:%d:%s:::\r\n" % [cred.user, cred.id, cred.pass]
      when "password"
        text_file.write cred.pass + "\r\n"
      when "userpass"
        text_file.write "%s %s\r\n" % [cred.user, cred.pass]
      end
    end
    text_file.flush

    # Just how awful is this? I don't see any problems with it
    # but I'm suspicious...
    Thread.new { select(nil,nil,nil,60)
      text_file.unlink rescue nil
    }
    send_data(
      File.open(text_file.path, "rb") { |f| f.read },
      :type => typ,
      :disposition => dis,
      :filename => @workspace.name + "-" + "imported_creds" + "." + ext
    )
  end

  private

  def render_new_task(task)
    if request.xhr?
      render_js_redirect(task_detail_url(task.workspace, task))
    else
      redirect_to task_detail_url(task.workspace, task)
    end
  end
    
    
  def render_task_errors(task)
    render 'errors_update', :locals => { :task => task }
  end

  def _get_hosts_by_ids_or_search(ids, search)
    if ids
      @workspace.hosts.find(ids)
    elsif search
      search_terms = Shellwords.shellwords(search) rescue search.split(/\s+/)

      hosts = @workspace.hosts
      search_terms.each do |term|
        if term =~ /^#(.*)/ and License.get.supports_tags?
          hosts = hosts.tag_search($1)
        else
          hosts = hosts.search(term)
        end
      end
      hosts
    else
      @workspace.hosts.all
    end
  end

  def cloned_settings
    return nil if not params[:clone_task]
    clone = Mdm::Task.find( params[:clone_task] )
    return nil if not clone
    return nil if not clone.settings
    return nil if not clone.settings[:task_type]
    settings = clone.settings.dup
    settings.delete(:task_type)

    # Fix up address strings
    if settings[:address_string]
      settings[:addresses] = settings.delete(:address_string).split(/\s+/)
    end

    # Fix up booleans for checkboxes
    settings.each_pair do |k,v|
      settings[k] = false if v == "false"
      settings[k] = true  if v == "true"
    end

    settings
  end
end

