class UpdatesController < ApplicationController

	skip_before_filter :require_license, :only => [:status, :restart]
	before_filter :require_admin

	def updates
		@status = UpdateStatus.get
		if @status.in_progress?
			render :action => :in_progress
		else
			@license = License.get
			set_http_proxy
			render :action => :updates
		end
	end

	# XHR request
	def status
		@status = UpdateStatus.get
		render :partial => 'status'
	end

	# XHR request
	def check
		proxy_params = if params[:use_proxy]
			@current_user.update_attributes(params[:current_user])
			{ :proxy_host => @current_user.http_proxy_host,
			:proxy_port => @current_user.http_proxy_port,
			:proxy_user => @current_user.http_proxy_user,
			:proxy_pass => @current_user.http_proxy_pass }
		else
			{}
		end
		@update = UpdateCheck.new(proxy_params)
		render :partial => 'update', :locals => { :update => @update }
	end

	# Starts the update install
	def apply
		proxy_params = if params[:use_proxy]
			@current_user.update_attributes(params[:current_user])
			{ :proxy_host => @current_user.http_proxy_host,
			:proxy_port => @current_user.http_proxy_port,
			:proxy_user => @current_user.http_proxy_user,
			:proxy_pass => @current_user.http_proxy_pass }
		else
			{}
		end
		@update = UpdateCheck.new(proxy_params)
		success = @update.apply
		if not success
			flash[:error] = "Update failed to start: #{@update.error}"
			redirect_to :action => :updates
		else
			@status = UpdateStatus.get
			render :action => :in_progress
		end
	end

	def offline_apply
		@license = License.get

		unless UpdateOffline.supported?
			redirect_to :action => :updates
		end

		temp = nil
		if params[:update_file]
			# Copy the uploaded file to a tempfile
			temp = ::Rex::Quickfile.new('update')
			uploaded_io = params[:update_file]
			
			begin
				while (buff = uploaded_io.read(1024*64))
					temp.write(buff)
				end
			rescue ::EOFError
			end
			
			temp.flush
		end
		
		if not temp
			return
		end
		
		@update = UpdateOffline.new
		
		# Explicitly close the file to prevent locking issues
		path = temp.path
		temp.close rescue nil
		
		success = @update.apply(path)
		if not success
			flash[:error] = "Update failed to start: #{@update.error}"
			redirect_to :action => :offline_apply
		else
			@status = UpdateStatus.get
			render :action => :in_progress
		end
	end
	
	# Restarts prosvc
	def restart
		@status = UpdateStatus.get
		success = @status.restart
		if not success
			flash[:error] = "Failed to restart"
		end
		redirect_to :action => :updates
	end

	private

	# Defaults the http proxy settings to the globally stored proxy settings.
	#
	# Returns nothing.
	def set_http_proxy
		@current_user.http_proxy_host = current_profile.settings["http_proxy_host"] if @current_user.http_proxy_host.blank?
		@current_user.http_proxy_port = current_profile.settings["http_proxy_port"] if @current_user.http_proxy_port.blank?
		@current_user.http_proxy_user = current_profile.settings["http_proxy_user"] if @current_user.http_proxy_user.blank?
		@current_user.http_proxy_pass = current_profile.settings["http_proxy_pass"] if @current_user.http_proxy_pass.blank?
	end

end
