class UserSessionsController < ApplicationController

	skip_before_filter :require_user
	skip_before_filter :require_license
	layout "login"

	def new
		@user_session = UserSession.new
		if FF_NEW_BRANDING
			render :new_branding
		end
	end

	def create
		@user_session = UserSession.new(params[:user_session])

		remote_ip = request.remote_ip
		user_agent = request.env['HTTP_USER_AGENT'].to_s

		if @user_session.save
			user = @user_session.record
			user.last_login_address = remote_ip
			user.session_key = session[:session_id]
			user.save!
			
			login_event(user.username, true, remote_ip, user_agent)
			
			redirect_back_or_default root_path
		else
			login_event(@user_session.username, false, remote_ip, user_agent)
			
			flash[:error] = "Invalid username or password."
			redirect_to login_path
		end 
	end

	def destroy
		@user_session = UserSession.find

		logout_event(@user_session.record.username, request.remote_ip, request.env['HTTP_USER_AGENT'].to_s)

		@user_session.destroy
		flash[:notice] = "You have been logged out."
		redirect_to root_path
	end

	private

	def login_event(username, success, remote_ip, user_agent)
		ev = Mdm::Event.new(:name => "user_login",
		               :username => username,
		               :info => {:success => success,
		                         :address => remote_ip,
		                         :user_agent => user_agent})
		ev.save!
	end

	def logout_event(username, remote_ip, user_agent)
		ev = Mdm::Event.new(:name => "user_logout",
		               :username => username,
		               :info => {:address => remote_ip,
		                         :user_agent => user_agent})
		ev.save!
	end

end
