require 'uri'
require 'rex/socket'
require 'rex/proto/http'

class WebTemplatesController < ApplicationController
	before_filter :load_workspace

	def index
		@web_templates = WebTemplate.all(:order=>:name)
	end

	def new
		@web_template = WebTemplate.new
		@web_templates = @workspace.web_templates.all(:order => :name)
		@web_template.name = "Automatic-#{Time.now.to_i}"
		@web_template.exploit_type = "autopwn"
		build_exploit_list
	end

	def edit
		@web_template = WebTemplate.find(params[:id])
		@web_templates = @workspace.web_templates.all(:order => :name) - [@web_template]
		build_exploit_list
		@web_template.exploit_name ||= @exploit_list[0].fullname
	end

	def update
		@web_template = WebTemplate.find(params[:id])
		if (params[:web_template][:clone].blank?)
			# We're not cloning, so save exploit options
			case (params[:web_template][:exploit_type])
			when "single"
				params[:web_template][:exploit_opts] = params[:options]
			else
				params[:web_template].delete(:exploit_opts)
			end
			attrs = params[:web_template]
		else
			# We are cloning, go grab the thing to clone
			begin
				t = WebTemplate.find(params[:web_template][:clone])
				attrs = t.attributes
			rescue ActiveRecord::RecordNotFound
				flash[:error] = "Template could not be found for cloning"
			end
		end

		if attrs
			attrs.delete :clone
			# Don't save the campaign_id from the template we're cloning
			attrs.merge!(:campaign_id => @web_template.campaign.id)
		end

		if attrs and @web_template.update_attributes(attrs)
			flash[:notice] = "Web template updated"
			redirect_to workspace_campaign_url(@workspace, @web_template.campaign)
		else
			flash[:error] = "Failed to update template"
			redirect_to edit_workspace_web_template_url(@workspace, @web_template)
		end
	end

	def create
		@web_template = WebTemplate.new(params[:web_template])
		if @web_template.save!
			flash[:notice] = "Web template created"
			redirect_to workspace_web_templates_url(@workspace)
		else
			render :action => 'edit'
		end
	end

	def clone
		uri = ::URI.parse( params[:clone_url] ) rescue nil unless params[:clone_url].empty?
		if not uri
			render :text => 'Invalid URL specified'
			return
		end

		cli = Rex::Proto::Http::Client.new( uri.host, uri.port, {}, uri.scheme == 'https' )
		if not cli
			render :text => 'Could not configure HTTP for this URL'
			return
		end
		
		cli.set_config(
			:vhost => uri.host,
			:agent => 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)'
		)

		$stderr.puts uri.inspect
		
		req = cli.request_raw(
			'uri'   => uri.path,
			'query' => uri.query
		)

		con = cli.connect rescue nil
		if not con
			render :text => 'Could not connect to this host'
			return
		end
		
		res = cli.send_recv(req, 10) rescue nil
		if not res
			render :text => 'No response from the host'
			return
		end
		
		render :text => res.body.to_s
	end

	def exploit_list
		build_exploit_list
		respond_to do |format|
			format.html
			format.js {
				render :update do |page|
					page.replace "exploit_list", :partial => "exploit_list", :object => @exploit_list
				end
			}
		end
	end

	def exploit_options
		m = MsfModule.find_by_fullname(params[:module_name])
		render :partial => 'module_fields', :locals => { :msf_module => m, :web_template => nil }
	end

	def preview
		body = params[:web_template][:body] || ''
		if params[:web_template][:clone].present?
			other_tpl = WebTemplate.find(params[:web_template][:clone])
			body = other_tpl.body
		end
		render :text => body.reverse, :content_type => 'text/html'
	end

protected

	def build_exploit_list
		@exploit_list = []
		MsfModule.exploits.each do |m|
			if m.fullname =~ /browser/i
				@exploit_list << m
			end
		end

		@exploit_list = @exploit_list.sort_by { |m|
			m.disclosure_date ? m.disclosure_date : Time.new
		}
		@exploit_list.reverse!
	end

end

