module ReportsHelper

	def report_types
		types = [:audit, :compromised, :auth, :service, :loot]
		types << :campaigns if License.get.supports_campaigns?
		types << :webapp_assessment if FF_ARTHAS_WEBSCAN && License.get.supports_webapp_exploitation?
		types << :webapp if License.get.supports_webapp_exploitation?
		types << :pci if License.get.supports_pci?
		types << :fisma if License.get.supports_fisma?
		types.map {|x| report_type_map(x)}
	end

  def report_type_map(type)
    # sometimes we don't care about the route and just want the label
    workspace = @workspace || Mdm::Workspace.first
    case type
    when :audit
      [:audit, "Audit", "Detailed Audit", new_report_path(workspace), "A comprehensive report covering all details of this workspace"]
    when :webapp_assessment
      [:webapp_assessment, "Web App Assessment", "Web Application Assessment", new_webapp_assessment_report_path(workspace), "A report assessing scanned web applications for vulnerabilities"]
    when :webapp
      [:webapp, "WebApp", "Web Application and Vulnerabilities", new_webapp_report_path(workspace), "A report covering all discovered web application vulnerabilities"]
    when :pci
      [:pci, "PCI", "PCI Compliance", new_pci_report_path(workspace), "A report covering all relevant PCI compliance checks for the workspace"]
    when :fisma
      [:fisma, "FISMA", "FISMA Compliance", new_fisma_report_path(workspace), "A report covering all relevant FISMA compliance checks for the workspace"]
    when :service
      [:service, "Services", "Network Services", new_services_report_path(workspace), "A report focused on network services"]
    when :compromised
      [:compromised, "Exploited", "Compromised and Exploited Hosts", new_compromised_report_path(workspace), "A report focused on systems compromised and exploited"]
    when :auth
      [:auth, "Auth", "Authentication Tokens", new_auth_report_path(workspace), "A report focused on the usernames and passwords obtained"]
    when :loot
      [:loot, "Loot", "Collected Evidence", new_loot_report_path(workspace), "A report focused on the data collected from compromised systems"]
    when :campaigns
      [:campaigns, "Campaign", "Social Engineering Campaigns", new_campaigns_report_path(workspace), "A report detailing the results of social engineering campaigns run in this workspace"]
    when :se_campaign_detail
      [:se_campaign_detail, "SE Campaign Details", "Social Engineering Campaign Details", new_campaigns_report_path(workspace), "A report detailing the results of social engineering campaigns run in this workspace"]
    else
      [:unknown, "Unknown", "Unknown #{type.inspect} Report", new_report_path(workspace), "Unimplemented report of type #{type.inspect}"]
    end
  end

	def ref_map(refs,width=3)
		ref_links = refs.map {|r| link_for_ref(*r.link_info)}
		ref_table = []
		td_width = ((1 / width.to_f) * 100).to_i
		ref_table = 0.upto(refs.size-1).collect {|i| ref_links[i,width] if i % width == 0}.compact
		# Allow sorting by number of refs on the client side.
		tbl = "<span title='#{refs.size}'></span>"
		tbl << '<table width="100%">'
		ref_table.each {|tr|
			tbl << "<tr>"
			td_count = 0
			tr.each {|td|
				tbl << "<td style=\"white-space: nowrap;\" width=\"#{td_width}%\">" << td << "</td>"
				td_count += 1
			}
			if tr.size == width 
				tbl << "</tr>"
			else
				(width - (tr.size % width)).times { tbl << "<td width=\"#{td_width}%\"></td>"}
				tbl << "</tr>"
			end
		}
		tbl << "</table>"
		return tbl
	end

end

