module TasksHelper

	def report_types_jquery_list
		rlist = report_available_types.map {|k| "#report_form_#{k}"}.join(",")
	end

  # Assigns desired name to be shown in web UI report selector
	def report_types_collection
		types = report_available_types
		types_collection = {}
		types.each do |t|
			desc = case t
			when :compromised
				"Compromised and Vulnerable Hosts"
			when :auth
				"Authentication Tokens"
			when :loot
				"Collected Evidence"
			when :fisma, :pci
				"#{t.upcase} Compliance"
      when :se_campaign_detail
        "Social Engineering Campaign Details"
      when :campaigns
        if FF_SOCIAL_ENGINEERING
          "Pre-4.5 Campaigns (Deprecated)"
        else
          "Campaigns"
        end
      when :webapp_assessment
        "Web Application Assessment"
			else
				"#{t.capitalize}"
			end
			types_collection[t] = desc
		end
		types_collection
	end

	def report_available_types
		types = [:audit, :auth, :compromised, :services, :loot]
		types << :campaigns if License.get.supports_campaigns?
		types << :se_campaign_detail if FF_SOCIAL_ENGINEERING
		types << :webapp if License.get.supports_webapp_exploitation?
		types << :webapp_assessment if FF_ARTHAS_WEBSCAN
		types << :pci if License.get.supports_pci?
		types << :fisma if License.get.supports_fisma?
		return types
	end

	# returns a zipped array of symbol and humanized label
	# useful for formastic's :collection option
	def formtastic_report_types_collection
		collection = report_types_collection
		collection.values.zip collection.keys
	end

	def autotag_checkbox(host_tag)
		if host_tag.kind_of? Mdm::Tag
			check_box_tag controller.action_name.split("_",2).last + "_task[autotag_tags][]", host_tag.id, false, :id => "tag_#{host_tag.id}"
		else
			check_box_tag controller.action_name.split("_",2).last + "_task[autotag_tags][]", -1, false, :id => nil 
		end
	end

	def next_report(rtype)
		if Mdm::Task.count > 0
			next_id = Mdm::Task.last.id + 1
		else
			next_id = 1
		end
		rep_name = case rtype
			when nil
				"Report"
			when :loot
				"Evidence"
			else
				rtype.to_s.downcase.capitalize
		end
		"#{rep_name}-#{next_id}"
	end

	def next_task
		if Mdm::Task.count > 0
			next_id = Mdm::Task.last.id + 1
		else
			next_id = 1
		end
		this_action = case controller.action_name
			when "new_scan"
				"discover"
			when "new_collect_evidence"
				"collect"
			when /^new_(.*)/
				$1.gsub(/[^A-Za-z0-9]/,"_")
			when /^(new|edit)$/
				if controller.class.to_s =~ /^Campaign/
					"campaign"
				else
					controller.class.to_s
				end
			else
				controller.action_name
			end
		[this_action,next_id].join("_")
	end

	# add an invisible element that responds to the ".end" selector at the
	# end of a tasklog to mark the task as completed on the client (js) side
	def tasklog_cap(task)
		unless task.running?
			'<span class="end" style="display:none;"></span>'.html_safe
		else
			''
		end
	end

	def tasklog_prettify(line)
		ltype = case line[0,3]
		when '[-]'
			'error'
		when '[+]'
			'good'
		else
			'normal'
		end

		case line
		when /Workspace:.*Progress:/
			ltype = 'status'
		end

		content_tag :div, h(line), :class => "logline_#{ltype}"
	end

	# Generate the options hash for the Nexpose console +input+ call.
	#
	# consoles - The Array of available consoles for selection.
	#
	# Returns the Hash options.
	def nexpose_console_select_options(consoles)
		options = { collection: options_for_select(consoles) }
		if consoles.size > 1
			options.merge! prompt: "Select a Nexpose console below" 
		else
			options.merge! include_blank: false
		end
	end

	def dynamic_exe_hook
		return nil unless License.get.supports_av_evasion?
		if @task.module.fullname == "exploit/windows/smb/psexec"
			exeopt = Option.new(:advanced => true, :default => false, :desc => "Generate a dynamic EXE payload", :name => "DynamicExe", :required => true, :type => "bool")
			@task.module.options.push(exeopt)
		end
		return nil
	end
end
