module WebHelper

	def web_vuln_risk_name(risk_value)
		%W{none info disclosure low medium high}[risk_value.to_i] || "unknown"
	end
	
	def web_vuln_status(vuln)
		rname = web_vuln_risk_name(vuln.risk)
		case vuln.category.to_s.downcase
		when "sql"
			[rname, "SQL Injection", "http://www.owasp.org/index.php/SQL_Injection"]
		when "rfi"
			[rname, "Remote File Include", "http://www.owasp.org/index.php/Top_10_2007-Insecure_Direct_Object_Reference"]
		when "lfi"
			[rname, "Local File Include", "http://www.owasp.org/index.php/Top_10_2007-Insecure_Direct_Object_Reference"]	
		when "cmd"
			[rname, "CMD Injection", "http://www.owasp.org/index.php/OS_Command_Injection"]
		when 'xss'
			[rname, "Cross-site Scripting", "http://www.owasp.org/index.php/Cross-site_Scripting_(XSS)"]		
		else
			[rname, "Information Disclosure", "#"]
		end
	end

	def web_vuln_status_html(vuln)
		risk,text,link = web_vuln_status(vuln)
		content_tag( :div, text, :class => "web_vuln_status_#{h risk}").html_safe
	end
	
	def web_vuln_status_text(vuln)
		risk,text,link = web_vuln_status(vuln)
		text
	end

	def web_vuln_status_risk(vuln)
		risk,text,link = web_vuln_status(vuln)
		risk
	end
	
	def web_vuln_status_link(vuln)
		risk,text,link = web_vuln_status(vuln)
		link
	end

	def web_site_status_html(site)
		risks = site.web_vulns.map{|v| v.risk || 0} || [0]
		risk  = risks.max
		rname = web_vuln_risk_name(risk)
		
		if rname == "none" and site.web_forms.length == 0
			rname = "unaudited"
		end
		
		content_tag(:div, rname.capitalize, :class => "web_vuln_status_#{rname}").html_safe
	end
	
	def web_sites_link(name, args)
		path_args = { :workspace_id => @workspace.id, :page => args[:page], :n => @sites_per_page }
		path_args[:search] = params[:search] if params[:search]
		path_args[:sort_by] = params[:sort_by] if params[:sort_by]
		path_args[:sort_dir] = params[:sort_dir] if params[:sort_dir]
		html_options = args[:html_options] || {}
	 	link_to name, web_sites_path(path_args), html_options
	end	
	
end
