class CredFile < ActiveRecord::Base
  #
  # Constants
  #

  DEFAULT_FILE_TYPE = 'userpass'
  FILE_TYPE_LABEL_BY_FILE_TYPE = {
      'pass' => 'Passwords',
      'pwdump' => 'PWDump',
      'ssh_keys' => 'SSH Keys',
      'user' => 'Usernames',
      'userpass' => 'UserPass'
  }

  #
  # Associations
  #

  belongs_to :workspace, :class_name => 'Mdm::Workspace'

  #
  # Mass Assignment Security
  #

  # User is NOT allowed to set timestamps, created_at and updated_at
  # User is NOT allowed to set created_by as it is derived from current user
  # User is allowed to set metadata about file, such as description (desc), file type (ftype), and name
  attr_accessible :desc
  attr_accessible :ftype
  attr_accessible :name
  # User is NOT allowed to set path as it is the on-disk path can allowing the user to set it would allow them to
  # download any path they want from the server
  # User is NOT allowed to set workspace_id as it is derived from the current workspace

  #
  # Validations
  #

  validates :ftype, :inclusion => { :in => FILE_TYPE_LABEL_BY_FILE_TYPE.keys }
  validates :workspace_id, :presence => true

  #
  # Methods
  #

	def display_name
		if name.present?
			name
    elsif path.present?
      File.basename(path)
    else
      "Nameless and pathless #{self.class}"
		end
	end
end
