class EmailTemplate < ActiveRecord::Base
	attr_reader :error

	belongs_to :campaign
	belongs_to :email_template, :foreign_key => :parent_id
	has_one :email_template, :foreign_key => :parent_id
  validate :ensure_valid_attachments

	serialize :prefs
	extend MetasploitDataModels::SerializedPrefs

	before_save :fix_bools


	# XXX: For some reason, assigning the attachments attribute of one
	# email_template object to another's gives us twice as many as we need.
	# Adding :uniq here fixes the display by ignoring duplicates when fetching,
	# but we still have extra records in the join table that aren't needed
	# XXX: This is a rails bug.  And :uniq doesn't appear to fix it in all
	# situations.  We also have to do +template.attachments.uniq!+ in some
	# places.
	# http://ryanbigg.com/2010/04/has_and_belongs_to_many-double-insert/
	has_and_belongs_to_many :attachments, :uniq => true

	accepts_nested_attributes_for :attachments, :reject_if => lambda { |a| a[:name].blank? }, :allow_destroy => true

	# XXX: This has no meaning anymore now that email templates are completely
	# subordinate to campaigns.  Remove this after confirming that nothing else
	# uses it.
	def master?
		#raise RuntimeError.new("Called deprecated method master?().")
		false
	end

	serialized_prefs_attr_accessor :exploit_module, :exploit_attach_name
	serialized_prefs_attr_accessor :attach_exe
	serialized_prefs_attr_accessor :attach_exploit

	attr_accessible :attach_exe
	attr_accessible :attach_exploit
	attr_accessible :body
	attr_accessible :campaign_id
	attr_accessible :exploit_attach_name
	attr_accessible :exploit_module
	attr_accessible :name
	attr_accessible :prefs
	attr_accessible :subject
	
	# Stolen from campaign model
	def fix_bools
		bools = [ "attach_exe", "attach_exploit" ]

		bools.each do |b|
			if self.send(b) == "0" or self.send(b).blank?
				self.send(b+"=", false)
			else
				self.send(b+"=", true)
			end
		end

		true
	end

private
  def ensure_valid_attachments
		@error = nil
		# Check attachments. TODO: need about a million more validations
		if attachments.present?
      attachments.each do |att|
        next if att.valid?
        @error = att.error
      end
		end
    return @error.blank?
	end

end

