class LPORTMapper 

  attr_accessor :reserved_lports, :allowed_ports

  def initialize
    @reserved_lports =  []
    @allowed_ports = Rex::Socket.portspec_crack("1024-65535")
  end

  def find_usable_lport(ipv6=false)
    lport = nil

    1.upto(allowed_ports.length + 100) do
      lport = configure_payload_listener
      next if reserved_lports.include? lport
      return lport if lport_available?(lport,ipv6)
    end
    return nil
  end

  def lport_available?(lport, ipv6=false)
    client = Pro::Client.get
    client.lport_available?(lport,ipv6)
  end

  def configure_payload_listener
    ports = allowed_ports.dup

    @mutex ||= ::Mutex.new

    ret = nil

    @mutex.synchronize do
      @listener_idx  ||= -1
      ret = ports[ ( @listener_idx += 1 ) % ports.length ]
    end

    ret
  end

  def allowed_ports=(ports)
    if ports.class == String
      raise ArgumentError, "Invalid Port Range Assignment: #{ports}" unless ports =~ /^(\d)+\-(\d)+$/
      ports = Rex::Socket.portspec_crack(ports) 
    end
    if ports_valid?(ports, false)
      @allowed_ports = ports
    else
      raise ArgumentError, "Invalid Port Range Assignment: #{ports}"
    end
  end

  def reserved_lports=(ports)
    if ports.class == String
      raise ArgumentError, "Invalid Port Range Assignment: #{ports}" unless ports =~ /^(\d)+\-(\d)+$/
      ports = Rex::Socket.portspec_crack(ports) 
    end
    if ports_valid?(ports, true)
      @reserved_lports = ports
    else
      raise ArgumentError, "Invalid Port Range Assignment: #{ports}"
    end
  end



  def ports_valid?(aports, can_be_empty = true)
    return false if aports.nil?
    return false unless aports.class == Array 
    unless can_be_empty
      return false if aports.empty?
    end
    aports.each do |ap| 
      return false unless ap.class == Fixnum
      return false if ap > 65535
      return false if ap < 1
    end
    return true
  end

end
