class ReportConfig < Hash

	attr_reader :error

	def initialize(opts={})
		opts.each_pair { |k,v| self[k.to_sym] = v }

		# Set up the defaults
		self[:mask] = false unless self[:mask] == '1'
		self[:tasklog] = false unless self[:tasklog] == '1'
		self[:display_session] = false unless self[:display_session] == '1'
		self[:display_charts] = false unless self[:display_charts] == '1'
		self[:loot_ex_shots] = false unless self[:loot_ex_shots] == '1'
		self[:loot_ex_passes] = false unless self[:loot_ex_passes] == '1'

		if self[:address_string]
			self[:addresses] = tags_and_addresses(self[:address_string].split(/\s+/))
		else
			self[:addresses] = []
		end
	end

    # Dispatches to the real TaskConfig#tags_and_addresses().
    def tags_and_addresses(arr=[])
      TaskConfig.new(self).tags_and_addresses(arr)
    end

	def address_ranges
		return @address_ranges if @address_ranges
		@address_ranges = self[:addresses].map { |str|
			if Rex::Socket::RangeWalker.new(str).valid?
				Rex::Socket::RangeWalker.new(str)
			else
				raise ArgumentError, "Invalid Address"
			end
		}
	end

	def valid?
		begin
			address_ranges
		rescue
			@error = "Invalid IP addresses"
			return false
		end
		true
	end

	def include_address?(address)
		# if we have no addresses, the user wants everything
		return true if address_ranges.empty?
		address_ranges.each do |range|
			if range.include?(address)
				return true
			end
		end
		return false
	end

end

