class ReportTask < TaskConfig
  include Reporting

  REPORT_TYPES =        %W{ AUDIT-PDF AUDIT-WORD AUDIT-RTF AUDIT-HTML}
  REPORT_TYPES_LABELS = %W{ PDF Word RTF HTML}

  EXPORT_TYPES =        ["XML",        "ZIP",         "REPLAY",         "PWDUMP", "ACTIVITY-XML",   "ACTIVITY-PDF",   "ACTIVITY-RTF"]
  EXPORT_TYPES_LABELS = ['XML Export', 'ZIP Workspace', 'Replay Scripts', 'PWDump', "Activity (XML)", "Activity (PDF)", "Activity (RTF)"]

  PCI_TYPES =         %W{ PCI-PDF PCI-RTF PCI-HTML PCI-XML}
  PCI_TYPES_LABELS  = %W{ PDF RTF HTML XML }

  FISMA_TYPES =         %W{ FISMA-PDF FISMA-RTF FISMA-HTML FISMA-XML}
  FISMA_TYPES_LABELS  = %W{ PDF RTF HTML XML }

  SERVICES_TYPES = %W{ SERVICES-PDF SERVICES-WORD SERVICES-RTF SERVICES-HTML}
  SERVICES_TYPES_LABELS = %W{ PDF Word RTF HTML}

  COMPROMISED_TYPES = %W{ COMPROMISED-PDF COMPROMISED-WORD COMPROMISED-RTF COMPROMISED-HTML }
  COMPROMISED_TYPES_LABELS = %W{ PDF Word RTF HTML }

  AUTH_TYPES = %W{ AUTH-PDF AUTH-WORD AUTH-RTF AUTH-HTML}
  AUTH_TYPES_LABELS = %W{ PDF Word RTF HTML }

  WEBAPP_REPORT_TYPES = %W{ WEBAPP-PDF WEBAPP-WORD WEBAPP-RTF WEBAPP-HTML }
  WEBAPP_REPORT_TYPES_LABELS = %W{ PDF Word RTF HTML }

  WEBAPP_ASSESSMENT_REPORT_TYPES = %W{ WEBAPP_ASSESSMENT-PDF WEBAPP_ASSESSMENT-WORD WEBAPP_ASSESSMENT-RTF WEBAPP_ASSESSMENT-HTML }
  WEBAPP_ASSESSMENT_REPORT_TYPES_LABELS = %W{ PDF Word RTF HTML }

  LOOT_REPORT_TYPES = %W{ EVIDENCE-PDF EVIDENCE-WORD EVIDENCE-RTF EVIDENCE-HTML }
  LOOT_REPORT_TYPES_LABELS = %W{ PDF Word RTF HTML }

  CAMPAIGNS_REPORT_TYPES = %W{ CAMPAIGNS-PDF CAMPAIGNS-WORD CAMPAIGNS-RTF CAMPAIGNS-HTML }
  CAMPAIGNS_REPORT_TYPES_LABELS = %W{ PDF Word RTF HTML }

  SE_CAMPAIGN_DETAIL_REPORT_TYPES = %W{ SE_CAMPAIGN_DETAIL-PDF SE_CAMPAIGN_DETAIL-WORD SE_CAMPAIGN_DETAIL-RTF SE_CAMPAIGN_DETAIL-HTML }
  SE_CAMPAIGN_DETAIL_REPORT_TYPES_LABELS = %W{ PDF Word RTF HTML }

  DEFAULT_TEMPLATE    = ["Default JRXML Template", REPORTING_DEFAULT_JRXML]
  SERVICES_REPORT_SECTIONS =  ["Project Summary", "Executive Summary","Network Service Summary", "Network Services Table"].compact
  COMPROMISED_REPORT_SECTIONS =  ["Project Summary", "Executive Summary","Compromised Summary",
    "Compromised Hosts", "Vulnerabilities and Exploits"].compact
  AUTH_REPORT_SECTIONS =  ["Project Summary", "Executive Summary","Authentication Token Summary",
    "Guessed Password Summary Table", "Authentication Details Table"].compact
  WEBAPP_REPORT_SECTIONS =  ["Project Summary", "Executive Summary","Web Vulnerabilities", "Web Forms", "Web Pages", "Per-Site Details"].compact
  WEBAPP_ASSESSMENT_REPORT_SECTIONS =  ["Cover Page", "Executive Summary", "Engagement Scope", "OWASP Status", "Summary Graphs", "Vulnerability Details", "Appendices"].compact
  CAMPAIGNS_REPORT_SECTIONS =  ["Project Summary", "Executive Summary", "Campaign Summary", "Campaign Details", "Web Templates", "E-Mail Templates", "Targeted E-Mail Addresses"].compact
  SE_CAMPAIGN_DETAIL_REPORT_SECTIONS =  ["Cover Page", "Executive Summary", "Social Engineering Funnel", "Exploits Used", "Form Submissions", "Browser/Platform Information", "Appendices: Host Details, Human Targets", "Appendix: Campaign Components (Web page/e-mail HTML content)", "Appendix: Remediation Advice"].compact
  LOOT_REPORT_SECTIONS =  ["Project Summary", "Executive Summary", "Evidence Summary Table", "Complete Evidence Table", "Collected Screenshots", "Collected Text Files"].compact

  CUSTOM_REPORT_TYPES =        %W{ CUSTOM-PDF CUSTOM-WORD CUSTOM-RTF CUSTOM-HTML}
  CUSTOM_REPORT_TYPES_LABELS = %W{ PDF Word RTF HTML}

  attr_accessor :report_type, :mask, :tasklog, :loot_ex_shots, :loot_ex_passes, :display_session
  attr_accessor :whitelist, :blacklist
  attr_accessor :report_template, :report_logo, :report_name, :campaign_id
  attr_accessor :show_full_loot, :show_cookies, :include_http
  attr_accessor :order_vulns_by
  attr_accessor :display_sections, :display_charts, :display_page_code, :additional_params
  attr_accessor :cloned_type, :type_list
  attr_reader   :email_report, :to_addresses
  attr_accessor :skip_host_existence_check

  def initialize(attributes)
    super(attributes)

    @report_type    = (attributes[:report_type] || ["PDF"]).join(", ")
    @cloned_type    = attributes[:report_type] ? attributes[:report_type][0].split("-")[0..-2].join("_").downcase : nil
    # TODO Make this name consistent everywhere
    if @cloned_type == 'evidence'
      @cloned_type = 'loot'
    end
    @type_list      = attributes[:report_type] ? attributes[:report_type] : nil
    @report_name    = (attributes[:report_name] || "")
    @campaign_id    = (attributes[:campaign_id] || "")
    @mask           = set_default_boolean(attributes[:mask], false)
    @tasklog        = set_default_boolean(attributes[:tasklog], true)
    @loot_ex_shots  = set_default_boolean(attributes[:loot_ex_shots], true)
    @loot_ex_passes = set_default_boolean(attributes[:mask], true)
    @report_template = (attributes[:report_template] || DEFAULT_TEMPLATE.last)
    @report_logo    = (attributes[:report_logo] || "")
    @display_sections = display_sections_string(attributes) || ""
    @display_session = set_default_boolean(attributes[:display_session], false)
    @display_charts = set_default_boolean(attributes[:display_charts], true)
    @display_page_code = set_default_boolean(attributes[:display_page_code], false)
    @show_full_loot = set_default_boolean(attributes[:show_full_loot], false)
    @show_cookies = set_default_boolean(attributes[:show_cookies], false)
    @include_http = set_default_boolean(attributes[:include_http], true)
    @order_vulns_by = (attributes[:order_vulns_by] || "")
    @additional_params = attributes[:additional_params]
    @skip_host_existence_check = attributes[:skip_host_existence_check]

    if attributes[:whitelist_string]
      @tags_whitelist = attributes[:whitelist_string].scan(/#[\w]+/)
    end

    @whitelist = []
    if attributes[:whitelist_string]
      @whitelist = tags_and_addresses(attributes[:whitelist_string].split(/\s+/))
    end
    @blacklist = []
    if attributes[:blacklist_string]
      @blacklist = tags_and_addresses(attributes[:blacklist_string].split(/\s+/))
    end

    @email_report = set_default_boolean(attributes[:email_report], false)
    assign_to_addresses(attributes[:to_addresses])
  end

	# FIXME In rails 2, this happened in the task controller proper, something
	# got disconnected in the switchover.
	def display_sections_string(attrib)
		return unless attrib and attrib.kind_of?(Hash)
    return unless attrib["report_task"] and attrib["report_task"].kind_of?(Hash)
		attrib["report_task"]["display_sections_string"]
	end

  def whitelist_string
    whitelist.join("\n")
  end

  def blacklist_string
    blacklist.join("\n")
  end

  def se_campaign_detail_type?
    !!(@report_type =~ /SE_CAMPAIGN_DETAIL-.*/)
  end

  def webapp_assessment_type?
    !!(@report_type =~ /WEBAPP_ASSESSMENT-.*/)
  end

  # TODO Need to check other reports to verify what should 
  # have this requirement.
  def hosts_required?
    not_req = Set.new [se_campaign_detail_type?, webapp_assessment_type?, @skip_host_existence_check]
    !not_req.member? true
  end

  def valid?

    if !@tags_whitelist.nil?
      @tags_whitelist.each do |tag|
        partial_name = tag.gsub(/^#+/,"")
        tag_arr = @workspace.tags.select {|t| t.name.to_s =~ /^#{partial_name}/}
        if tag_arr.empty?
          @error = "Cannot find any hosts associated with #{tag}"
          break
        end
      end
      return false if @error
    end

    if se_campaign_detail_type? and @campaign_id.empty?
      @error = "No campaign was selected."
      return false
    end

    if @report_name and !@report_name.empty?
      if @report_name.length > 63
        @error = "Report name is too long"
        return false
      end

      if @report_name =~ /[^A-Za-z0-9\x20\x2d\x2e\x5f]/
        @error = "Report names may contain only alphanumerics, spaces, dots, dashes, and underscores."
        return false
      end
    end

    if @report_type.nil? || @report_type.empty?
      @error = "No report or export types were selected."
      return false
    end

    if hosts_required? && (@workspace.nil? || @workspace.hosts.nil? || @workspace.hosts.empty?)
      @error = "No discovered hosts to report on. Try running a Scan or Import first."
      return false
    end

    if whitelist && !whitelist.empty?
      whitelist.each do |ip|
        next if valid_ip_or_range?(ip)
        @error = "Invalid target IP address: #{ip}"
        return false
      end
    end

    if blacklist && !blacklist.empty?
      blacklist.each do |ip|
        next if valid_ip_or_range?(ip)
        @error = "Invalid excluded IP address: #{ip}"
        return false
      end
    end

    @error = nil
    return true
  end

  # This is a singleton which means it's difficult to get a hold of
  # the workspace ID from here. XXX Fix this.
  def self.default_template
    DEFAULT_TEMPLATE
  end

  def product_name
    if License.get.supports_custom_reporting?
      "Metasploit Pro"
    else
      "Metasploit Express"
    end
  end

  def config_to_hash
    conf = {
      'DS_WHITELIST_HOSTS'        => whitelist.join(" "),
      'DS_BLACKLIST_HOSTS'        => blacklist.join(" "),
      'workspace'                 => workspace.name,
      'username'                  => username,
      'DS_MaskPasswords'          => mask,
      'DS_IncludeTaskLog'         => tasklog,
      'DS_JasperDisplaySession'   => display_session,
      'DS_JasperDisplayCharts'    => (License.get.supports_charts? && display_charts),
      'DS_JasperDisplayPageCode'  => display_page_code,
      'DS_JasperShowFullLoot'     => show_full_loot,
      'DS_JasperShowCookies'      => show_cookies,
      'DS_JasperIncludeHttp'      => include_http,
      'DS_JasperOrderVulnsBy'     => order_vulns_by,
      'DS_LootExcludeScreenshots' => loot_ex_shots, # Still used?
      'DS_LootExcludePasswords'   => loot_ex_passes, # Still used?
      'DS_JasperTemplate'         => report_template,
      'DS_REPORT_TYPE'            => report_type,
      'DS_REPORT_NAME'            => report_name,
      'DS_CAMPAIGN_ID'            => campaign_id,
      'DS_UseJasper'              => License.get.supports_jasper?,
      'DS_UseCustomReporting'     => License.get.supports_custom_reporting?,
      'DS_JasperProductName'      => product_name,
      'DS_JasperDbEnv'            => Rails.env,
      'DS_JasperLogo'             => report_logo || "",
      'DS_JasperDisplaySections'  => display_sections,
      'DS_EnablePCIReport'        => License.get.supports_pci?,
      'DS_EnableFISMAReport'      => License.get.supports_fisma?,
      'DS_JasperDisplayWeb'       => License.get.supports_webapp_exploitation?,
      'DS_ADDITIONAL_PARAMS'      => additional_params
    }

    if send_email?
      conf['DS_EmailReport'] = true
      conf['DS_EmailTo'] = to_addresses
    end
    conf
  end

  def rpc_call
    conf = config_to_hash
    client.start_report(conf)
  end

  def allows_replay?
    true
  end

  def send_email?
    email_report and (not @to_addresses.blank?)
  end

  private

  def assign_to_addresses(addresses)
    new_addresses = addresses.nil? ? '' : addresses.strip
    return if new_addresses.blank?
    @to_addresses = new_addresses
  end
  
  def self.available_report_sections
    [
      ( License.get.supports_tags? ? "Executive Summary and Tags" : "Executive Summary"),
      "Compromised Hosts", 
      "Compromised Credentials",
      "Discovered OSes", 
      "Discovered Hosts", 
      "Host Details", 
      "Discovered Services", 
      ( License.get.supports_webapp_exploitation? ? "Web Sites" : nil)
    ].compact  
  end
end
