require 'acts_as_list'

class ScheduledTask < ActiveRecord::Base
  #
  # CONSTANTS
  #

  VALID_TASK_KINDS = %w(bruteforce cleanup collect_evidence exploit import module_run nexpose report scan webscan)

  #
  # Associations
  #

  belongs_to :task_chain

  #
  # Mass Assignment Security
  #

  # User is allowed to set config_hash from form
  attr_accessible :config_hash
  # User is NOT allowed to set position, since it comes from acts_as_list
  # User is NOT allowed to set created_at or updated_at since they are internal timestamp fields
  # User is NOT allowed to set kind since it must be derived from type field and the mapping is not direct.
  # User is NOT allowed to set last_run_at or last_run_status since those are filled in when the task is run.
  # User is NOT allowed to set state since the backend keeps track of that
  # User is NOT allowed to set task_chain_id as a ScheduledTask cannot move between TaskChains.

  #
  # Serializations
  #

  serialize :config_hash # config for Task

  #
  # Validations
  #

  validates :kind, :inclusion => {:in => VALID_TASK_KINDS}
  validates :config_hash, :presence => true

  acts_as_list :scope => :task_chain
end
