class SocialEngineering::CampaignFile < ActiveRecord::Base
  self.table_name = :se_campaign_files

  #
  # CONSTANTS
  #

  VALID_CONTENT_DISPOSITIONS = ["attachment", "inline"]

  #
  # Associations
  #

  belongs_to :attachable, :polymorphic => true

  #
  # Callbacks
  #

  before_validation(:on => :create) do
    if self.content_disposition.blank?
      self.content_disposition = "attachment"
      return true
    end
  end

  #
  # Mass Assignment Security
  #

  # User is NOT allowed to set attachable_id or attachable_type since its a polymorphic field
  # User is allowed to set attachment since that's managed by CarrierWave for uploads
  attr_accessible :attachment
  # User is NOT allowed to set content_disposition because it defaults to 'attachment'
  # User is NOT allowed to set created_at or updated_at because they are internal timestamps
  # User is NOT allowed to set file_size because its derived from attachment
  # User is allowed to set name so they can differentiate CampaignFiles
  attr_accessible :name

  #
  # Uploaders
  #

  mount_uploader :attachment, SocialEngineering::CampaignFileUploader

  #
  # Validations
  #

  validates :content_disposition, :inclusion => VALID_CONTENT_DISPOSITIONS, :presence => true

  def attachment_url
    "/campaign_files/#{id}"
  end

  # For setting a MIME message part's Content-ID header
  def content_id
    @content_id ||= Rex::Text.to_hex(attachment.identifier + id.to_s, '')
  end

  attr_writer :content_id

  def to_s
    attachment.to_s
  end
end
