class SocialEngineering::EmailOpening < ActiveRecord::Base
  self.table_name = :se_email_openings

  #
  # Associations
  #

  belongs_to :email, :class_name => 'SocialEngineering::Email'
  belongs_to :human_target, :class_name => 'SocialEngineering::HumanTarget'

  #
  # Mass Assignment Security
  #

  # User is allowed to set address as part of a VisitRequest
  attr_accessible :address
  # User is NOT allowed to set timestamp columns, created_at and updated_at
  # User is allowed to set email_id as part of a VisitRequest
  attr_accessible :email_id
  # User is allowed to set human_target_id as part of a VisitRequest
  attr_accessible :human_target_id

  #
  # Scopes
  #

  scope :for_human_target, lambda{|ht| where(:human_target_id => ht.id).includes(:email => :campaign) }
  scope :in_campaign, lambda { |campaign| where("email_id in (?)", campaign.emails) }

  #
  # Validations
  #

  validates :address, :presence => true, :ip_format => true
  validates :email, :presence => true
  validates :human_target, :presence => true

  def self.remove_for(campaign)
    results = in_campaign(campaign)
    destroy(results.collect(&:id))
  end
end
