class SocialEngineering::EmailSend < ActiveRecord::Base
  self.table_name = :se_email_sends

  #
  # Associations
  #

  belongs_to :email, :class_name => "SocialEngineering::Email"
  belongs_to :human_target, :class_name => "SocialEngineering::HumanTarget"

  #
  # Mass Assignment Security
  #

  # User is NOT allowed to set timestamps, created_at and updated_at
  # User is allowed to set email_id to identify the email that was sent.
  attr_accessible :email_id
  # User is allowed to set human_target_id to identify who was sent the email.
  attr_accessible :human_target_id

  #
  # Scopes
  #

  scope :in_campaign, lambda { |campaign| where("email_id in (?)", campaign.emails) }

  #
  # Validations
  #

  validates :email, :presence => true
  validates :human_target, :presence => true

  def self.remove_for(campaign)
    results = in_campaign(campaign)
    destroy(results.collect(&:id))
  end
end
