class SocialEngineering::PhishingResult < ActiveRecord::Base
  self.table_name = :se_phishing_results

  #
  # Associations
  #

  belongs_to :human_target, :class_name => 'SocialEngineering::HumanTarget'
  belongs_to :web_page, :class_name => 'SocialEngineering::WebPage'

  #
  # Mass Assignment Security
  #

  # User is NOT allowed to set the address as it must be derived from the request to the phishing web server
  # User is NOT allowed to set the browser_name as it must be derived from the User-Agent string
  # User is NOT allowed to set the browser_version as it must be derived from the User-Agent string
  # User is NOT allowed to set timestamps, created_at or updated_at
  # User is NOT allowed to set data as it must be parsed as JSON from raw_data
  # User is NOT allowed to set human_target_id as it must be derived from the VisitRequest
  # User is NOT allowed to set os_name as it must be derived from the User-Agent string
  # User is NOT allowed to set os_version as it must be derived from User-Agent string
  # User is NOT allowed to set raw_data as it must be the entire request body.
  # User is NOT allowed to set web_page_id as it must be derived from the request to the phishing web server
  # make nothing accessible
  attr_accessible

  #
  # Scopes
  #

  scope :for_human_target, lambda { |ht| where(:human_target_id => ht.id) }
  scope :in_campaign, lambda { |campaign| where("web_page_id in (?)", campaign.web_pages) }

  def self.remove_for(campaign)
    results = in_campaign(campaign)
    destroy(results.collect(&:id))
  end
end
