require "csv"

class SocialEngineering::TargetList < ActiveRecord::Base
  include SocialEngineering::TargetList::Rows

  self.table_name = :se_target_lists

  #
  #
  # Associations - sorted by name
  #
  #

  has_many :emails, :class_name => 'SocialEngineering::Email', :dependent => :restrict
  has_many :target_list_targets, :class_name => 'SocialEngineering::TargetListHumanTarget', :dependent => :destroy
  belongs_to :user, :class_name => 'Mdm::User'
  belongs_to :workspace, :class_name => 'Mdm::Workspace'

  #
  # Through :target_list_targets
  #

  has_many :human_targets, :class_name => 'SocialEngineering::HumanTarget', :through => :target_list_targets

  #
  # Callbacks
  #

  after_save  :save_related

  #
  # Mass Assignment Security
  #

  # User is NOT allowed to set timestamps, created_at and updated_at
  # User is NOT allowed to set file_name as it is derived from the tempfile basename on disk
  # User is allowed to set name for the TargetList's name
  attr_accessible :name
  # User is NOT allowed to set user_id as it is derived from current user.
  # User is NOT allowed to set workspace_id as it is derived from current workspace.

  #
  # Scopes
  #

  scope :for_workspace, lambda { |workspace| where(:workspace_id => workspace.id) }

  #
  # Validations
  #

  validates :name, :presence => true, :uniqueness => { :scope => :workspace_id }
  validates :user, :presence => true
  validates :workspace, :presence => true

  private

  # Saves related new HTs and ALHT refs.
  def save_related
    self.class.transaction do
      self.human_targets.each { |ht| ht.save if ht.new_record? }
      self.target_list_targets.each { |ref| ref.save if ref.new_record? }
    end
  end
end
