class SocialEngineering::Visit < ActiveRecord::Base
  self.table_name = :se_visits

  #
  # Associations
  #

  belongs_to :email, :class_name => 'SocialEngineering::Email'
  belongs_to :human_target, :class_name => 'SocialEngineering::HumanTarget'
  belongs_to :web_page, :class_name => 'SocialEngineering::WebPage'

  #
  # Mass Assignment Security
  #

  # User is allowed to set address
  attr_accessible :address
  # User is NOT allowed to set timestamps, created_at and updated_at
  # User is allowed to set email_id for the Email that contained the link that caused this Visit
  attr_accessible :email_id
  # User is allowed to set human_target_id for the person that is visiting the page
  attr_accessible :human_target_id
  # User is allowed to set web_page_id for the WebPage pointed to by the link in the Email
  attr_accessible :web_page_id

  #
  # Scopes
  #

  scope :for_human_target, lambda{|ht| where(:human_target_id => ht.id).includes(:email => :campaign) }

  #
  # Validations
  #

  validates :human_target, :presence => true

  def self.remove_for(campaign)
    visits = []

    if campaign.web_campaign?
      visits << where("web_page_id in (?)", campaign.web_pages)
    end

    if campaign.email_campaign?
      visits << where("email_id in (?)", campaign.emails)
    end

    destroy(visits.flatten.uniq.collect(&:id))
  end
end
