require File.expand_path('../boot', __FILE__)

require 'rails/all'

if defined?(Bundler)
  # Precompile assets before deploying to production
  Bundler.require(*Rails.groups(:assets => %w(development test cucumber)))
end

module Pro
  class Application < Rails::Application
    # Settings in config/environments/* take precedence over those specified here.
    # Application configuration should go into files in config/initializers
    # -- all .rb files in that directory are automatically loaded.

    # Activate observers that should always be running.
    config.active_record.observers = :running_campaign_observer,
                                     :"social_engineering/configurable_campaign_observer"

    # Enable the asset pipeline
    config.assets.enabled = true
    # Disable initialization when precompiling assets
    config.assets.initialize_on_precompile = false
    # Version of your assets, change this if you want to expire all your assets
    config.assets.version = '1.0'

     # Custom directories with classes and modules you want to be autoloadable.
    config.autoload_paths += ['app/presenters', 'lib'].collect { |relative_path|
      # qualify relative path with rails root
      "#{config.root}/#{relative_path}"
    }

    # Configure the default encoding used in templates for Ruby 1.9.
    config.encoding = "utf-8"

    # Configure sensitive parameters which will be filtered from the log file.
    config.filter_parameters += [
        'social_engineering_email[smtp_password]',
        'social_engineering_email[smtp_username]',
        :SMBDomain,
        :SMBPass,
        :SMBUser,
        :cookie,
        :http_password,
        :http_username,
        :nexpose_creds_pass,
        :nexpose_creds_user,
        :nexpose_pass,
        :options,
        :pass,
        :password,
        :password_confirmation,
        :proxy_pass,
        :proxy_pass,
        :quickmode_creds,
        :smtp_pass,
        :smtp_user,
        :user
    ]

    config.generators do |g|
      g.fixture_replacement :factory_girl
    end

    # Add this for Spork 
    if Rails.env.test? 
      initializer :after => :initialize_dependency_mechanism do 
        ActiveSupport::Dependencies.mechanism = :load 
      end 
    end
  end
end
