#
# Metasploit Initialization
#


#
# Load architecture specific library paths
#
arch = 'unknown'
case RUBY_PLATFORM
when /x86_64-linux/
  arch = "linux64"
when /i[3456]86-linux/
  arch = "linux32"
when /mingw32/
  arch = "win32"
end

engine_dir = File.expand_path(File.join(Rails.root, '..', 'engine', 'lib'))
$:.unshift(engine_dir)

msf_dir = File.expand_path(File.join(Rails.root, '..', 'msf3', 'lib'))
$:.unshift(msf_dir)

#
# Load specific patches for system libraries
#
if arch == "win32"
  require "win32/registry"
  require "patches/win32_registry"
end

if arch == "linux32" or arch == "linux64"
  # require 'openssl_nonblock'
  # Expand our maximum file limit as high as it will go
  while true
    begin; Process.setrlimit(Process::RLIMIT_NOFILE, 32768, 32768); break; rescue ::Exception; end
    begin; Process.setrlimit(Process::RLIMIT_NOFILE, 16384, 16384); break; rescue ::Exception; end
    begin; Process.setrlimit(Process::RLIMIT_NOFILE, 8192, 8192); break; rescue ::Exception; end
    begin; Process.setrlimit(Process::RLIMIT_NOFILE, 4096, 4096); break; rescue ::Exception; end
    begin; Process.setrlimit(Process::RLIMIT_NOFILE, 1024, 1024); break; rescue ::Exception; end
    break
  end
end


require 'timeout'
require 'pro/client'
require 'pro/client_connection_manager'

#
# Configure the process environment to work with java and other tools
#
pdel = (RUBY_PLATFORM =~ /mingw/) ? ";" : ":"
bins = %W{ common/bin ruby/bin nmap/bin postgresql/bin apache2/bin java/bin jruby/bin }
bins.each do |bpath|
  ENV['PATH'] += pdel + File.expand_path(File.join(File.dirname(__FILE__), "..", "..", "..", "..", "..", bpath))
end
ENV['PATH'] = ENV['PATH'].split(pdel).uniq.join(pdel)
ENV['JAVA_HOME'] = File.expand_path(File.join(File.dirname(__FILE__), "..", "..", "..", 'java'))

# Convert path delimiters for Windows
if arch == "win32"
  ENV['PATH']      = ENV['PATH'].gsub("/", "\\")
  ENV['JAVA_HOME'] = ENV['JAVA_HOME'].gsub("/", "\\")
end

#
# Preload the module tree and ensure ProSvc is happy and running
#
if Rails.env.production? and ! ENV['MSFRPC_SKIP']
  prosvc_running = false
  connection_cnt = 0
  until prosvc_running
    begin
      # Wrap this in a timeout to prevent a hung reply from hanging this
      ::Timeout.timeout(20) do
        MsfModule.all
        prosvc_running = true
      end
    rescue ::Exception
      select(nil, nil, nil, 3)
      connection_cnt += 1
    end

    # If its been 50 seconds, go ahead and exit
    break if connection_cnt > 10
  end
end

#
# Monkey-patch AuthLogic to always set :secure => true
#
module Authlogic
  module Session

    module Cookies
      module InstanceMethods
        def save_cookie
          controller.cookies[cookie_key] = {
            :value => "#{record.persistence_token}::#{record.send(record.class.primary_key)}",
            :expires => remember_me_until,
            :domain => controller.cookie_domain,
            :secure => true,
            :httponly => true
          }
        end
      end
    end

  end
end


#
# Monkey-patch Thin to avoid a DoS with reserved file names
#
module Rack
  module Adapter
    class Rails

      def file_exist?(path)
        full_path = ::File.join(@file_app.root, Utils.unescape(path))
        if full_path =~ /\/(CON|PRN|AUX|NUL|COM1|COM2|COM3|COM4|COM5|COM6|COM7|COM8|COM9|LPT1|LPT2|LPT3|LPT4|LPT5|LPT6|LPT7|LPT8|LPT9)([\.\/]|$)/i
          return false
        end
        ::File.file?(full_path) && ::File.readable_real?(full_path)
      end

    end
  end
end


#
# Monkey-patch a typo still present in the latest EventMachine gem
#
module EventMachine
  class Connection
    def associate_callback_target(sig)
      # Nothing
    end
  end
end

#
# ProSvc creates our schema, so wait at least this long before 
# trying to do anything fancy with the database.
#

1.upto(300) do
  break
  begin
    if not ( 
            ActiveRecord::Base.connection.tables.index("schema_migrations") and
            ActiveRecord::Base.connection.tables.index("users")
           )
           $stdout.puts "Waiting for schema to be created..."
           sleep 1
           next
    end
    break
  rescue ::Exception
    $stdout.puts "Waiting for schema to be created: #{$!}.class #{$!} #{$!.backtrace}"
    sleep 1
  end
end


# Clear the cached data for modules
MsfModule.reload_modules

