Pro::Application.routes.draw do

  match "/campaign_files/:id" => "SocialEngineering::files#download"

  # Quick Pentest Wizard
  # have to define this outside of the resource because the
  #   Wicked::Wizard gem overwrites any GET routes
  match "/quick_pentest/first_task_ready" => "quick_pentest#first_task_ready"
  match "/quick_pentest/redirect_after_success" => "quick_pentest#redirect_after_success"
  resources :quick_pentest

  # Setup & Licenses
  scope "/setup" do
    match ""           => "licenses#setup", :as => "setup"
    match "request"    => "licenses#request_key", :as => "request_key"
    match "activation" => "licenses#activation", :as => "activation"
  end

  scope "/licenses" do
    post "register" => "licenses#register", :as => "register"
    post "activate" => "licenses#activate", :as => "activate"
    match "offline_activation" => "licenses#offline_activation", :as => "offline_activation", :via => [:get, :post] 
  end


  # Updates
  scope "/updates" do
    match ""             => "updates#updates", :as => "updates"
    post "check"         => "updates#check"
    post "apply"         => "updates#apply"
    post "restart"       => "updates#restart"
    post "status"        => "updates#status"
    match "offline_apply" => "updates#offline_apply"
  end

  # NeXpose Consoles
  resources :nexpose_consoles do
    collection do
      delete "/destroy", :as => "destroy"
    end
  end

  # API Keys
  resources :api_keys do
    collection do
      delete "/destroy", :as => "destroy"
    end

    member do
      post "/reveal" => "api_keys#reveal"
    end
  end



  # Macros
  resources :macros
  scope "/macro" do
    match ":id/delete_action/:action_id" => "macros#delete_action", :as => "macro_delete_action"
    match ":id/delete_action" => "macros#delete_action", :as => "macro_delete_actions"
    match ":id/add_action" => "macros#add_action", :as => "macro_add_action"
    match "module_options" => "macros#module_options", :as => "module_options"
  end
  delete "macros" => "macros#destroy", :as => "delete_macros"


  # Listeners
  resources :listeners
  scope "/listener" do
    match ":id/stop" => "listeners#stop", :as => "listener_stop"
    match ":id/start" => "listeners#start", :as => "listener_start"
    match "module_options" => "listeners#module_options", :as => "macro_module_options"
  end
  delete "listeners" => "listeners#destroy", :as => "delete_listeners"


  # Global Settings
  scope "/settings" do
    match "" => "settings#index", :as => "settings"
    match "update_profile" => "settings#update_profile", :as => "update_profile"
  end


  # Mdm::User settings
  resources :users

  match "login" => "user_sessions#new", :as => "login"
  match "logout" => "user_sessions#destroy", :as => "logout"

  resources :user_sessions


  # Email addresses
  scope "/workspaces/:workspace_id/campaigns/:campaign_id/email_addresses" do
    match "import" => "email_addresses#new_import", :as => "new_import_email_addresses"
    match "import_addresses" => "email_addresses#import", :as => "import_email_addresses"
  end

  # Workspaces
  match "/workspaces/search" => "workspaces#search"
  delete "/workspaces/destroy" => "workspaces#destroy"

  resources :workspaces do
    match "sessions" => "sessions#index"
    match "events" => "events#index"
    match "reports" => "reports#index"
    match "web" => "web#index"

    # TaskChains
    resources :task_chains do
      collection do
        delete :destroy_multiple
      end
      member do
        put :run_now
      end
    end


    # NEW STYLE!  
    # Social Engineering Campaigns
    namespace :social_engineering do
      resources :human_targets
      
      resources :web_templates do
        collection do
          delete "" => "web_templates#destroy", :as => "delete"
        end
      end

      resources :email_templates do
        collection do
          delete "" => "email_templates#destroy", :as => "delete"
        end
      end

      resources :target_lists do
        get "export" => "target_lists#export"
        post "remove_targets" => "target_lists#remove_targets"
        collection do
          delete "" => "target_lists#destroy", :as => "delete"
        end
      end

      resources :web_templates do
        collection do
          match :clone_proxy, :via => 'post'
          delete "" => "web_templates#destroy", :as => "delete"
        end
      end

      resources :files do
        collection do
          delete "" => "files#destroy", :as => "delete"
        end
      end

      # ------ Campaigns, components, and tracking data -----
      resources :campaigns do
      
        resource :web_server_config, {
          :only => [:edit, :update],
          :controller => :web_server_config
        }
        resource :email_server_config, {
          :only => [:edit, :update],
          :controller => :email_server_config
        } do
          collection do
            match :check_smtp
          end
        end

        resources :visits

        resources :emails do
          match :preview
          match :preview_pane
          collection do
            match :custom_content_preview, :via => 'post'
          end
        end

        resources :web_pages do
          match :preview
          match :preview_pane
          collection do
            match :clone_proxy, :via => 'post'
            match :custom_content_preview, :via => 'post'
          end
        end

        resources :portable_files do
          member do
            match :download
          end
        end

        member do
          match :execute
          match :reset, :via => 'post'
          match :to_task
          match :sent_emails
          match :opened_emails
          match :opened_sessions
          match :submitted_forms
          match :links_clicked
          match :check_for_configuration_errors
        end

        collection do
          delete "" => "campaigns#destroy", :as => "delete"
          match :logged_in
        end
        resources :phishing_results, :only => [:index, :show]
      end
    end

    # old-style Social Engineering Campaigns
    #   # Campaigns
    resources :campaigns do
      member do
        match :status
        match :start
        match :stop
        match :edit
        match :download
      end
      collection do
        delete "" => "campaigns#destroy", :as => "delete"
      end
      resources :email_addresses do
        collection do
          match :mass_edit
          match :mass_update
          delete :mass_delete
        end
      end
    end 

    resources :email_templates

    resources :web_templates do
      member do
        post :preview
      end

      collection do
        match :exploit_options
        post :clone
      end
    end

    delete "/tags/destroy" => "tags#destroy"
    resources :tags do
      collection do
        delete :destroy, :as => "destroy"
        get :search
      end
    end

    resources :loots, :only => [:index] do
      collection do
        delete :destroy_multiple
      end
    end

    
    # Notes, Services
    [:notes, :services].each do |res|
      resources res, :only => [:index] do
        collection do
          get :combined
          delete :destroy_multiple
        end
      end
    end

    # Vulnerabilities
    
    resources :vulns, :only => [:index, :show] do
      collection do
        get :combined
        delete :destroy_multiple
      end
    end

    match "vulns/:id/details"  => "vulns#details", :as => "vuln_details"
    match "vulns/:id/attempts" => "vulns#attempts", :as => "vuln_attempts"
    match "vulns/:id/exploits" => "vulns#exploits", :as => "vuln_exploits"
   
  end # end Workspaces routes
  
  
  # Creds
  resources :creds do
    collection do
      delete :destroy_multiple
    end
  end

  # Mdm::Tag Search
  match "tags" => "tags#search", :as => "search_tags"


  # Hosts
  # TODO: make tag creation/deletion routes make more sense
  scope :hosts do
    scope "workspaces/:workspace_id" do
      delete "hosts(/:id)" => "hosts#destroy", :as => "delete_host"    # FIXME (by deleting prototype): made id optional for named route, see hosts/index.html.erb:15
      post "hosts" => "hosts#create", :as => "create_host"
      match "hosts" => "hosts#index",  :as => "hosts"
      match "hosts/new" => "hosts#new", :as => "new_host"
      match "hosts/map" => "hosts#map", :as => "map_host"
      post "hosts/multi_tag" => "hosts#multi_tag", :as => "multi_tag"
      post "hosts/quick_multi_tag" => "hosts#quick_multi_tag", :as => "quick_multi_tag"
      post "hosts/:id" => "hosts#create_or_update_tag", :as => "create_or_update_tag"
    end
    
    scope "hosts/:id" do
      match "new_loot" => "loots#new", :as => "new_loot"
      match "add_loot" => "loots#add_loot", :as => "add_loot"
      get "" => "hosts#show", :as => "host"
      put "" => "hosts#update", :as => "update_host"
      post "" => "hosts#create_or_update_tag", :as => "create_or_update_tag"
      delete "" => "hosts#remove_tag", :as => "remove_tag"
      match "edit" => "hosts#edit", :as => "edit_host"
      match "services" => "hosts#services", :as => "host_services"
      match "sessions" => "hosts#sessions", :as => "host_sessions"
      get "vulns" => "hosts#vulns", :as => "host_vulns_tab"
      match "shares" => "hosts#shares", :as => "host_shares"
      match "loots" => "hosts#loots", :as => "host_loots"
      match "notes" => "hosts#notes", :as => "host_notes"
      match "creds" => "hosts#creds", :as => "host_creds"
      match "tags" => "hosts#tags", :as => "host_tags"
      match "attempts" => "hosts#attempts", :as => "host_attempts"
      match "details" => "hosts#details", :as => "host_details"
      match "exploits" => "hosts#exploits", :as => "host_exploits"
    end
  end

  # Vulns
  scope "/hosts/:host_id", :as => "host" do
    resources :vulns
  end


  # Sessions
  scope "sessions/:id" do 
    delete "" => "sessions#destroy", :as => "delete_session"
    match "route" => "sessions#route", :as => "session_route"
    match "vnc" => "sessions#vnc", :as => "session_vnc"
    match "shell" => "sessions#shell", :as => "session_shell"
    match "files" => "sessions#files", :as => "session_files"
    match "search" => "sessions#search", :as => "session_search"
    match "upload" => "sessions#upload", :as => "session_upload"
    match "download" => "sessions#download", :as => "session_download"
    match "delete" => "sessions#delete", :as => "session_delete"
  end

  scope "workspaces/:workspace_id" do
    get "sessions/:id" => "sessions#show", :as => "session"
    match "session_history/:id" => "sessions#history", :as => "session_history"
    match "consoles/:id" => "sessions#console_interact", :as => "session_console_interact"
    match "console" => "sessions#console_create", :as => "session_console_create"
    match "sessions/:id/reopen" => "tasks#session_reopen", :as => "session_reopen"
  end


  # Tasks
  match "workspace/:workspace_id/tasks/update_cred_file" => "tasks#update_cred_file", :as => "update_cred_file" # FIX ME (workspaces misspelled)
  scope "workspaces/:workspace_id/tasks" do

    match "new_import_creds" => "tasks#new_import_creds", :as => "new_import_creds"
    match "start_import_creds" => "tasks#start_import_creds", :as => "start_import_creds"
    match "update_cred_file" => "tasks#update_cred_file", :as => "update_cred_file"
    
    match "new_scan" => "tasks#new_scan", :as => "new_scan"
    match "start_scan" => "tasks#start_scan", :as => "start_scan"
    
    match "new_import" => "tasks#new_import", :as => "new_import"
    match "start_import" => "tasks#start_import", :as => "start_import"
    
    match "new_exploit" => "tasks#new_exploit", :as => "new_exploit"
    match "start_exploit" => "tasks#start_exploit", :as => "start_exploit"
    
    match "new_webscan" => "tasks#new_webscan", :as => "new_webscan"
    match "start_webscan" => "tasks#start_webscan", :as => "start_webscan"

    match "new_webaudit" => "tasks#new_webaudit", :as => "new_webaudit"
    match "start_webaudit" => "tasks#start_webaudit", :as => "start_webaudit"

    match "new_websploit" => "tasks#new_websploit", :as => "new_websploit"
    match "start_websploit" => "tasks#start_websploit", :as => "start_websploit"

    match "new_bruteforce" => "tasks#new_bruteforce", :as => "new_bruteforce"
    match "start_bruteforce" => "tasks#start_bruteforce", :as => "start_bruteforce"

    match "new_collect" => "tasks#new_collect_evidence", :as => "new_collect_evidence"
    match "start_collect" => "tasks#start_collect_evidence", :as => "start_collect_evidence"

    match "clone_module_run/:clone_id" => "tasks#new_module_run", :as => "clone_module_run"
    match "new_module_run/(*path)" => "tasks#new_module_run", :as => "new_module_run"
    match "start_module_run/*path" => "tasks#start_module_run", :as => "start_module_run"

    match "new_nexpose" => "tasks#new_nexpose", :as => "new_nexpose"
    match "start_nexpose" => "tasks#start_nexpose", :as => "start_nexpose"

    match "new_replay" => "tasks#new_replay", :as => "new_replay"
    match "start_replay" => "tasks#start_replay", :as => "start_replay"

    # New report, new export, and new PCI and FISMA reports are all UIs for the over-arching ReportTask,
    # and all generate data via the start_report task.
    %w(report export pci_report fisma_report custom_report services_report compromised_report auth_report webapp_report loot_report campaigns_report).each do |rep|
      match "new_#{rep}" => "tasks#new_#{rep}", :as => "new_#{rep}"
    end

    match "start_report" => "tasks#start_report", :as => "start_report"

    match "new_report_template" => "tasks#new_report_template", :as => "new_report_template"
    match "start_report_template" => "tasks#start_report_template", :as => "start_report_template"

    match "new_cleanup" => "tasks#new_cleanup", :as => "new_cleanup"
    match "start_cleanup" => "tasks#start_cleanup", :as => "start_cleanup"
    
    match "new_upgrade_sessions" => "tasks#new_upgrade_sessions", :as => "new_upgrade_sessions"
    match "start_upgrade_sessions" => "tasks#start_upgrade_sessions", :as => "start_upgrade_sessions"

    match "start_campaign" => "tasks#start_campaign", :as => "start_campaign"

    match "new_tunnel" => "tasks#new_tunnel", :as => "new_tunnel"
    match "start_tunnel" => "tasks#start_tunnel", :as => "start_tunnel"

    match "new_nexpose_asset_group_push" => "tasks#new_nexpose_asset_group_push", :as => "new_nexpose_asset_group_push"
    match "start_nexpose_asset_group_push" => "tasks#start_nexpose_asset_group_push", :as => "start_nexpose_asset_group_push"

    match "new_nexpose_exception_push" => "tasks#new_nexpose_exception_push", :as => "new_nexpose_exception_push"
    match "start_nexpose_exception_push" => "tasks#start_nexpose_exception_push", :as => "start_nexpose_exception_push"

    match "" => "tasks#index", :as => "tasks"
    match ":id" => "tasks#show", :as => "task_detail"
  end

  scope "tasks" do
    match "stop" => "tasks#stop", :as => "stop_task"
    match "stop_all" => "tasks#stop_all", :as => "stop_all_tasks"
    match "replay/:id" => "tasks#replay", :as => "replay_task"
    match ":id/logs" => "tasks#logs", :as => "task_log"
  end

  delete "workspace/:workspace_id/events" => "events#delete_all", :as => 'delete_workspace_events' # FIX ME (workspaces misspelled)
  scope "workspaces/:workspace_id" do
    match "reports" => "reports#index", :as => "reports"
    match "reports_delete" => "reports#reports_delete", :as => "reports_delete"
    match "report_view/:report_id" => "reports#report_view", :as => "report_view"
    match "report_download/:report_id" => "reports#report_download", :as => "report_download"

    # Mdm::Report Templates
    match "report_template_upload" => "reports#report_template_upload", :as => "report_template_upload"
    match "report_template_download/:report_template_id" => "reports#report_template_download", :as => "report_template_download"
    match "report_template_download_static/:static_id" => "reports#report_template_download_static", :as => "report_template_download_static"
    match "report_template_delete/:report_template_id" => "reports#report_template_delete", :as => "report_template_delete"

    delete "events" => "events#delete_all", :as => 'delete_workspace_events'

    # Modules
    match "modules" => "modules#index", :as => "modules"
    match "modules/*path" => "modules#show", :as => "module"

    # Web Applications
    scope "web" do
      match "sites" => "web#index", :as => "web_sites"
      match "forms/:site_id" => "web#forms", :as => "web_forms"
      match "form/:site_id" => "web#form", :as => "web_form"
      match "vulns/:site_id" => "web#vulns", :as => "web_vulns"
      match "vuln/:vuln_id" => "web#vuln", :as => "web_vuln"

      delete "vuln/delete/:site_id" => "web#delete_vulns", :as => "delete_web_vulns"
      delete "site/delete(/:id)" => "web#delete_sites", :as => "delete_web_sites"
    end
  end

  # Events
  resources :events


  # Mdm::Loot
  get "loot/:id" => "loots#get", :as => "loot"


  # Admin area
  namespace :admin do |admin|
    resources :users do 
      collection do
        match "delete" => "users#destroy", :as => "mass_delete"
      end
    end
  end

  root :to => "workspaces#index"
end
