module Campaign::Status
  # XXX: Make sure this stays synced with modules/auxiliary/pro/campaign.rb
  Statuses = {
      :created => 0,
      :started => 1,
      :email_sent => 2,
      :completed => 3,
      :error => 99
  }

  def status_string
    Statuses.invert[self.status].to_s
  end

  def status=(rhs)
    if rhs.is_a? Symbol
      rhs = Statuses[rhs]
    end

    super(rhs)
  end

  #
  # Update the current status based on the status of the task associated with
  # this campaign.  Basically just monitors for errors
  #
  def update_status!
    errors = []
    if prefs[:task]
      task = Mdm::Task.find(prefs[:task]) rescue nil
      if task
        errors << task.error if task.has_error?
      else
        errors << "No such task"
      end
    end

    unless errors.empty?
      self.status = :error
      self.error = errors.join("\n")
      save!
      return
    end

    if task and task.done?
      self.status = :completed
    end

    save! if changed?
  end
end