module Mdm::WebVuln::Web
  extend ActiveSupport::Concern

  included do
    #
    # Associations
    #

    # @!attribute [rw] request
    #   @return [Web::Request] if this web vuln is either found or verified by a {Web::Request}
    #   @return [nil] if this web vuln is associated with a Mdm::WebSite
    belongs_to :request, :class_name => 'Web::Request'

    validate :request_xor_web_site
  end

  private

  # Records an error against :base if either #request or #web_site is not present, or if both are present.
  #
  # @return [void]
  def request_xor_web_site
    unless request.present? ^ web_site.present?
      errors[:base] << 'can have either request or web_site present, but not both'
    end
  end
end