module Mdm::Workspace::Statistics
  extend ActiveSupport::Concern

  included do
    @@stats_cache = {}
    @@stats_cache_time = 0
    @@stats_cache_time_limit = 5
  end

  def stats
    if @@stats_cache_time and @@stats_cache_time > 0 and (Time.now.to_i < @@stats_cache_time + @@stats_cache_time_limit)
      return @@stats_cache[self[:id]] if @@stats_cache[self[:id]]
    end
    @@stats_cache_time = Time.now.to_i

    res = {
        :loot_count => 0,
        :loot_hosts => {},
        :dead_sess_count => 0,
        :sess_count => 0,
        :sess_hosts => {},
        :pass_count => 0,
        :hash_count => 0,
        :key_count => 0,
        :pubkey_count => 0,
        :pass_hosts => {}
    }

    loots.all(:include => :host).each do |ev|
      res[:loot_count] += 1
      if ev.host
        res[:loot_hosts][ev.host.address] = true
      end
    end

    sessions.each do |s|

      thost = s.host.address
      next if not thost or thost == "127.0.0.1"
      thost = thost.split(':')[0]

      res[:sess_count] += 1
      res[:sess_hosts][thost] = true
    end

    res[:dead_sess_count] = sessions.dead.size

    # Tabulate credentials
    # For #2093, drop the uniqueness requirement in order to
    # make sure this count and the report counts are both the same.
    # Also drop the anti-readonly requirement, since you can end
    # up with counts that are way off for the case of users having
    # multiple passwords (snmp community strings namely, also
    # ftp's anonymous accounts). These subtilties should be expanded
    # in reports, not in this overview.
    creds.each do |cred|
      next if not cred.service
      next if not cred.service.host
      next if cred.service.host.created_at > cred.created_at
      case cred.ptype
        when /^password/ # Include password, password_rw, and password_ro
          res[:pass_count] += 1
        when "smb_hash"
          res[:hash_count] += 1
        when "ssh_key"
          res[:key_count] += 1
        when "ssh_pubkey"
          res[:pubkey_count] +=1
      end
      if( (cred.service.name.to_s.downcase == "ftp" and cred.user.to_s.downcase == "anonymous") ||
          (cred.service.name.to_s.downcase == "snmp" and cred.pass.to_s.downcase == "public") ||
          (cred.service.name.to_s.downcase == "smb" and cred.service.host.os_name =~ /windows|microsoft/i and cred.user =~ /guest/i)
      )
        next # Don't count these hits for awarding the cracked badge.
      else
        res[:pass_hosts][cred.service.host.address] ||= 0
        res[:pass_hosts][cred.service.host.address] += 1
      end
    end

    @@stats_cache[self[:id]] = res
  end
end