# From Rake gem
require 'rake/packagetask'

module Metasploit
  module Pro
    class ScriptsTask < Rake::TaskLib
      def define
        namespace :scripts do
          namespace operating_system do
            desc "Remove the packed scripts for #{operating_system}"
            task :clobber do
              rm_f packed_scripts_pathname
            end

            file packed_scripts_pathname.to_path => unpacked_scripts_pathname.to_path do |t|
              parent_directory = File.dirname(t.name)
              FileUtils.mkdir_p parent_directory

              root = unpacked_scripts_pathname

              Dir.chdir(root) do
                relative_source_pathname = unpacked_scripts_pathname.relative_path_from(root)
                relative_destination_pathname = packed_scripts_pathname.relative_path_from(root)

                sh '7za',
                   'a',
                   '-t7z',
                   relative_destination_pathname.to_path,
                   relative_source_pathname.to_path

                puts "Scripts for #{operating_system} saved to #{t.name}"
              end
            end

            desc <<-EOS
            Packages the config/build/update/scripts/#{operating_system} erb template for use in updating derived
            scripts on #{operating_system} by install.rb during updates.
            EOS
            task :package => packed_scripts_pathname.to_path

            desc "Removes the old packed scripts for #{operating_system} and creates a new one"
            task :repackage => [:clobber, :package]
          end
        end
      end

      def init(options={})
        options.assert_valid_keys(:operating_system)

        # use fetch so it errors if :operating_system is not passed
        @operating_system = options.fetch(:operating_system)
      end

      def initialize(options={}, &block)
        init(options)

        unless block.nil?
          block.call(self)
        end

        define
      end

      attr_reader :operating_system

      def packed_scripts_name
        @packed_scripts_name ||= "#{operating_system}-scripts.7z"
      end

      def packed_scripts_pathname
        @packed_scripts_pathname ||= self.class.scripts_pathname.join(packed_scripts_name)
      end

      def self.scripts_pathname
        @scripts_pathname ||= Rails.root.join('..', 'scripts').expand_path
      end

      def unpacked_scripts_pathname
        @unpacked_scripts_pathname ||= Rails.root.join('config', 'build', 'update', 'scripts', operating_system.to_s)
      end
    end
  end
end
