module SocialEngineering::TargetList::Rows
  extend ActiveSupport::Concern

  load 'social_engineering/target_list/rows/CSV.rb'
  include SocialEngineering::TargetList::Rows::CSV    
  include SocialEngineering::TargetList::Rows::QuickAdd

  included do
    #
    # Callbacks
    #

    before_save :process_rows, :if => :any_targets_present?
  end

  private

  def any_targets_present?
    csv_targets_present? || quick_add_targets_present?
  end

  # Pulls out email addresses from extracted list,
  # searches for matches amongst current HTs.
  # In the case of a match: only add join table entry.
  # Otherwise, add HT as usual, along with join table entry.
  def process_rows
    # combine csv_uniq_quick_add and csv_uniq_csv members
    @uniq_quick_add_members ||= []
    @uniq_csv_members ||= []

    uniq_members = (@uniq_quick_add_members + @uniq_csv_members)
      .index_by { |e| e[:email_address] }.values

    # fail here if there are NO rows to process
    if uniq_members.empty?
      errors.add(:base, "No valid human targets to add.")
      return false
    end

    # Remove email addresses that are already in the human_targets
    present_email_address_set = Set.new

    self.human_targets.each do |human_target|
      present_email_address_set.add(human_target.email_address)
    end

    uniq_members.delete_if { |member| present_email_address_set.include? member[:email_address] }

    email_addresses = uniq_members.map{|x| x[:email_address]}
    matching_addresses_ids = SocialEngineering::HumanTarget.where(:email_address => email_addresses).map {|x| {x.email_address => x.id} }
    matching_addresses = matching_addresses_ids.map {|ma| ma.keys[0]}

    # If present in DB, add to join table (ALHT)
    matching_addresses_ids.each do |ht|
      self.target_list_targets.build(
        :target_list_id  => self.id,
        :human_target_id => ht.values[0],
      )
    end

    uniq_members.each do |nt|
      # Skip HT creation if already present
      next if matching_addresses.include? nt[:email_address]
      # don't doubly add email addresses!
      #next if .include? nt[:email_address]

      target = self.human_targets.build(
          :email_address => nt[:email_address],
          :first_name    => nt[:first_name],
          :last_name     => nt[:last_name]
      )

      # Set user and workspace directly as their aren't attr_accessible since they should not be settable from web
      # forms.
      target.user = self.user
      target.workspace = self.workspace

      unless target.valid?
        errors.add(:base, "Invalid human target.")
      end
    end
  end

  def validate_row(row, row_cnt=0, filename='')
    # Email must be somewhat recognizable as an email:
    email = row["email_address"]
    email_pat = /.+@.+\..+/
    email_valid = (email =~ email_pat) == 0 ? true : false

    if not email or email == ''
      errors.add(:base, "Error: Undefined email address for human target at row #{row_cnt} in #{filename}.")
    elsif not email_valid
      errors.add(:base, "Error: Email address not properly formatted at row #{row_cnt} in #{filename}.")
    else
      return true
    end
    false
  end
end