# require 'curb'
# require 'nokogiri'
# require "base64"
class Web::HtmlRewrite
  
  attr_reader :body
  # attr_reader :headers
  def headers
    h = {}
    @raw_headers.each do |key, value|
      h[key.to_s] = value.to_s
    end
    h.slice("Content-Type")
  end

  def self.hash_url(url, hostname = "")
    if url.downcase.include?("javascript:") || url == "#"
      url
    else
      begin
        uri = URI.parse(url)
        if uri.scheme.nil? && uri.host.nil?
          new_uri = URI.join(hostname, url)
          Base64.urlsafe_encode64(new_uri.to_s)
        elsif uri.scheme.nil?
          new_scheme = URI.parse(hostname)
          Base64.urlsafe_encode64(URI.join(new_scheme, uri).to_s)
        else
          Base64.urlsafe_encode64(url)
        end
        
      rescue URI::InvalidURIError => e
        "unhashable-url"
      end
      
    end
  end

  def self.lookup_hash(digest)
    begin
      Base64.urlsafe_decode64(digest)
    rescue Exception => e
      :invalid_url
    end
  end

  def hash_url(url)
    self.class.hash_url(url, @hostname)
  end

  def new_location
    "http://localhost:3791/"
  end

  def initialize(opts = {})
    @raw_headers = opts[:headers]
    @raw_body = opts[:body]
    @hostname = opts[:hostname]
    rewrite
  end
  
  def rewrite
    if ["text/html"].any? {|type| @raw_headers["Content-Type"].downcase.start_with?(type)}
      @noko = Nokogiri::HTML(@raw_body)

      rewrite_links
      rewrite_forms
      rewrite_script_src
      rewrite_img
      
      @body = @noko.to_html(:indent_text => false).gsub("\n",'')
    else
      @body = @raw_body
    end
  end
  
  def rewrite_links
    ["a","link"].each do |css|
      @noko.css(css).each do |link|
        next unless link.attributes.keys.include? "href"
        link.attributes["href"].value = "#{new_location}#{hash_url( link.attributes["href"].value )}"
      end
    end
  end
  
  def rewrite_forms
    @noko.css("form").each do |form|
      form.attributes["action"].value = "#{new_location}#{hash_url(form.attributes["action"].value)}"
    end
  end
  
  def rewrite_script_src
    @noko.xpath("/*/head/script/@src").each do |script_tag|
      script_tag.value = "#{new_location}#{hash_url( script_tag.value )}"
    end
  end
  
  def rewrite_img
    @noko.css("img").each do |img|
      img.attributes["src"].value = "#{new_location}#{hash_url(img.attributes["src"].value)}"
    end
  end

end
